@@ -20,25 +20,95 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <ccode/valaccodenode.h>
-#include <ccode/valaccodewriter.h>
-#include <ccode/valaccodelinedirective.h>
+#include <glib.h>
+#include <glib-object.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_CCODE_NODE (vala_ccode_node_get_type ())
+#define VALA_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNode))
+#define VALA_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))
+#define VALA_IS_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_NODE))
+#define VALA_IS_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_NODE))
+#define VALA_CCODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))
 
+typedef struct _ValaCCodeNode ValaCCodeNode;
+typedef struct _ValaCCodeNodeClass ValaCCodeNodeClass;
+typedef struct _ValaCCodeNodePrivate ValaCCodeNodePrivate;
+
+#define VALA_TYPE_CCODE_WRITER (vala_ccode_writer_get_type ())
+#define VALA_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriter))
+#define VALA_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))
+#define VALA_IS_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_WRITER))
+#define VALA_IS_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_WRITER))
+#define VALA_CCODE_WRITER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))
+
+typedef struct _ValaCCodeWriter ValaCCodeWriter;
+typedef struct _ValaCCodeWriterClass ValaCCodeWriterClass;
+
+#define VALA_TYPE_CCODE_LINE_DIRECTIVE (vala_ccode_line_directive_get_type ())
+#define VALA_CCODE_LINE_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirective))
+#define VALA_CCODE_LINE_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirectiveClass))
+#define VALA_IS_CCODE_LINE_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE))
+#define VALA_IS_CCODE_LINE_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_LINE_DIRECTIVE))
+#define VALA_CCODE_LINE_DIRECTIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirectiveClass))
+
+typedef struct _ValaCCodeLineDirective ValaCCodeLineDirective;
+typedef struct _ValaCCodeLineDirectiveClass ValaCCodeLineDirectiveClass;
+typedef struct _ValaParamSpecCCodeNode ValaParamSpecCCodeNode;
+
+/**
+ * Represents a node in the C code tree.
+ */
+struct _ValaCCodeNode {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaCCodeNodePrivate * priv;
+};
+
+struct _ValaCCodeNodeClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaCCodeNode *self);
+	void (*write) (ValaCCodeNode* self, ValaCCodeWriter* writer);
+	void (*write_declaration) (ValaCCodeNode* self, ValaCCodeWriter* writer);
+	void (*write_combined) (ValaCCodeNode* self, ValaCCodeWriter* writer);
+};
 
 struct _ValaCCodeNodePrivate {
 	ValaCCodeLineDirective* _line;
 };
 
+struct _ValaParamSpecCCodeNode {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_ccode_node_ref (gpointer instance);
+void vala_ccode_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_ccode_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_ccode_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_ccode_node (const GValue* value);
+GType vala_ccode_node_get_type (void);
+gpointer vala_ccode_writer_ref (gpointer instance);
+void vala_ccode_writer_unref (gpointer instance);
+GParamSpec* vala_param_spec_ccode_writer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_ccode_writer (GValue* value, gpointer v_object);
+gpointer vala_value_get_ccode_writer (const GValue* value);
+GType vala_ccode_writer_get_type (void);
+GType vala_ccode_line_directive_get_type (void);
 #define VALA_CCODE_NODE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_NODE, ValaCCodeNodePrivate))
 enum  {
 	VALA_CCODE_NODE_DUMMY_PROPERTY
 };
+void vala_ccode_node_write (ValaCCodeNode* self, ValaCCodeWriter* writer);
 static void vala_ccode_node_real_write (ValaCCodeNode* self, ValaCCodeWriter* writer);
+void vala_ccode_node_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer);
 static void vala_ccode_node_real_write_declaration (ValaCCodeNode* self, ValaCCodeWriter* writer);
+void vala_ccode_node_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer);
 static void vala_ccode_node_real_write_combined (ValaCCodeNode* self, ValaCCodeWriter* writer);
+ValaCCodeLineDirective* vala_ccode_node_get_line (ValaCCodeNode* self);
+void vala_ccode_node_set_line (ValaCCodeNode* self, ValaCCodeLineDirective* value);
 static gpointer vala_ccode_node_parent_class = NULL;
 static void vala_ccode_node_finalize (ValaCCodeNode* obj);
 
