@@ -20,14 +20,60 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <ccode/valaccodewriter.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
 #include <stdio.h>
 #include <glib/gstdio.h>
-#include <ccode/valaccodelinedirective.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_CCODE_WRITER (vala_ccode_writer_get_type ())
+#define VALA_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriter))
+#define VALA_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))
+#define VALA_IS_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_WRITER))
+#define VALA_IS_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_WRITER))
+#define VALA_CCODE_WRITER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))
 
+typedef struct _ValaCCodeWriter ValaCCodeWriter;
+typedef struct _ValaCCodeWriterClass ValaCCodeWriterClass;
+typedef struct _ValaCCodeWriterPrivate ValaCCodeWriterPrivate;
+
+#define VALA_TYPE_CCODE_NODE (vala_ccode_node_get_type ())
+#define VALA_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNode))
+#define VALA_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))
+#define VALA_IS_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_NODE))
+#define VALA_IS_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_NODE))
+#define VALA_CCODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))
+
+typedef struct _ValaCCodeNode ValaCCodeNode;
+typedef struct _ValaCCodeNodeClass ValaCCodeNodeClass;
+
+#define VALA_TYPE_CCODE_LINE_DIRECTIVE (vala_ccode_line_directive_get_type ())
+#define VALA_CCODE_LINE_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirective))
+#define VALA_CCODE_LINE_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirectiveClass))
+#define VALA_IS_CCODE_LINE_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE))
+#define VALA_IS_CCODE_LINE_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_LINE_DIRECTIVE))
+#define VALA_CCODE_LINE_DIRECTIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_LINE_DIRECTIVE, ValaCCodeLineDirectiveClass))
+
+typedef struct _ValaCCodeLineDirective ValaCCodeLineDirective;
+typedef struct _ValaCCodeLineDirectiveClass ValaCCodeLineDirectiveClass;
+typedef struct _ValaParamSpecCCodeWriter ValaParamSpecCCodeWriter;
+
+/**
+ * Represents a writer to write C source files.
+ */
+struct _ValaCCodeWriter {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaCCodeWriterPrivate * priv;
+};
+
+struct _ValaCCodeWriterClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaCCodeWriter *self);
+};
 
 struct _ValaCCodeWriterPrivate {
 	char* _filename;
@@ -39,10 +85,46 @@ struct _ValaCCodeWriterPrivate {
 	gboolean _bol;
 };
 
+struct _ValaParamSpecCCodeWriter {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_ccode_writer_ref (gpointer instance);
+void vala_ccode_writer_unref (gpointer instance);
+GParamSpec* vala_param_spec_ccode_writer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_ccode_writer (GValue* value, gpointer v_object);
+gpointer vala_value_get_ccode_writer (const GValue* value);
+GType vala_ccode_writer_get_type (void);
 #define VALA_CCODE_WRITER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterPrivate))
 enum  {
 	VALA_CCODE_WRITER_DUMMY_PROPERTY
 };
+void vala_ccode_writer_set_filename (ValaCCodeWriter* self, const char* value);
+ValaCCodeWriter* vala_ccode_writer_new (const char* filename);
+ValaCCodeWriter* vala_ccode_writer_construct (GType object_type, const char* filename);
+ValaCCodeWriter* vala_ccode_writer_new (const char* filename);
+const char* vala_ccode_writer_get_filename (ValaCCodeWriter* self);
+gboolean vala_ccode_writer_open (ValaCCodeWriter* self);
+void vala_ccode_writer_close (ValaCCodeWriter* self);
+gboolean vala_ccode_writer_get_line_directives (ValaCCodeWriter* self);
+gpointer vala_ccode_node_ref (gpointer instance);
+void vala_ccode_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_ccode_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_ccode_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_ccode_node (const GValue* value);
+GType vala_ccode_node_get_type (void);
+void vala_ccode_node_write (ValaCCodeNode* self, ValaCCodeWriter* writer);
+gboolean vala_ccode_writer_get_bol (ValaCCodeWriter* self);
+GType vala_ccode_line_directive_get_type (void);
+void vala_ccode_writer_write_indent (ValaCCodeWriter* self, ValaCCodeLineDirective* line);
+void vala_ccode_writer_write_string (ValaCCodeWriter* self, const char* s);
+void vala_ccode_writer_write_newline (ValaCCodeWriter* self);
+void vala_ccode_writer_write_begin_block (ValaCCodeWriter* self);
+void vala_ccode_writer_write_end_block (ValaCCodeWriter* self);
+void vala_ccode_writer_write_comment (ValaCCodeWriter* self, const char* text);
+void vala_ccode_writer_set_line_directives (ValaCCodeWriter* self, gboolean value);
 static gpointer vala_ccode_writer_parent_class = NULL;
 static void vala_ccode_writer_finalize (ValaCCodeWriter* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
