@@ -22,10 +22,65 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/arraylist.h>
+#include <glib.h>
+#include <glib-object.h>
 #include <string.h>
 
 
+#define GEE_TYPE_COLLECTION_OBJECT (gee_collection_object_get_type ())
+#define GEE_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObject))
+#define GEE_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+#define GEE_IS_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_IS_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_COLLECTION_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+
+typedef struct _GeeCollectionObject GeeCollectionObject;
+typedef struct _GeeCollectionObjectClass GeeCollectionObjectClass;
+typedef struct _GeeCollectionObjectPrivate GeeCollectionObjectPrivate;
+
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_LIST (gee_list_get_type ())
+#define GEE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LIST, GeeList))
+#define GEE_IS_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LIST))
+#define GEE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_LIST, GeeListIface))
+
+typedef struct _GeeList GeeList;
+typedef struct _GeeListIface GeeListIface;
+
+#define GEE_TYPE_ARRAY_LIST (gee_array_list_get_type ())
+#define GEE_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ARRAY_LIST, GeeArrayList))
+#define GEE_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ARRAY_LIST, GeeArrayListClass))
+#define GEE_IS_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ARRAY_LIST))
+#define GEE_IS_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ARRAY_LIST))
+#define GEE_ARRAY_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ARRAY_LIST, GeeArrayListClass))
+
+typedef struct _GeeArrayList GeeArrayList;
+typedef struct _GeeArrayListClass GeeArrayListClass;
+typedef struct _GeeArrayListPrivate GeeArrayListPrivate;
+
 #define GEE_ARRAY_LIST_TYPE_ITERATOR (gee_array_list_iterator_get_type ())
 #define GEE_ARRAY_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIterator))
 #define GEE_ARRAY_LIST_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorClass))
@@ -37,6 +92,89 @@ typedef struct _GeeArrayListIterator GeeArrayListIterator;
 typedef struct _GeeArrayListIteratorClass GeeArrayListIteratorClass;
 typedef struct _GeeArrayListIteratorPrivate GeeArrayListIteratorPrivate;
 
+/**
+ * Base class for all collections.
+ */
+struct _GeeCollectionObject {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	GeeCollectionObjectPrivate * priv;
+};
+
+struct _GeeCollectionObjectClass {
+	GTypeClass parent_class;
+	void (*finalize) (GeeCollectionObject *self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GType (*get_element_type) (GeeIterable* self);
+	GeeIterator* (*iterator) (GeeIterable* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gint (*get_size) (GeeCollection* self);
+};
+
+/**
+ * Represents a collection of items in a well-defined order.
+ */
+struct _GeeListIface {
+	GTypeInterface parent_iface;
+	gpointer (*get) (GeeList* self, gint index);
+	void (*set) (GeeList* self, gint index, gconstpointer item);
+	gint (*index_of) (GeeList* self, gconstpointer item);
+	void (*insert) (GeeList* self, gint index, gconstpointer item);
+	void (*remove_at) (GeeList* self, gint index);
+};
+
+/**
+ * Arrays of arbitrary elements which grow automatically as elements are added.
+ */
+struct _GeeArrayList {
+	GeeCollectionObject parent_instance;
+	GeeArrayListPrivate * priv;
+};
+
+struct _GeeArrayListClass {
+	GeeCollectionObjectClass parent_class;
+};
+
+struct _GeeArrayListPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
+	gpointer* _items;
+	gint _items_length1;
+	gint _items_size;
+	gint _size;
+	GEqualFunc _equal_func;
+	gint _stamp;
+};
+
 struct _GeeArrayListIterator {
 	GeeCollectionObject parent_instance;
 	GeeArrayListIteratorPrivate * priv;
@@ -47,59 +185,66 @@ struct _GeeArrayListIteratorClass {
 	GeeCollectionObjectClass parent_class;
 };
 
-
-
-struct _GeeArrayListPrivate {
-	gpointer* _items;
-	gint _items_length1;
-	gint _items_size;
-	gint _size;
-	GEqualFunc _equal_func;
-	gint _stamp;
+struct _GeeArrayListIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
+	GeeArrayList* _list;
+	gint _index;
 };
 
+
+
+gpointer gee_collection_object_ref (gpointer instance);
+void gee_collection_object_unref (gpointer instance);
+GParamSpec* gee_param_spec_collection_object (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void gee_value_set_collection_object (GValue* value, gpointer v_object);
+gpointer gee_value_get_collection_object (const GValue* value);
+GType gee_collection_object_get_type (void);
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_list_get_type (void);
+GType gee_array_list_get_type (void);
 #define GEE_ARRAY_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_ARRAY_LIST, GeeArrayListPrivate))
 enum  {
 	GEE_ARRAY_LIST_DUMMY_PROPERTY
 };
+void gee_array_list_set_equal_func (GeeArrayList* self, GEqualFunc value);
+GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GEqualFunc equal_func);
+GeeArrayList* gee_array_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GEqualFunc equal_func);
+GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GEqualFunc equal_func);
 static GType gee_array_list_real_get_element_type (GeeIterable* base);
+static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
+static GeeArrayListIterator* gee_array_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
+static GType gee_array_list_iterator_get_type (void);
 static GeeIterator* gee_array_list_real_iterator (GeeIterable* base);
+gint gee_list_index_of (GeeList* self, gconstpointer item);
 static gboolean gee_array_list_real_contains (GeeCollection* base, gconstpointer item);
 static gint gee_array_list_real_index_of (GeeList* base, gconstpointer item);
 static gpointer gee_array_list_real_get (GeeList* base, gint index);
 static void gee_array_list_real_set (GeeList* base, gint index, gconstpointer item);
+static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count);
 static gboolean gee_array_list_real_add (GeeCollection* base, gconstpointer item);
+static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta);
 static void gee_array_list_real_insert (GeeList* base, gint index, gconstpointer item);
+void gee_list_remove_at (GeeList* self, gint index);
 static gboolean gee_array_list_real_remove (GeeCollection* base, gconstpointer item);
 static void gee_array_list_real_remove_at (GeeList* base, gint index);
 static void gee_array_list_real_clear (GeeCollection* base);
-static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta);
-static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count);
 static void gee_array_list_set_capacity (GeeArrayList* self, gint value);
-struct _GeeArrayListIteratorPrivate {
-	GeeArrayList* _list;
-	gint _index;
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
 #define GEE_ARRAY_LIST_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorPrivate))
 enum  {
 	GEE_ARRAY_LIST_ITERATOR_DUMMY_PROPERTY
 };
-static GeeArrayListIterator* gee_array_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
+static void gee_array_list_iterator_set_list (GeeArrayListIterator* self, GeeArrayList* value);
 static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
 static gboolean gee_array_list_iterator_real_next (GeeIterator* base);
+gpointer gee_list_get (GeeList* self, gint index);
 static gpointer gee_array_list_iterator_real_get (GeeIterator* base);
-static void gee_array_list_iterator_set_list (GeeArrayListIterator* self, GeeArrayList* value);
 static gpointer gee_array_list_iterator_parent_class = NULL;
 static GeeIteratorIface* gee_array_list_iterator_gee_iterator_parent_iface = NULL;
 static void gee_array_list_iterator_finalize (GeeCollectionObject* obj);
-static GType gee_array_list_iterator_get_type (void);
 static gpointer gee_array_list_parent_class = NULL;
 static GeeIterableIface* gee_array_list_gee_iterable_parent_iface = NULL;
 static GeeCollectionIface* gee_array_list_gee_collection_parent_iface = NULL;
