@@ -22,8 +22,71 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/hashmap.h>
+#include <glib.h>
+#include <glib-object.h>
 
+
+#define GEE_TYPE_COLLECTION_OBJECT (gee_collection_object_get_type ())
+#define GEE_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObject))
+#define GEE_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+#define GEE_IS_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_IS_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_COLLECTION_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+
+typedef struct _GeeCollectionObject GeeCollectionObject;
+typedef struct _GeeCollectionObjectClass GeeCollectionObjectClass;
+typedef struct _GeeCollectionObjectPrivate GeeCollectionObjectPrivate;
+
+#define GEE_TYPE_MAP (gee_map_get_type ())
+#define GEE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP, GeeMap))
+#define GEE_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP))
+#define GEE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP, GeeMapIface))
+
+typedef struct _GeeMap GeeMap;
+typedef struct _GeeMapIface GeeMapIface;
+
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_SET (gee_set_get_type ())
+#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
+#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
+#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))
+
+typedef struct _GeeSet GeeSet;
+typedef struct _GeeSetIface GeeSetIface;
+
+#define GEE_TYPE_HASH_MAP (gee_hash_map_get_type ())
+#define GEE_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_HASH_MAP, GeeHashMap))
+#define GEE_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_HASH_MAP, GeeHashMapClass))
+#define GEE_IS_HASH_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_HASH_MAP))
+#define GEE_IS_HASH_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_HASH_MAP))
+#define GEE_HASH_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_HASH_MAP, GeeHashMapClass))
+
+typedef struct _GeeHashMap GeeHashMap;
+typedef struct _GeeHashMapClass GeeHashMapClass;
+typedef struct _GeeHashMapPrivate GeeHashMapPrivate;
 typedef struct _GeeHashMapNode GeeHashMapNode;
 
 #define GEE_HASH_MAP_TYPE_KEY_SET (gee_hash_map_key_set_get_type ())
@@ -35,6 +98,16 @@ typedef struct _GeeHashMapNode GeeHashMapNode;
 
 typedef struct _GeeHashMapKeySet GeeHashMapKeySet;
 typedef struct _GeeHashMapKeySetClass GeeHashMapKeySetClass;
+
+#define GEE_HASH_MAP_TYPE_VALUE_COLLECTION (gee_hash_map_value_collection_get_type ())
+#define GEE_HASH_MAP_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection))
+#define GEE_HASH_MAP_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
+#define GEE_HASH_MAP_IS_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
+#define GEE_HASH_MAP_IS_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
+#define GEE_HASH_MAP_VALUE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
+
+typedef struct _GeeHashMapValueCollection GeeHashMapValueCollection;
+typedef struct _GeeHashMapValueCollectionClass GeeHashMapValueCollectionClass;
 typedef struct _GeeHashMapKeySetPrivate GeeHashMapKeySetPrivate;
 
 #define GEE_HASH_MAP_TYPE_KEY_ITERATOR (gee_hash_map_key_iterator_get_type ())
@@ -47,16 +120,6 @@ typedef struct _GeeHashMapKeySetPrivate GeeHashMapKeySetPrivate;
 typedef struct _GeeHashMapKeyIterator GeeHashMapKeyIterator;
 typedef struct _GeeHashMapKeyIteratorClass GeeHashMapKeyIteratorClass;
 typedef struct _GeeHashMapKeyIteratorPrivate GeeHashMapKeyIteratorPrivate;
-
-#define GEE_HASH_MAP_TYPE_VALUE_COLLECTION (gee_hash_map_value_collection_get_type ())
-#define GEE_HASH_MAP_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection))
-#define GEE_HASH_MAP_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
-#define GEE_HASH_MAP_IS_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
-#define GEE_HASH_MAP_IS_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
-#define GEE_HASH_MAP_VALUE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
-
-typedef struct _GeeHashMapValueCollection GeeHashMapValueCollection;
-typedef struct _GeeHashMapValueCollectionClass GeeHashMapValueCollectionClass;
 typedef struct _GeeHashMapValueCollectionPrivate GeeHashMapValueCollectionPrivate;
 
 #define GEE_HASH_MAP_TYPE_VALUE_ITERATOR (gee_hash_map_value_iterator_get_type ())
@@ -70,6 +133,105 @@ typedef struct _GeeHashMapValueIterator GeeHashMapValueIterator;
 typedef struct _GeeHashMapValueIteratorClass GeeHashMapValueIteratorClass;
 typedef struct _GeeHashMapValueIteratorPrivate GeeHashMapValueIteratorPrivate;
 
+/**
+ * Base class for all collections.
+ */
+struct _GeeCollectionObject {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	GeeCollectionObjectPrivate * priv;
+};
+
+struct _GeeCollectionObjectClass {
+	GTypeClass parent_class;
+	void (*finalize) (GeeCollectionObject *self);
+};
+
+/**
+ * A map is a generic collection of key/value pairs.
+ */
+struct _GeeMapIface {
+	GTypeInterface parent_iface;
+	GeeSet* (*get_keys) (GeeMap* self);
+	GeeCollection* (*get_values) (GeeMap* self);
+	gboolean (*contains) (GeeMap* self, gconstpointer key);
+	gpointer (*get) (GeeMap* self, gconstpointer key);
+	void (*set) (GeeMap* self, gconstpointer key, gconstpointer value);
+	gboolean (*remove) (GeeMap* self, gconstpointer key);
+	void (*clear) (GeeMap* self);
+	gint (*get_size) (GeeMap* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GType (*get_element_type) (GeeIterable* self);
+	GeeIterator* (*iterator) (GeeIterable* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gint (*get_size) (GeeCollection* self);
+};
+
+/**
+ * A set is a collection without duplicates.
+ */
+struct _GeeSetIface {
+	GTypeInterface parent_iface;
+};
+
+/**
+ * Hashtable implementation of the Map interface.
+ */
+struct _GeeHashMap {
+	GeeCollectionObject parent_instance;
+	GeeHashMapPrivate * priv;
+};
+
+struct _GeeHashMapClass {
+	GeeCollectionObjectClass parent_class;
+};
+
+struct _GeeHashMapPrivate {
+	GType k_type;
+	GBoxedCopyFunc k_dup_func;
+	GDestroyNotify k_destroy_func;
+	GType v_type;
+	GBoxedCopyFunc v_dup_func;
+	GDestroyNotify v_destroy_func;
+	gint _array_size;
+	gint _nnodes;
+	GeeHashMapNode** _nodes;
+	gint _nodes_length1;
+	gint _nodes_size;
+	gint _stamp;
+	GHashFunc _key_hash_func;
+	GEqualFunc _key_equal_func;
+	GEqualFunc _value_equal_func;
+};
+
 struct _GeeHashMapNode {
 	gpointer key;
 	gpointer value;
@@ -86,6 +248,16 @@ struct _GeeHashMapKeySetClass {
 	GeeCollectionObjectClass parent_class;
 };
 
+struct _GeeHashMapKeySetPrivate {
+	GType k_type;
+	GBoxedCopyFunc k_dup_func;
+	GDestroyNotify k_destroy_func;
+	GType v_type;
+	GBoxedCopyFunc v_dup_func;
+	GDestroyNotify v_destroy_func;
+	GeeHashMap* _map;
+};
+
 struct _GeeHashMapKeyIterator {
 	GeeCollectionObject parent_instance;
 	GeeHashMapKeyIteratorPrivate * priv;
@@ -95,6 +267,19 @@ struct _GeeHashMapKeyIteratorClass {
 	GeeCollectionObjectClass parent_class;
 };
 
+struct _GeeHashMapKeyIteratorPrivate {
+	GType k_type;
+	GBoxedCopyFunc k_dup_func;
+	GDestroyNotify k_destroy_func;
+	GType v_type;
+	GBoxedCopyFunc v_dup_func;
+	GDestroyNotify v_destroy_func;
+	GeeHashMap* _map;
+	gint _index;
+	GeeHashMapNode* _node;
+	gint _stamp;
+};
+
 struct _GeeHashMapValueCollection {
 	GeeCollectionObject parent_instance;
 	GeeHashMapValueCollectionPrivate * priv;
@@ -104,6 +289,16 @@ struct _GeeHashMapValueCollectionClass {
 	GeeCollectionObjectClass parent_class;
 };
 
+struct _GeeHashMapValueCollectionPrivate {
+	GType k_type;
+	GBoxedCopyFunc k_dup_func;
+	GDestroyNotify k_destroy_func;
+	GType v_type;
+	GBoxedCopyFunc v_dup_func;
+	GDestroyNotify v_destroy_func;
+	GeeHashMap* _map;
+};
+
 struct _GeeHashMapValueIterator {
 	GeeCollectionObject parent_instance;
 	GeeHashMapValueIteratorPrivate * priv;
@@ -113,152 +308,131 @@ struct _GeeHashMapValueIteratorClass {
 	GeeCollectionObjectClass parent_class;
 };
 
-
-
-struct _GeeHashMapPrivate {
-	gint _array_size;
-	gint _nnodes;
-	GeeHashMapNode** _nodes;
-	gint _nodes_length1;
-	gint _stamp;
-	GHashFunc _key_hash_func;
-	GEqualFunc _key_equal_func;
-	GEqualFunc _value_equal_func;
+struct _GeeHashMapValueIteratorPrivate {
 	GType k_type;
 	GBoxedCopyFunc k_dup_func;
 	GDestroyNotify k_destroy_func;
 	GType v_type;
 	GBoxedCopyFunc v_dup_func;
 	GDestroyNotify v_destroy_func;
+	GeeHashMap* _map;
+	gint _index;
+	GeeHashMapNode* _node;
+	gint _stamp;
 };
 
+
+
+gpointer gee_collection_object_ref (gpointer instance);
+void gee_collection_object_unref (gpointer instance);
+GParamSpec* gee_param_spec_collection_object (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void gee_value_set_collection_object (GValue* value, gpointer v_object);
+gpointer gee_value_get_collection_object (const GValue* value);
+GType gee_collection_object_get_type (void);
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_set_get_type (void);
+GType gee_map_get_type (void);
+GType gee_hash_map_get_type (void);
 #define GEE_HASH_MAP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_HASH_MAP, GeeHashMapPrivate))
 enum  {
 	GEE_HASH_MAP_DUMMY_PROPERTY
 };
 #define GEE_HASH_MAP_MIN_SIZE 11
 #define GEE_HASH_MAP_MAX_SIZE 13845163
+void gee_hash_map_set_key_hash_func (GeeHashMap* self, GHashFunc value);
+void gee_hash_map_set_key_equal_func (GeeHashMap* self, GEqualFunc value);
+void gee_hash_map_set_value_equal_func (GeeHashMap* self, GEqualFunc value);
+GeeHashMap* gee_hash_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GHashFunc key_hash_func, GEqualFunc key_equal_func, GEqualFunc value_equal_func);
+GeeHashMap* gee_hash_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GHashFunc key_hash_func, GEqualFunc key_equal_func, GEqualFunc value_equal_func);
+GeeHashMap* gee_hash_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GHashFunc key_hash_func, GEqualFunc key_equal_func, GEqualFunc value_equal_func);
+static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GeeHashMapKeySet* gee_hash_map_key_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GType gee_hash_map_key_set_get_type (void);
 static GeeSet* gee_hash_map_real_get_keys (GeeMap* base);
+static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GeeHashMapValueCollection* gee_hash_map_value_collection_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GType gee_hash_map_value_collection_get_type (void);
 static GeeCollection* gee_hash_map_real_get_values (GeeMap* base);
 static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self, gconstpointer key);
 static gboolean gee_hash_map_real_contains (GeeMap* base, gconstpointer key);
 static gpointer gee_hash_map_real_get (GeeMap* base, gconstpointer key);
+static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash);
+static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash);
+static void gee_hash_map_resize (GeeHashMap* self);
 static void gee_hash_map_real_set (GeeMap* base, gconstpointer key, gconstpointer value);
 static gboolean gee_hash_map_real_remove (GeeMap* base, gconstpointer key);
 static void gee_hash_map_real_clear (GeeMap* base);
-static void gee_hash_map_resize (GeeHashMap* self);
-static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash);
-static void gee_hash_map_node_instance_init (GeeHashMapNode * self);
+void gee_map_clear (GeeMap* self);
 static void gee_hash_map_node_free (GeeHashMapNode* self);
-struct _GeeHashMapKeySetPrivate {
-	GeeHashMap* _map;
-	GType k_type;
-	GBoxedCopyFunc k_dup_func;
-	GDestroyNotify k_destroy_func;
-	GType v_type;
-	GBoxedCopyFunc v_dup_func;
-	GDestroyNotify v_destroy_func;
-};
-
+static void gee_hash_map_node_instance_init (GeeHashMapNode * self);
 #define GEE_HASH_MAP_KEY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetPrivate))
 enum  {
 	GEE_HASH_MAP_KEY_SET_DUMMY_PROPERTY
 };
-static GeeHashMapKeySet* gee_hash_map_key_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static void gee_hash_map_key_set_set_map (GeeHashMapKeySet* self, GeeHashMap* value);
 static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
 static GType gee_hash_map_key_set_real_get_element_type (GeeIterable* base);
+static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GType gee_hash_map_key_iterator_get_type (void);
 static GeeIterator* gee_hash_map_key_set_real_iterator (GeeIterable* base);
 static gboolean gee_hash_map_key_set_real_add (GeeCollection* base, gconstpointer key);
 static void gee_hash_map_key_set_real_clear (GeeCollection* base);
 static gboolean gee_hash_map_key_set_real_remove (GeeCollection* base, gconstpointer key);
+gboolean gee_map_contains (GeeMap* self, gconstpointer key);
 static gboolean gee_hash_map_key_set_real_contains (GeeCollection* base, gconstpointer key);
-static void gee_hash_map_key_set_set_map (GeeHashMapKeySet* self, GeeHashMap* value);
+gint gee_map_get_size (GeeMap* self);
 static gpointer gee_hash_map_key_set_parent_class = NULL;
 static GeeIterableIface* gee_hash_map_key_set_gee_iterable_parent_iface = NULL;
 static GeeCollectionIface* gee_hash_map_key_set_gee_collection_parent_iface = NULL;
 static GeeSetIface* gee_hash_map_key_set_gee_set_parent_iface = NULL;
 static void gee_hash_map_key_set_finalize (GeeCollectionObject* obj);
-static GType gee_hash_map_key_set_get_type (void);
-struct _GeeHashMapKeyIteratorPrivate {
-	GeeHashMap* _map;
-	gint _index;
-	GeeHashMapNode* _node;
-	gint _stamp;
-	GType k_type;
-	GBoxedCopyFunc k_dup_func;
-	GDestroyNotify k_destroy_func;
-	GType v_type;
-	GBoxedCopyFunc v_dup_func;
-	GDestroyNotify v_destroy_func;
-};
-
 #define GEE_HASH_MAP_KEY_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorPrivate))
 enum  {
 	GEE_HASH_MAP_KEY_ITERATOR_DUMMY_PROPERTY
 };
-static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static void gee_hash_map_key_iterator_set_map (GeeHashMapKeyIterator* self, GeeHashMap* value);
 static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
 static gboolean gee_hash_map_key_iterator_real_next (GeeIterator* base);
 static gpointer gee_hash_map_key_iterator_real_get (GeeIterator* base);
-static void gee_hash_map_key_iterator_set_map (GeeHashMapKeyIterator* self, GeeHashMap* value);
 static gpointer gee_hash_map_key_iterator_parent_class = NULL;
 static GeeIteratorIface* gee_hash_map_key_iterator_gee_iterator_parent_iface = NULL;
 static void gee_hash_map_key_iterator_finalize (GeeCollectionObject* obj);
-static GType gee_hash_map_key_iterator_get_type (void);
-struct _GeeHashMapValueCollectionPrivate {
-	GeeHashMap* _map;
-	GType k_type;
-	GBoxedCopyFunc k_dup_func;
-	GDestroyNotify k_destroy_func;
-	GType v_type;
-	GBoxedCopyFunc v_dup_func;
-	GDestroyNotify v_destroy_func;
-};
-
 #define GEE_HASH_MAP_VALUE_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionPrivate))
 enum  {
 	GEE_HASH_MAP_VALUE_COLLECTION_DUMMY_PROPERTY
 };
-static GeeHashMapValueCollection* gee_hash_map_value_collection_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static void gee_hash_map_value_collection_set_map (GeeHashMapValueCollection* self, GeeHashMap* value);
 static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
 static GType gee_hash_map_value_collection_real_get_element_type (GeeIterable* base);
+static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static GType gee_hash_map_value_iterator_get_type (void);
 static GeeIterator* gee_hash_map_value_collection_real_iterator (GeeIterable* base);
 static gboolean gee_hash_map_value_collection_real_add (GeeCollection* base, gconstpointer value);
 static void gee_hash_map_value_collection_real_clear (GeeCollection* base);
 static gboolean gee_hash_map_value_collection_real_remove (GeeCollection* base, gconstpointer value);
+GeeIterator* gee_iterable_iterator (GeeIterable* self);
+gboolean gee_iterator_next (GeeIterator* self);
+gpointer gee_iterator_get (GeeIterator* self);
 static gboolean gee_hash_map_value_collection_real_contains (GeeCollection* base, gconstpointer value);
-static void gee_hash_map_value_collection_set_map (GeeHashMapValueCollection* self, GeeHashMap* value);
 static gpointer gee_hash_map_value_collection_parent_class = NULL;
 static GeeIterableIface* gee_hash_map_value_collection_gee_iterable_parent_iface = NULL;
 static GeeCollectionIface* gee_hash_map_value_collection_gee_collection_parent_iface = NULL;
 static void gee_hash_map_value_collection_finalize (GeeCollectionObject* obj);
-static GType gee_hash_map_value_collection_get_type (void);
-struct _GeeHashMapValueIteratorPrivate {
-	GeeHashMap* _map;
-	gint _index;
-	GeeHashMapNode* _node;
-	gint _stamp;
-	GType k_type;
-	GBoxedCopyFunc k_dup_func;
-	GDestroyNotify k_destroy_func;
-	GType v_type;
-	GBoxedCopyFunc v_dup_func;
-	GDestroyNotify v_destroy_func;
-};
-
 #define GEE_HASH_MAP_VALUE_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorPrivate))
 enum  {
 	GEE_HASH_MAP_VALUE_ITERATOR_DUMMY_PROPERTY
 };
-static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
+static void gee_hash_map_value_iterator_set_map (GeeHashMapValueIterator* self, GeeHashMap* value);
 static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
 static gboolean gee_hash_map_value_iterator_real_next (GeeIterator* base);
 static gpointer gee_hash_map_value_iterator_real_get (GeeIterator* base);
-static void gee_hash_map_value_iterator_set_map (GeeHashMapValueIterator* self, GeeHashMap* value);
 static gpointer gee_hash_map_value_iterator_parent_class = NULL;
 static GeeIteratorIface* gee_hash_map_value_iterator_gee_iterator_parent_iface = NULL;
 static void gee_hash_map_value_iterator_finalize (GeeCollectionObject* obj);
-static GType gee_hash_map_value_iterator_get_type (void);
 static gpointer gee_hash_map_parent_class = NULL;
 static GeeMapIface* gee_hash_map_gee_map_parent_iface = NULL;
 static void gee_hash_map_finalize (GeeCollectionObject* obj);
@@ -281,7 +455,7 @@ GeeHashMap* gee_hash_map_construct (GType object_type, GType k_type, GBoxedCopyF
 	gee_hash_map_set_value_equal_func (self, value_equal_func);
 	self->priv->_array_size = GEE_HASH_MAP_MIN_SIZE;
 	_tmp0 = NULL;
-	self->priv->_nodes = (_tmp0 = g_new0 (GeeHashMapNode*, self->priv->_array_size + 1), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL), self->priv->_nodes_length1 = self->priv->_array_size, _tmp0);
+	self->priv->_nodes = (_tmp0 = g_new0 (GeeHashMapNode*, self->priv->_array_size + 1), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL), self->priv->_nodes_length1 = self->priv->_array_size, self->priv->_nodes_size = self->priv->_nodes_length1, _tmp0);
 	return self;
 }
 
@@ -322,14 +496,14 @@ static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self, gconstpointe
 			} else {
 				_tmp1 = !self->priv->_key_equal_func ((*node)->key, key);
 			}
-			_tmp0 = (_tmp1);
+			_tmp0 = _tmp1;
 		} else {
 			_tmp0 = FALSE;
 		}
 		if (!_tmp0) {
 			break;
 		}
-		node = &((*node)->next);
+		node = &(*node)->next;
 	}
 	return node;
 }
@@ -340,7 +514,7 @@ static gboolean gee_hash_map_real_contains (GeeMap* base, gconstpointer key) {
 	GeeHashMapNode** node;
 	self = (GeeHashMap*) base;
 	node = gee_hash_map_lookup_node (self, key);
-	return ((*node) != NULL);
+	return (*node) != NULL;
 }
 
 
@@ -348,7 +522,7 @@ static gpointer gee_hash_map_real_get (GeeMap* base, gconstpointer key) {
 	GeeHashMap * self;
 	GeeHashMapNode* node;
 	self = (GeeHashMap*) base;
-	node = (*gee_hash_map_lookup_node (self, key));
+	node = *gee_hash_map_lookup_node (self, key);
 	if (node != NULL) {
 		gconstpointer _tmp0;
 		_tmp0 = NULL;
@@ -455,7 +629,7 @@ static void gee_hash_map_resize (GeeHashMap* self) {
 	} else {
 		_tmp1 = FALSE;
 	}
-	if ((_tmp1)) {
+	if (_tmp1) {
 		_tmp0 = TRUE;
 	} else {
 		gboolean _tmp2;
@@ -465,11 +639,12 @@ static void gee_hash_map_resize (GeeHashMap* self) {
 		} else {
 			_tmp2 = FALSE;
 		}
-		_tmp0 = (_tmp2);
+		_tmp0 = _tmp2;
 	}
 	if (_tmp0) {
 		gint new_array_size;
 		GeeHashMapNode** _tmp3;
+		gint new_nodes_size;
 		gint new_nodes_length1;
 		GeeHashMapNode** new_nodes;
 		GeeHashMapNode** _tmp15;
@@ -477,7 +652,7 @@ static void gee_hash_map_resize (GeeHashMap* self) {
 		new_array_size = (gint) g_spaced_primes_closest ((guint) self->priv->_nnodes);
 		new_array_size = CLAMP (new_array_size, GEE_HASH_MAP_MIN_SIZE, GEE_HASH_MAP_MAX_SIZE);
 		_tmp3 = NULL;
-		new_nodes = (_tmp3 = g_new0 (GeeHashMapNode*, new_array_size + 1), new_nodes_length1 = new_array_size, _tmp3);
+		new_nodes = (_tmp3 = g_new0 (GeeHashMapNode*, new_array_size + 1), new_nodes_length1 = new_array_size, new_nodes_size = new_nodes_length1, _tmp3);
 		{
 			gint i;
 			i = 0;
@@ -519,7 +694,7 @@ static void gee_hash_map_resize (GeeHashMap* self) {
 		}
 		_tmp15 = NULL;
 		_tmp14 = NULL;
-		self->priv->_nodes = (_tmp15 = (_tmp14 = new_nodes, new_nodes = NULL, _tmp14), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL), self->priv->_nodes_length1 = new_nodes_length1, _tmp15);
+		self->priv->_nodes = (_tmp15 = (_tmp14 = new_nodes, new_nodes = NULL, _tmp14), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL), self->priv->_nodes_length1 = new_nodes_length1, self->priv->_nodes_size = self->priv->_nodes_length1, _tmp15);
 		self->priv->_array_size = new_array_size;
 		new_nodes = (_vala_array_free (new_nodes, new_nodes_length1, (GDestroyNotify) gee_hash_map_node_free), NULL);
 	}
@@ -753,7 +928,7 @@ static gboolean gee_hash_map_key_iterator_real_next (GeeIterator* base) {
 		self->priv->_index++;
 		self->priv->_node = self->priv->_map->priv->_nodes[self->priv->_index];
 	}
-	return (self->priv->_node != NULL);
+	return self->priv->_node != NULL;
 }
 
 
@@ -1001,7 +1176,7 @@ static gboolean gee_hash_map_value_iterator_real_next (GeeIterator* base) {
 		self->priv->_index++;
 		self->priv->_node = self->priv->_map->priv->_nodes[self->priv->_index];
 	}
-	return (self->priv->_node != NULL);
+	return self->priv->_node != NULL;
 }
 
 
@@ -1119,14 +1294,10 @@ GType gee_hash_map_get_type (void) {
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
 	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) array)[i] != NULL)
-			destroy_func (((gpointer*) array)[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) array)[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
 	g_free (array);
