@@ -22,8 +22,63 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/hashset.h>
+#include <glib.h>
+#include <glib-object.h>
 
+
+#define GEE_TYPE_COLLECTION_OBJECT (gee_collection_object_get_type ())
+#define GEE_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObject))
+#define GEE_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+#define GEE_IS_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_IS_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_COLLECTION_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+
+typedef struct _GeeCollectionObject GeeCollectionObject;
+typedef struct _GeeCollectionObjectClass GeeCollectionObjectClass;
+typedef struct _GeeCollectionObjectPrivate GeeCollectionObjectPrivate;
+
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_SET (gee_set_get_type ())
+#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
+#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
+#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))
+
+typedef struct _GeeSet GeeSet;
+typedef struct _GeeSetIface GeeSetIface;
+
+#define GEE_TYPE_HASH_SET (gee_hash_set_get_type ())
+#define GEE_HASH_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_HASH_SET, GeeHashSet))
+#define GEE_HASH_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_HASH_SET, GeeHashSetClass))
+#define GEE_IS_HASH_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_HASH_SET))
+#define GEE_IS_HASH_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_HASH_SET))
+#define GEE_HASH_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_HASH_SET, GeeHashSetClass))
+
+typedef struct _GeeHashSet GeeHashSet;
+typedef struct _GeeHashSetClass GeeHashSetClass;
+typedef struct _GeeHashSetPrivate GeeHashSetPrivate;
 typedef struct _GeeHashSetNode GeeHashSetNode;
 
 #define GEE_HASH_SET_TYPE_ITERATOR (gee_hash_set_iterator_get_type ())
@@ -37,6 +92,86 @@ typedef struct _GeeHashSetIterator GeeHashSetIterator;
 typedef struct _GeeHashSetIteratorClass GeeHashSetIteratorClass;
 typedef struct _GeeHashSetIteratorPrivate GeeHashSetIteratorPrivate;
 
+/**
+ * Base class for all collections.
+ */
+struct _GeeCollectionObject {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	GeeCollectionObjectPrivate * priv;
+};
+
+struct _GeeCollectionObjectClass {
+	GTypeClass parent_class;
+	void (*finalize) (GeeCollectionObject *self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GType (*get_element_type) (GeeIterable* self);
+	GeeIterator* (*iterator) (GeeIterable* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gint (*get_size) (GeeCollection* self);
+};
+
+/**
+ * A set is a collection without duplicates.
+ */
+struct _GeeSetIface {
+	GTypeInterface parent_iface;
+};
+
+/**
+ * Hashtable implementation of the Set interface.
+ */
+struct _GeeHashSet {
+	GeeCollectionObject parent_instance;
+	GeeHashSetPrivate * priv;
+};
+
+struct _GeeHashSetClass {
+	GeeCollectionObjectClass parent_class;
+};
+
+struct _GeeHashSetPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
+	gint _array_size;
+	gint _nnodes;
+	GeeHashSetNode** _nodes;
+	gint _nodes_length1;
+	gint _nodes_size;
+	gint _stamp;
+	GHashFunc _hash_func;
+	GEqualFunc _equal_func;
+};
+
 struct _GeeHashSetNode {
 	gpointer key;
 	GeeHashSetNode* next;
@@ -52,61 +187,67 @@ struct _GeeHashSetIteratorClass {
 	GeeCollectionObjectClass parent_class;
 };
 
-
-
-struct _GeeHashSetPrivate {
-	gint _array_size;
-	gint _nnodes;
-	GeeHashSetNode** _nodes;
-	gint _nodes_length1;
-	gint _stamp;
-	GHashFunc _hash_func;
-	GEqualFunc _equal_func;
+struct _GeeHashSetIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
+	GeeHashSet* _set;
+	gint _index;
+	GeeHashSetNode* _node;
+	gint _stamp;
 };
 
+
+
+gpointer gee_collection_object_ref (gpointer instance);
+void gee_collection_object_unref (gpointer instance);
+GParamSpec* gee_param_spec_collection_object (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void gee_value_set_collection_object (GValue* value, gpointer v_object);
+gpointer gee_value_get_collection_object (const GValue* value);
+GType gee_collection_object_get_type (void);
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_set_get_type (void);
+GType gee_hash_set_get_type (void);
 #define GEE_HASH_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_HASH_SET, GeeHashSetPrivate))
 enum  {
 	GEE_HASH_SET_DUMMY_PROPERTY
 };
 #define GEE_HASH_SET_MIN_SIZE 11
 #define GEE_HASH_SET_MAX_SIZE 13845163
+void gee_hash_set_set_hash_func (GeeHashSet* self, GHashFunc value);
+void gee_hash_set_set_equal_func (GeeHashSet* self, GEqualFunc value);
+GeeHashSet* gee_hash_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GHashFunc hash_func, GEqualFunc equal_func);
+GeeHashSet* gee_hash_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GHashFunc hash_func, GEqualFunc equal_func);
+GeeHashSet* gee_hash_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GHashFunc hash_func, GEqualFunc equal_func);
 static GeeHashSetNode** gee_hash_set_lookup_node (GeeHashSet* self, gconstpointer key);
 static gboolean gee_hash_set_real_contains (GeeCollection* base, gconstpointer key);
 static GType gee_hash_set_real_get_element_type (GeeIterable* base);
+static GeeHashSetIterator* gee_hash_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeHashSet* set);
+static GeeHashSetIterator* gee_hash_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeHashSet* set);
+static GType gee_hash_set_iterator_get_type (void);
 static GeeIterator* gee_hash_set_real_iterator (GeeIterable* base);
+static GeeHashSetNode* gee_hash_set_node_new (gpointer k, guint hash);
+static GeeHashSetNode* gee_hash_set_node_new (gpointer k, guint hash);
+static void gee_hash_set_resize (GeeHashSet* self);
 static gboolean gee_hash_set_real_add (GeeCollection* base, gconstpointer key);
 static gboolean gee_hash_set_real_remove (GeeCollection* base, gconstpointer key);
 static void gee_hash_set_real_clear (GeeCollection* base);
-static void gee_hash_set_resize (GeeHashSet* self);
-static GeeHashSetNode* gee_hash_set_node_new (gpointer k, guint hash);
-static void gee_hash_set_node_instance_init (GeeHashSetNode * self);
+void gee_collection_clear (GeeCollection* self);
 static void gee_hash_set_node_free (GeeHashSetNode* self);
-struct _GeeHashSetIteratorPrivate {
-	GeeHashSet* _set;
-	gint _index;
-	GeeHashSetNode* _node;
-	gint _stamp;
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
+static void gee_hash_set_node_instance_init (GeeHashSetNode * self);
 #define GEE_HASH_SET_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_SET_TYPE_ITERATOR, GeeHashSetIteratorPrivate))
 enum  {
 	GEE_HASH_SET_ITERATOR_DUMMY_PROPERTY
 };
-static GeeHashSetIterator* gee_hash_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeHashSet* set);
+static void gee_hash_set_iterator_set_set (GeeHashSetIterator* self, GeeHashSet* value);
 static GeeHashSetIterator* gee_hash_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeHashSet* set);
 static gboolean gee_hash_set_iterator_real_next (GeeIterator* base);
 static gpointer gee_hash_set_iterator_real_get (GeeIterator* base);
-static void gee_hash_set_iterator_set_set (GeeHashSetIterator* self, GeeHashSet* value);
 static gpointer gee_hash_set_iterator_parent_class = NULL;
 static GeeIteratorIface* gee_hash_set_iterator_gee_iterator_parent_iface = NULL;
 static void gee_hash_set_iterator_finalize (GeeCollectionObject* obj);
-static GType gee_hash_set_iterator_get_type (void);
 static gpointer gee_hash_set_parent_class = NULL;
 static GeeIterableIface* gee_hash_set_gee_iterable_parent_iface = NULL;
 static GeeCollectionIface* gee_hash_set_gee_collection_parent_iface = NULL;
@@ -127,7 +268,7 @@ GeeHashSet* gee_hash_set_construct (GType object_type, GType g_type, GBoxedCopyF
 	gee_hash_set_set_equal_func (self, equal_func);
 	self->priv->_array_size = GEE_HASH_SET_MIN_SIZE;
 	_tmp0 = NULL;
-	self->priv->_nodes = (_tmp0 = g_new0 (GeeHashSetNode*, self->priv->_array_size + 1), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_set_node_free), NULL), self->priv->_nodes_length1 = self->priv->_array_size, _tmp0);
+	self->priv->_nodes = (_tmp0 = g_new0 (GeeHashSetNode*, self->priv->_array_size + 1), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_set_node_free), NULL), self->priv->_nodes_length1 = self->priv->_array_size, self->priv->_nodes_size = self->priv->_nodes_length1, _tmp0);
 	return self;
 }
 
@@ -154,14 +295,14 @@ static GeeHashSetNode** gee_hash_set_lookup_node (GeeHashSet* self, gconstpointe
 			} else {
 				_tmp1 = !self->priv->_equal_func ((*node)->key, key);
 			}
-			_tmp0 = (_tmp1);
+			_tmp0 = _tmp1;
 		} else {
 			_tmp0 = FALSE;
 		}
 		if (!_tmp0) {
 			break;
 		}
-		node = &((*node)->next);
+		node = &(*node)->next;
 	}
 	return node;
 }
@@ -172,7 +313,7 @@ static gboolean gee_hash_set_real_contains (GeeCollection* base, gconstpointer k
 	GeeHashSetNode** node;
 	self = (GeeHashSet*) base;
 	node = gee_hash_set_lookup_node (self, key);
-	return ((*node) != NULL);
+	return (*node) != NULL;
 }
 
 
@@ -275,7 +416,7 @@ static void gee_hash_set_resize (GeeHashSet* self) {
 	} else {
 		_tmp1 = FALSE;
 	}
-	if ((_tmp1)) {
+	if (_tmp1) {
 		_tmp0 = TRUE;
 	} else {
 		gboolean _tmp2;
@@ -285,11 +426,12 @@ static void gee_hash_set_resize (GeeHashSet* self) {
 		} else {
 			_tmp2 = FALSE;
 		}
-		_tmp0 = (_tmp2);
+		_tmp0 = _tmp2;
 	}
 	if (_tmp0) {
 		gint new_array_size;
 		GeeHashSetNode** _tmp3;
+		gint new_nodes_size;
 		gint new_nodes_length1;
 		GeeHashSetNode** new_nodes;
 		GeeHashSetNode** _tmp15;
@@ -297,7 +439,7 @@ static void gee_hash_set_resize (GeeHashSet* self) {
 		new_array_size = (gint) g_spaced_primes_closest ((guint) self->priv->_nnodes);
 		new_array_size = CLAMP (new_array_size, GEE_HASH_SET_MIN_SIZE, GEE_HASH_SET_MAX_SIZE);
 		_tmp3 = NULL;
-		new_nodes = (_tmp3 = g_new0 (GeeHashSetNode*, new_array_size + 1), new_nodes_length1 = new_array_size, _tmp3);
+		new_nodes = (_tmp3 = g_new0 (GeeHashSetNode*, new_array_size + 1), new_nodes_length1 = new_array_size, new_nodes_size = new_nodes_length1, _tmp3);
 		{
 			gint i;
 			i = 0;
@@ -339,7 +481,7 @@ static void gee_hash_set_resize (GeeHashSet* self) {
 		}
 		_tmp15 = NULL;
 		_tmp14 = NULL;
-		self->priv->_nodes = (_tmp15 = (_tmp14 = new_nodes, new_nodes = NULL, _tmp14), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_set_node_free), NULL), self->priv->_nodes_length1 = new_nodes_length1, _tmp15);
+		self->priv->_nodes = (_tmp15 = (_tmp14 = new_nodes, new_nodes = NULL, _tmp14), self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, (GDestroyNotify) gee_hash_set_node_free), NULL), self->priv->_nodes_length1 = new_nodes_length1, self->priv->_nodes_size = self->priv->_nodes_length1, _tmp15);
 		self->priv->_array_size = new_array_size;
 		new_nodes = (_vala_array_free (new_nodes, new_nodes_length1, (GDestroyNotify) gee_hash_set_node_free), NULL);
 	}
@@ -424,7 +566,7 @@ static gboolean gee_hash_set_iterator_real_next (GeeIterator* base) {
 		self->priv->_index++;
 		self->priv->_node = self->priv->_set->priv->_nodes[self->priv->_index];
 	}
-	return (self->priv->_node != NULL);
+	return self->priv->_node != NULL;
 }
 
 
@@ -556,14 +698,10 @@ GType gee_hash_set_get_type (void) {
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
 	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) array)[i] != NULL)
-			destroy_func (((gpointer*) array)[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) array)[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
 	g_free (array);
