@@ -20,9 +20,56 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/readonlycollection.h>
+#include <glib.h>
+#include <glib-object.h>
 
 
+#define GEE_TYPE_COLLECTION_OBJECT (gee_collection_object_get_type ())
+#define GEE_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObject))
+#define GEE_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+#define GEE_IS_COLLECTION_OBJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_IS_COLLECTION_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_COLLECTION_OBJECT))
+#define GEE_COLLECTION_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_COLLECTION_OBJECT, GeeCollectionObjectClass))
+
+typedef struct _GeeCollectionObject GeeCollectionObject;
+typedef struct _GeeCollectionObjectClass GeeCollectionObjectClass;
+typedef struct _GeeCollectionObjectPrivate GeeCollectionObjectPrivate;
+
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
+#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
+#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
+#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
+#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
+#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
+
+typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
+typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
+typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;
+
 #define GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR (gee_read_only_collection_iterator_get_type ())
 #define GEE_READ_ONLY_COLLECTION_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator))
 #define GEE_READ_ONLY_COLLECTION_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))
@@ -34,6 +81,72 @@ typedef struct _GeeReadOnlyCollectionIterator GeeReadOnlyCollectionIterator;
 typedef struct _GeeReadOnlyCollectionIteratorClass GeeReadOnlyCollectionIteratorClass;
 typedef struct _GeeReadOnlyCollectionIteratorPrivate GeeReadOnlyCollectionIteratorPrivate;
 
+/**
+ * Base class for all collections.
+ */
+struct _GeeCollectionObject {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	GeeCollectionObjectPrivate * priv;
+};
+
+struct _GeeCollectionObjectClass {
+	GTypeClass parent_class;
+	void (*finalize) (GeeCollectionObject *self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GType (*get_element_type) (GeeIterable* self);
+	GeeIterator* (*iterator) (GeeIterable* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gint (*get_size) (GeeCollection* self);
+};
+
+/**
+ * Represents a read-only collection of items.
+ */
+struct _GeeReadOnlyCollection {
+	GeeCollectionObject parent_instance;
+	GeeReadOnlyCollectionPrivate * priv;
+};
+
+struct _GeeReadOnlyCollectionClass {
+	GeeCollectionObjectClass parent_class;
+};
+
+struct _GeeReadOnlyCollectionPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
+	GeeCollection* _collection;
+};
+
 struct _GeeReadOnlyCollectionIterator {
 	GeeCollectionObject parent_instance;
 	GeeReadOnlyCollectionIteratorPrivate * priv;
@@ -43,42 +156,53 @@ struct _GeeReadOnlyCollectionIteratorClass {
 	GeeCollectionObjectClass parent_class;
 };
 
-
-
-struct _GeeReadOnlyCollectionPrivate {
-	GeeCollection* _collection;
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
-#define GEE_READ_ONLY_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionPrivate))
-enum  {
-	GEE_READ_ONLY_COLLECTION_DUMMY_PROPERTY
-};
-static GType gee_read_only_collection_real_get_element_type (GeeIterable* base);
-static GeeIterator* gee_read_only_collection_real_iterator (GeeIterable* base);
-static gboolean gee_read_only_collection_real_contains (GeeCollection* base, gconstpointer item);
-static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpointer item);
-static gboolean gee_read_only_collection_real_remove (GeeCollection* base, gconstpointer item);
-static void gee_read_only_collection_real_clear (GeeCollection* base);
 struct _GeeReadOnlyCollectionIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
 };
 
+
+
+gpointer gee_collection_object_ref (gpointer instance);
+void gee_collection_object_unref (gpointer instance);
+GParamSpec* gee_param_spec_collection_object (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void gee_value_set_collection_object (GValue* value, gpointer v_object);
+gpointer gee_value_get_collection_object (const GValue* value);
+GType gee_collection_object_get_type (void);
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_read_only_collection_get_type (void);
+#define GEE_READ_ONLY_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionPrivate))
+enum  {
+	GEE_READ_ONLY_COLLECTION_DUMMY_PROPERTY
+};
+void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value);
+GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
+GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
+GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
+static GType gee_read_only_collection_real_get_element_type (GeeIterable* base);
+static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GType gee_read_only_collection_iterator_get_type (void);
+GeeIterator* gee_iterable_iterator (GeeIterable* self);
+static GeeIterator* gee_read_only_collection_real_iterator (GeeIterable* base);
+gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
+static gboolean gee_read_only_collection_real_contains (GeeCollection* base, gconstpointer item);
+static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpointer item);
+static gboolean gee_read_only_collection_real_remove (GeeCollection* base, gconstpointer item);
+static void gee_read_only_collection_real_clear (GeeCollection* base);
+gint gee_collection_get_size (GeeCollection* self);
 #define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorPrivate))
 enum  {
 	GEE_READ_ONLY_COLLECTION_ITERATOR_DUMMY_PROPERTY
 };
 static gboolean gee_read_only_collection_iterator_real_next (GeeIterator* base);
 static gpointer gee_read_only_collection_iterator_real_get (GeeIterator* base);
-static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static gpointer gee_read_only_collection_iterator_parent_class = NULL;
 static GeeIteratorIface* gee_read_only_collection_iterator_gee_iterator_parent_iface = NULL;
-static GType gee_read_only_collection_iterator_get_type (void);
 static gpointer gee_read_only_collection_parent_class = NULL;
 static GeeIterableIface* gee_read_only_collection_gee_iterable_parent_iface = NULL;
 static GeeCollectionIface* gee_read_only_collection_gee_collection_parent_iface = NULL;
