@@ -1,6 +1,6 @@
 /* valaccodecompiler.vala
  *
- * Copyright (C) 2007-2008  Jürg Billeter
+ * Copyright (C) 2007-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,23 +20,62 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gobject/valaccodecompiler.h>
-#include <vala/valareport.h>
-#include <vala/valasourcereference.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <gee/list.h>
-#include <vala/valasourcefile.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
+#include <vala.h>
+#include <gee.h>
 #include <glib/gstdio.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_CCODE_COMPILER (vala_ccode_compiler_get_type ())
+#define VALA_CCODE_COMPILER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_COMPILER, ValaCCodeCompiler))
+#define VALA_CCODE_COMPILER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_COMPILER, ValaCCodeCompilerClass))
+#define VALA_IS_CCODE_COMPILER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_COMPILER))
+#define VALA_IS_CCODE_COMPILER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_COMPILER))
+#define VALA_CCODE_COMPILER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_COMPILER, ValaCCodeCompilerClass))
+
+typedef struct _ValaCCodeCompiler ValaCCodeCompiler;
+typedef struct _ValaCCodeCompilerClass ValaCCodeCompilerClass;
+typedef struct _ValaCCodeCompilerPrivate ValaCCodeCompilerPrivate;
+typedef struct _ValaParamSpecCCodeCompiler ValaParamSpecCCodeCompiler;
+
+/**
+ * Interface to the C compiler.
+ */
+struct _ValaCCodeCompiler {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaCCodeCompilerPrivate * priv;
+};
+
+struct _ValaCCodeCompilerClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaCCodeCompiler *self);
+};
+
+struct _ValaParamSpecCCodeCompiler {
+	GParamSpec parent_instance;
+};
 
 
+
+gpointer vala_ccode_compiler_ref (gpointer instance);
+void vala_ccode_compiler_unref (gpointer instance);
+GParamSpec* vala_param_spec_ccode_compiler (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_ccode_compiler (GValue* value, gpointer v_object);
+gpointer vala_value_get_ccode_compiler (const GValue* value);
+GType vala_ccode_compiler_get_type (void);
 enum  {
 	VALA_CCODE_COMPILER_DUMMY_PROPERTY
 };
+ValaCCodeCompiler* vala_ccode_compiler_new (void);
+ValaCCodeCompiler* vala_ccode_compiler_construct (GType object_type);
+ValaCCodeCompiler* vala_ccode_compiler_new (void);
 static gboolean vala_ccode_compiler_package_exists (const char* package_name);
+void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* context, const char* cc_command, char** cc_options, int cc_options_length1);
 static gpointer vala_ccode_compiler_parent_class = NULL;
 static void vala_ccode_compiler_finalize (ValaCCodeCompiler* obj);
 static int _vala_strcmp0 (const char * str1, const char * str2);
@@ -70,10 +109,9 @@ static gboolean vala_ccode_compiler_package_exists (const char* package_name) {
 			if (inner_error->domain == G_SPAWN_ERROR) {
 				goto __catch0_g_spawn_error;
 			}
-			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-			g_clear_error (&inner_error);
+			goto __finally0;
 		}
-		return (_tmp0 = (0 == exit_status), pc = (g_free (pc), NULL), _tmp0);
+		return (_tmp0 = 0 == exit_status, pc = (g_free (pc), NULL), _tmp0);
 	}
 	goto __finally0;
 	__catch0_g_spawn_error:
@@ -88,7 +126,12 @@ static gboolean vala_ccode_compiler_package_exists (const char* package_name) {
 		}
 	}
 	__finally0:
-	;
+	if (inner_error != NULL) {
+		pc = (g_free (pc), NULL);
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+		g_clear_error (&inner_error);
+		return FALSE;
+	}
 	pc = (g_free (pc), NULL);
 }
 
@@ -99,7 +142,7 @@ static gboolean vala_ccode_compiler_package_exists (const char* package_name) {
  *
  * @param context a code context
  */
-void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* context, const char* cc_command, char** cc_options) {
+void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* context, const char* cc_command, char** cc_options, int cc_options_length1) {
 	GError * inner_error;
 	char* pc;
 	char* _tmp1;
@@ -107,11 +150,10 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 	gint exit_status;
 	const char* _tmp11;
 	char* cmdline;
-	gboolean _tmp21;
 	GeeList* source_files;
 	GeeList* c_source_files;
-	char* _tmp33;
-	char* _tmp32;
+	char* _tmp29;
+	char* _tmp28;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (context != NULL);
 	inner_error = NULL;
@@ -119,14 +161,14 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 	if (!vala_code_context_get_compile_only (context)) {
 		char* _tmp0;
 		_tmp0 = NULL;
-		pc = (_tmp0 = g_strconcat (pc, (" --libs"), NULL), pc = (g_free (pc), NULL), _tmp0);
+		pc = (_tmp0 = g_strconcat (pc, " --libs", NULL), pc = (g_free (pc), NULL), _tmp0);
 	}
 	_tmp1 = NULL;
-	pc = (_tmp1 = g_strconcat (pc, (" gobject-2.0"), NULL), pc = (g_free (pc), NULL), _tmp1);
+	pc = (_tmp1 = g_strconcat (pc, " gobject-2.0", NULL), pc = (g_free (pc), NULL), _tmp1);
 	if (vala_code_context_get_thread (context)) {
 		char* _tmp2;
 		_tmp2 = NULL;
-		pc = (_tmp2 = g_strconcat (pc, (" gthread-2.0"), NULL), pc = (g_free (pc), NULL), _tmp2);
+		pc = (_tmp2 = g_strconcat (pc, " gthread-2.0", NULL), pc = (g_free (pc), NULL), _tmp2);
 	}
 	{
 		GeeList* _tmp3;
@@ -143,7 +185,7 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 				char* _tmp5;
 				_tmp6 = NULL;
 				_tmp5 = NULL;
-				pc = (_tmp6 = g_strconcat (pc, _tmp5 = (g_strconcat (" ", pkg, NULL)), NULL), pc = (g_free (pc), NULL), _tmp6);
+				pc = (_tmp6 = g_strconcat (pc, _tmp5 = g_strconcat (" ", pkg, NULL), NULL), pc = (g_free (pc), NULL), _tmp6);
 				_tmp5 = (g_free (_tmp5), NULL);
 			}
 			pkg = (g_free (pkg), NULL);
@@ -165,8 +207,7 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			if (inner_error->domain == G_SPAWN_ERROR) {
 				goto __catch1_g_spawn_error;
 			}
-			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-			g_clear_error (&inner_error);
+			goto __finally1;
 		}
 		if (exit_status != 0) {
 			char* _tmp10;
@@ -193,7 +234,13 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 		}
 	}
 	__finally1:
-	;
+	if (inner_error != NULL) {
+		pc = (g_free (pc), NULL);
+		pkgflags = (g_free (pkgflags), NULL);
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+		g_clear_error (&inner_error);
+		return;
+	}
 	/* TODO compile the C code files in parallel*/
 	if (cc_command == NULL) {
 		cc_command = "cc";
@@ -203,12 +250,12 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 	if (vala_code_context_get_debug (context)) {
 		char* _tmp12;
 		_tmp12 = NULL;
-		cmdline = (_tmp12 = g_strconcat (cmdline, (" -g"), NULL), cmdline = (g_free (cmdline), NULL), _tmp12);
+		cmdline = (_tmp12 = g_strconcat (cmdline, " -g", NULL), cmdline = (g_free (cmdline), NULL), _tmp12);
 	}
 	if (vala_code_context_get_compile_only (context)) {
 		char* _tmp13;
 		_tmp13 = NULL;
-		cmdline = (_tmp13 = g_strconcat (cmdline, (" -c"), NULL), cmdline = (g_free (cmdline), NULL), _tmp13);
+		cmdline = (_tmp13 = g_strconcat (cmdline, " -c", NULL), cmdline = (g_free (cmdline), NULL), _tmp13);
 	} else {
 		if (vala_code_context_get_output (context) != NULL) {
 			const char* _tmp14;
@@ -240,30 +287,12 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			_tmp20 = NULL;
 			_tmp19 = NULL;
 			_tmp18 = NULL;
-			cmdline = (_tmp20 = g_strconcat (cmdline, _tmp19 = (g_strconcat (" -o ", _tmp18 = g_shell_quote (output), NULL)), NULL), cmdline = (g_free (cmdline), NULL), _tmp20);
+			cmdline = (_tmp20 = g_strconcat (cmdline, _tmp19 = g_strconcat (" -o ", _tmp18 = g_shell_quote (output), NULL), NULL), cmdline = (g_free (cmdline), NULL), _tmp20);
 			_tmp19 = (g_free (_tmp19), NULL);
 			_tmp18 = (g_free (_tmp18), NULL);
 			output = (g_free (output), NULL);
 		}
 	}
-	_tmp21 = FALSE;
-	if (vala_code_context_get_directory (context) != NULL) {
-		_tmp21 = _vala_strcmp0 (vala_code_context_get_directory (context), "") != 0;
-	} else {
-		_tmp21 = FALSE;
-	}
-	/* make sure include files can be found if -d is used */
-	if (_tmp21) {
-		char* _tmp24;
-		char* _tmp23;
-		char* _tmp22;
-		_tmp24 = NULL;
-		_tmp23 = NULL;
-		_tmp22 = NULL;
-		cmdline = (_tmp24 = g_strconcat (cmdline, _tmp23 = (g_strconcat (" -I", _tmp22 = g_shell_quote (vala_code_context_get_directory (context)), NULL)), NULL), cmdline = (g_free (cmdline), NULL), _tmp24);
-		_tmp23 = (g_free (_tmp23), NULL);
-		_tmp22 = (g_free (_tmp22), NULL);
-	}
 	/* we're only interested in non-pkg source files */
 	source_files = vala_code_context_get_source_files (context);
 	{
@@ -273,18 +302,18 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			ValaSourceFile* file;
 			file = (ValaSourceFile*) gee_iterator_get (_file_it);
 			if (!vala_source_file_get_external_package (file)) {
-				char* _tmp28;
-				char* _tmp27;
-				char* _tmp26;
-				char* _tmp25;
-				_tmp28 = NULL;
-				_tmp27 = NULL;
-				_tmp26 = NULL;
-				_tmp25 = NULL;
-				cmdline = (_tmp28 = g_strconcat (cmdline, _tmp27 = (g_strconcat (" ", _tmp26 = g_shell_quote (_tmp25 = vala_source_file_get_csource_filename (file)), NULL)), NULL), cmdline = (g_free (cmdline), NULL), _tmp28);
-				_tmp27 = (g_free (_tmp27), NULL);
-				_tmp26 = (g_free (_tmp26), NULL);
-				_tmp25 = (g_free (_tmp25), NULL);
+				char* _tmp24;
+				char* _tmp23;
+				char* _tmp22;
+				char* _tmp21;
+				_tmp24 = NULL;
+				_tmp23 = NULL;
+				_tmp22 = NULL;
+				_tmp21 = NULL;
+				cmdline = (_tmp24 = g_strconcat (cmdline, _tmp23 = g_strconcat (" ", _tmp22 = g_shell_quote (_tmp21 = vala_source_file_get_csource_filename (file)), NULL), NULL), cmdline = (g_free (cmdline), NULL), _tmp24);
+				_tmp23 = (g_free (_tmp23), NULL);
+				_tmp22 = (g_free (_tmp22), NULL);
+				_tmp21 = (g_free (_tmp21), NULL);
 			}
 			(file == NULL) ? NULL : (file = (vala_source_file_unref (file), NULL));
 		}
@@ -296,47 +325,47 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 		_file_it = gee_iterable_iterator ((GeeIterable*) c_source_files);
 		while (gee_iterator_next (_file_it)) {
 			char* file;
-			char* _tmp31;
-			char* _tmp30;
-			char* _tmp29;
+			char* _tmp27;
+			char* _tmp26;
+			char* _tmp25;
 			file = (char*) gee_iterator_get (_file_it);
-			_tmp31 = NULL;
-			_tmp30 = NULL;
-			_tmp29 = NULL;
-			cmdline = (_tmp31 = g_strconcat (cmdline, _tmp30 = (g_strconcat (" ", _tmp29 = g_shell_quote (file), NULL)), NULL), cmdline = (g_free (cmdline), NULL), _tmp31);
-			_tmp30 = (g_free (_tmp30), NULL);
-			_tmp29 = (g_free (_tmp29), NULL);
+			_tmp27 = NULL;
+			_tmp26 = NULL;
+			_tmp25 = NULL;
+			cmdline = (_tmp27 = g_strconcat (cmdline, _tmp26 = g_strconcat (" ", _tmp25 = g_shell_quote (file), NULL), NULL), cmdline = (g_free (cmdline), NULL), _tmp27);
+			_tmp26 = (g_free (_tmp26), NULL);
+			_tmp25 = (g_free (_tmp25), NULL);
 			file = (g_free (file), NULL);
 		}
 		(_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL));
 	}
 	/* add libraries after source files to fix linking
 	 with --as-needed and on Windows*/
-	_tmp33 = NULL;
-	_tmp32 = NULL;
-	cmdline = (_tmp33 = g_strconcat (cmdline, _tmp32 = (g_strconcat (" ", g_strstrip (pkgflags), NULL)), NULL), cmdline = (g_free (cmdline), NULL), _tmp33);
-	_tmp32 = (g_free (_tmp32), NULL);
+	_tmp29 = NULL;
+	_tmp28 = NULL;
+	cmdline = (_tmp29 = g_strconcat (cmdline, _tmp28 = g_strconcat (" ", g_strstrip (pkgflags), NULL), NULL), cmdline = (g_free (cmdline), NULL), _tmp29);
+	_tmp28 = (g_free (_tmp28), NULL);
 	{
 		char** cc_option_collection;
 		int cc_option_collection_length1;
-		char** cc_option_it;
+		int cc_option_it;
 		cc_option_collection = cc_options;
-		cc_option_collection_length1 = -1;
-		for (cc_option_it = cc_option_collection; *cc_option_it != NULL; cc_option_it = cc_option_it + 1) {
-			const char* _tmp37;
+		cc_option_collection_length1 = cc_options_length1;
+		for (cc_option_it = 0; cc_option_it < cc_options_length1; cc_option_it = cc_option_it + 1) {
+			const char* _tmp33;
 			char* cc_option;
-			_tmp37 = NULL;
-			cc_option = (_tmp37 = *cc_option_it, (_tmp37 == NULL) ? NULL : g_strdup (_tmp37));
+			_tmp33 = NULL;
+			cc_option = (_tmp33 = cc_option_collection[cc_option_it], (_tmp33 == NULL) ? NULL : g_strdup (_tmp33));
 			{
-				char* _tmp36;
-				char* _tmp35;
-				char* _tmp34;
-				_tmp36 = NULL;
-				_tmp35 = NULL;
-				_tmp34 = NULL;
-				cmdline = (_tmp36 = g_strconcat (cmdline, _tmp35 = (g_strconcat (" ", _tmp34 = g_shell_quote (cc_option), NULL)), NULL), cmdline = (g_free (cmdline), NULL), _tmp36);
-				_tmp35 = (g_free (_tmp35), NULL);
-				_tmp34 = (g_free (_tmp34), NULL);
+				char* _tmp32;
+				char* _tmp31;
+				char* _tmp30;
+				_tmp32 = NULL;
+				_tmp31 = NULL;
+				_tmp30 = NULL;
+				cmdline = (_tmp32 = g_strconcat (cmdline, _tmp31 = g_strconcat (" ", _tmp30 = g_shell_quote (cc_option), NULL), NULL), cmdline = (g_free (cmdline), NULL), _tmp32);
+				_tmp31 = (g_free (_tmp31), NULL);
+				_tmp30 = (g_free (_tmp30), NULL);
 				cc_option = (g_free (cc_option), NULL);
 			}
 		}
@@ -347,14 +376,13 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			if (inner_error->domain == G_SPAWN_ERROR) {
 				goto __catch2_g_spawn_error;
 			}
-			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-			g_clear_error (&inner_error);
+			goto __finally2;
 		}
 		if (exit_status != 0) {
-			char* _tmp38;
-			_tmp38 = NULL;
-			vala_report_error (NULL, _tmp38 = g_strdup_printf ("cc exited with status %d", exit_status));
-			_tmp38 = (g_free (_tmp38), NULL);
+			char* _tmp34;
+			_tmp34 = NULL;
+			vala_report_error (NULL, _tmp34 = g_strdup_printf ("cc exited with status %d", exit_status));
+			_tmp34 = (g_free (_tmp34), NULL);
 		}
 	}
 	goto __finally2;
@@ -369,7 +397,16 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 		}
 	}
 	__finally2:
-	;
+	if (inner_error != NULL) {
+		pc = (g_free (pc), NULL);
+		pkgflags = (g_free (pkgflags), NULL);
+		cmdline = (g_free (cmdline), NULL);
+		(source_files == NULL) ? NULL : (source_files = (gee_collection_object_unref (source_files), NULL));
+		(c_source_files == NULL) ? NULL : (c_source_files = (gee_collection_object_unref (c_source_files), NULL));
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+		g_clear_error (&inner_error);
+		return;
+	}
 	/* remove generated C source and header files */
 	{
 		GeeIterator* _file_it;
@@ -382,16 +419,10 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			file = (ValaSourceFile*) gee_iterator_get (_file_it);
 			if (!vala_source_file_get_external_package (file)) {
 				if (!vala_code_context_get_save_csources (context)) {
-					char* _tmp39;
-					_tmp39 = NULL;
-					g_unlink (_tmp39 = vala_source_file_get_csource_filename (file));
-					_tmp39 = (g_free (_tmp39), NULL);
-				}
-				if (!vala_code_context_get_save_cheaders (context)) {
-					char* _tmp40;
-					_tmp40 = NULL;
-					g_unlink (_tmp40 = vala_source_file_get_cheader_filename (file));
-					_tmp40 = (g_free (_tmp40), NULL);
+					char* _tmp35;
+					_tmp35 = NULL;
+					g_unlink (_tmp35 = vala_source_file_get_csource_filename (file));
+					_tmp35 = (g_free (_tmp35), NULL);
 				}
 			}
 			(file == NULL) ? NULL : (file = (vala_source_file_unref (file), NULL));
