@@ -21,73 +21,389 @@
  *	Raffaele Sandrini <raffaele@sandrini.ch>
  */
 
-#include <gobject/valaccodecontrolflowmodule.h>
-#include <vala/valacodenode.h>
-#include <vala/valacodevisitor.h>
-#include <ccode/valaccodeifstatement.h>
-#include <ccode/valaccodeexpression.h>
-#include <ccode/valaccodestatement.h>
-#include <vala/valaexpression.h>
-#include <vala/valastatement.h>
-#include <gee/list.h>
-#include <vala/valadatatype.h>
-#include <vala/valalocalvariable.h>
-#include <ccode/valaccodeidentifier.h>
-#include <vala/valasymbol.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <vala.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ccode/valaccodeassignment.h>
-#include <ccode/valaccodeconstant.h>
-#include <ccode/valaccodefragment.h>
-#include <ccode/valaccodebinaryexpression.h>
-#include <ccode/valaccodefunctioncall.h>
-#include <ccode/valaccodeconditionalexpression.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <ccode/valaccodedeclaration.h>
-#include <ccode/valaccodemodifiers.h>
-#include <ccode/valaccodevariabledeclarator.h>
-#include <ccode/valaccodedeclarator.h>
-#include <ccode/valaccodenode.h>
-#include <ccode/valaccodeexpressionstatement.h>
-#include <vala/valablock.h>
-#include <ccode/valaccodeblock.h>
-#include <ccode/valaccodedostatement.h>
-#include <ccode/valaccodeswitchstatement.h>
-#include <ccode/valaccodelabel.h>
-#include <ccode/valaccodecasestatement.h>
-#include <ccode/valaccodewhilestatement.h>
-#include <ccode/valaccodeforstatement.h>
-#include <vala/valamethod.h>
-#include <ccode/valaccodestruct.h>
-#include <vala/valaarraytype.h>
-#include <ccode/valaccodeelementaccess.h>
-#include <gee/collection.h>
-#include <vala/valaobjecttype.h>
-#include <vala/valaobjecttypesymbol.h>
-#include <ccode/valaccodememberaccess.h>
-#include <vala/valareport.h>
-#include <vala/valasourcereference.h>
-#include <vala/valamemberaccess.h>
-#include <ccode/valaccodebreakstatement.h>
-#include <ccode/valaccodecontinuestatement.h>
-#include <gobject/valaccodebasemodule.h>
+#include <valaccode.h>
+#include <gee.h>
 
 
+#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
+#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
+#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
+#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
+#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
+#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
+
+typedef struct _ValaCCodeModule ValaCCodeModule;
+typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
+typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;
+
+#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
+#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
+#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
+#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
+#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
+#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
+
+typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
+typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
+typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;
+
+#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
+#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
+#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
+#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
+#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
+#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
+
+typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
+typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;
+
+#define VALA_TYPE_CCODE_STRUCT_MODULE (vala_ccode_struct_module_get_type ())
+#define VALA_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModule))
+#define VALA_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
+#define VALA_IS_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_STRUCT_MODULE))
+#define VALA_IS_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_STRUCT_MODULE))
+#define VALA_CCODE_STRUCT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
+
+typedef struct _ValaCCodeStructModule ValaCCodeStructModule;
+typedef struct _ValaCCodeStructModuleClass ValaCCodeStructModuleClass;
+typedef struct _ValaCCodeStructModulePrivate ValaCCodeStructModulePrivate;
+
+#define VALA_TYPE_CCODE_METHOD_MODULE (vala_ccode_method_module_get_type ())
+#define VALA_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModule))
+#define VALA_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))
+#define VALA_IS_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_MODULE))
+#define VALA_IS_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_MODULE))
+#define VALA_CCODE_METHOD_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))
+
+typedef struct _ValaCCodeMethodModule ValaCCodeMethodModule;
+typedef struct _ValaCCodeMethodModuleClass ValaCCodeMethodModuleClass;
+typedef struct _ValaCCodeMethodModulePrivate ValaCCodeMethodModulePrivate;
+
+#define VALA_TYPE_CCODE_CONTROL_FLOW_MODULE (vala_ccode_control_flow_module_get_type ())
+#define VALA_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModule))
+#define VALA_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))
+#define VALA_IS_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
+#define VALA_IS_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
+#define VALA_CCODE_CONTROL_FLOW_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))
+
+typedef struct _ValaCCodeControlFlowModule ValaCCodeControlFlowModule;
+typedef struct _ValaCCodeControlFlowModuleClass ValaCCodeControlFlowModuleClass;
+typedef struct _ValaCCodeControlFlowModulePrivate ValaCCodeControlFlowModulePrivate;
+
+#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
+#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
+#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
+#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
+#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
+#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
+
+typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
+typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;
+
+/**
+ * Code visitor generating C Code.
+ */
+struct _ValaCCodeModule {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaCCodeModulePrivate * priv;
+};
+
+struct _ValaCCodeModuleClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaCCodeModule *self);
+	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
+	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
+	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
+	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
+	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
+	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
+	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
+	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
+	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
+	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
+	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
+	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
+	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
+	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
+	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
+	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
+	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
+	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
+	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
+	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
+	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
+	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
+	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
+	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
+	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
+	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
+	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
+	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
+	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
+	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
+	void (*visit_while_statement) (ValaCCodeModule* self, ValaWhileStatement* stmt);
+	void (*visit_do_statement) (ValaCCodeModule* self, ValaDoStatement* stmt);
+	void (*visit_for_statement) (ValaCCodeModule* self, ValaForStatement* stmt);
+	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
+	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
+	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
+	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
+	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
+	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
+	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
+	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
+	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
+	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
+	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
+	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
+	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
+	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
+	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
+	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
+	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
+	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
+	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
+	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
+	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
+	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
+	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
+	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
+	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
+	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
+	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
+	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
+	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
+	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
+	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
+	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
+	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
+	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
+	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
+	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
+	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
+	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
+	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
+	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
+	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
+	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
+	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
+	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
+	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
+	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
+	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
+	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
+	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
+	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
+	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
+	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
+	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
+	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
+	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
+};
+
+/**
+ * Code visitor generating C Code.
+ */
+struct _ValaCCodeBaseModule {
+	ValaCCodeModule parent_instance;
+	ValaCCodeBaseModulePrivate * priv;
+	ValaSymbol* root_symbol;
+	ValaSymbol* current_symbol;
+	ValaTypeSymbol* current_type_symbol;
+	ValaClass* current_class;
+	ValaMethod* current_method;
+	ValaDataType* current_return_type;
+	ValaTryStatement* current_try;
+	ValaPropertyAccessor* current_property_accessor;
+	ValaCCodeDeclarationSpace* header_declarations;
+	ValaCCodeDeclarationSpace* internal_header_declarations;
+	ValaCCodeDeclarationSpace* source_declarations;
+	ValaCCodeFragment* source_signal_marshaller_declaration;
+	ValaCCodeFragment* source_type_member_definition;
+	ValaCCodeFragment* class_init_fragment;
+	ValaCCodeFragment* base_init_fragment;
+	ValaCCodeFragment* class_finalize_fragment;
+	ValaCCodeFragment* base_finalize_fragment;
+	ValaCCodeFragment* instance_init_fragment;
+	ValaCCodeFragment* instance_finalize_fragment;
+	ValaCCodeFragment* source_signal_marshaller_definition;
+	ValaCCodeFragment* module_init_fragment;
+	ValaCCodeStruct* param_spec_struct;
+	ValaCCodeStruct* closure_struct;
+	ValaCCodeEnum* prop_enum;
+	ValaCCodeFunction* function;
+	ValaCCodeFragment* pre_statement_fragment;
+	GeeArrayList* temp_vars;
+	GeeArrayList* temp_ref_vars;
+	GeeSet* user_marshal_set;
+	GeeSet* predefined_marshal_set;
+	GeeSet* c_keywords;
+	gint next_temp_var_id;
+	gboolean in_creation_method;
+	gboolean in_constructor;
+	gboolean in_static_or_class_ctor;
+	gboolean current_method_inner_error;
+	gint next_coroutine_state;
+	ValaDataType* bool_type;
+	ValaDataType* char_type;
+	ValaDataType* uchar_type;
+	ValaDataType* unichar_type;
+	ValaDataType* short_type;
+	ValaDataType* ushort_type;
+	ValaDataType* int_type;
+	ValaDataType* uint_type;
+	ValaDataType* long_type;
+	ValaDataType* ulong_type;
+	ValaDataType* int8_type;
+	ValaDataType* uint8_type;
+	ValaDataType* int16_type;
+	ValaDataType* uint16_type;
+	ValaDataType* int32_type;
+	ValaDataType* uint32_type;
+	ValaDataType* int64_type;
+	ValaDataType* uint64_type;
+	ValaDataType* string_type;
+	ValaDataType* float_type;
+	ValaDataType* double_type;
+	ValaTypeSymbol* gtype_type;
+	ValaTypeSymbol* gobject_type;
+	ValaErrorType* gerror_type;
+	ValaClass* glist_type;
+	ValaClass* gslist_type;
+	ValaTypeSymbol* gstringbuilder_type;
+	ValaTypeSymbol* garray_type;
+	ValaTypeSymbol* gbytearray_type;
+	ValaTypeSymbol* gptrarray_type;
+	ValaDataType* gquark_type;
+	ValaStruct* gvalue_type;
+	ValaStruct* mutex_type;
+	ValaTypeSymbol* type_module_type;
+	ValaTypeSymbol* dbus_object_type;
+	gboolean in_plugin;
+	char* module_init_param_name;
+	gboolean string_h_needed;
+	gboolean gvaluecollector_h_needed;
+	gboolean gio_h_needed;
+	gboolean requires_array_free;
+	gboolean requires_array_move;
+	gboolean requires_array_length;
+	gboolean requires_strcmp0;
+	gboolean dbus_glib_h_needed;
+	gboolean dbus_glib_h_needed_in_header;
+	GeeSet* wrappers;
+	GeeMap* variable_name_map;
+};
+
+struct _ValaCCodeBaseModuleClass {
+	ValaCCodeModuleClass parent_class;
+	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
+	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
+	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
+	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
+	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
+	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
+	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
+	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference);
+	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
+	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
+	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
+	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
+	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
+	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
+	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
+	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
+};
+
+struct _ValaCCodeStructModule {
+	ValaCCodeBaseModule parent_instance;
+	ValaCCodeStructModulePrivate * priv;
+};
+
+struct _ValaCCodeStructModuleClass {
+	ValaCCodeBaseModuleClass parent_class;
+};
+
+/**
+ * The link between a method and generated code.
+ */
+struct _ValaCCodeMethodModule {
+	ValaCCodeStructModule parent_instance;
+	ValaCCodeMethodModulePrivate * priv;
+};
+
+struct _ValaCCodeMethodModuleClass {
+	ValaCCodeStructModuleClass parent_class;
+	void (*generate_method_result_declaration) (ValaCCodeMethodModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeFunction* cfunc, GeeMap* cparam_map, GeeMap* carg_map);
+	void (*generate_parameter) (ValaCCodeMethodModule* self, ValaFormalParameter* param, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, GeeMap* carg_map);
+};
+
+struct _ValaCCodeControlFlowModule {
+	ValaCCodeMethodModule parent_instance;
+	ValaCCodeControlFlowModulePrivate * priv;
+};
+
+struct _ValaCCodeControlFlowModuleClass {
+	ValaCCodeMethodModuleClass parent_class;
+};
+
 
 
+gpointer vala_ccode_module_ref (gpointer instance);
+void vala_ccode_module_unref (gpointer instance);
+GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_ccode_module (GValue* value, gpointer v_object);
+gpointer vala_value_get_ccode_module (const GValue* value);
+GType vala_ccode_module_get_type (void);
+GType vala_ccode_base_module_get_type (void);
+gpointer vala_ccode_declaration_space_ref (gpointer instance);
+void vala_ccode_declaration_space_unref (gpointer instance);
+GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
+gpointer vala_value_get_ccode_declaration_space (const GValue* value);
+GType vala_ccode_declaration_space_get_type (void);
+GType vala_ccode_struct_module_get_type (void);
+GType vala_ccode_method_module_get_type (void);
+GType vala_ccode_control_flow_module_get_type (void);
 enum  {
 	VALA_CCODE_CONTROL_FLOW_MODULE_DUMMY_PROPERTY
 };
+GType vala_ccode_generator_get_type (void);
+ValaCCodeMethodModule* vala_ccode_method_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
+ValaCCodeMethodModule* vala_ccode_method_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
+ValaCCodeControlFlowModule* vala_ccode_control_flow_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
+ValaCCodeControlFlowModule* vala_ccode_control_flow_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
+ValaCCodeControlFlowModule* vala_ccode_control_flow_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
+ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
+void vala_ccode_base_module_create_temp_decl (ValaCCodeBaseModule* self, ValaStatement* stmt, GeeList* temp_vars);
 static void vala_ccode_control_flow_module_real_visit_if_statement (ValaCCodeModule* base, ValaIfStatement* stmt);
+ValaLocalVariable* vala_ccode_base_module_get_temp_variable (ValaCCodeBaseModule* self, ValaDataType* type, gboolean value_owned, ValaCodeNode* node_reference);
+gboolean vala_ccode_base_module_is_constant_ccode_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr);
 static void vala_ccode_control_flow_module_visit_string_switch_statement (ValaCCodeControlFlowModule* self, ValaSwitchStatement* stmt);
 static void vala_ccode_control_flow_module_real_visit_switch_statement (ValaCCodeModule* base, ValaSwitchStatement* stmt);
+void vala_ccode_module_visit_block (ValaCCodeModule* self, ValaBlock* b);
 static void vala_ccode_control_flow_module_real_visit_switch_section (ValaCCodeModule* base, ValaSwitchSection* section);
 static void vala_ccode_control_flow_module_real_visit_switch_label (ValaCCodeModule* base, ValaSwitchLabel* label);
 static void vala_ccode_control_flow_module_real_visit_while_statement (ValaCCodeModule* base, ValaWhileStatement* stmt);
 static void vala_ccode_control_flow_module_real_visit_do_statement (ValaCCodeModule* base, ValaDoStatement* stmt);
 static void vala_ccode_control_flow_module_real_visit_for_statement (ValaCCodeModule* base, ValaForStatement* stmt);
+void vala_ccode_base_module_append_temp_decl (ValaCCodeBaseModule* self, ValaCCodeFragment* cfrag, GeeList* temp_vars);
+ValaCCodeExpression* vala_ccode_base_module_get_variable_cexpression (ValaCCodeBaseModule* self, const char* name);
+ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
+void vala_ccode_module_add_simple_check (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
+ValaCCodeExpression* vala_ccode_module_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
+char* vala_ccode_module_get_array_length_cname (ValaCCodeModule* self, const char* array_cname, gint dim);
+ValaCCodeExpression* vala_ccode_base_module_transform_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
+ValaCCodeExpression* vala_ccode_base_module_convert_from_generic_pointer (ValaCCodeBaseModule* self, ValaCCodeExpression* cexpr, ValaDataType* actual_type);
+gboolean vala_ccode_base_module_requires_destroy (ValaCCodeBaseModule* self, ValaDataType* type);
+ValaCCodeExpression* vala_ccode_base_module_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr);
 static void vala_ccode_control_flow_module_real_visit_foreach_statement (ValaCCodeModule* base, ValaForeachStatement* stmt);
+void vala_ccode_base_module_create_local_free (ValaCCodeBaseModule* self, ValaCodeNode* stmt, gboolean stop_at_loop);
 static void vala_ccode_control_flow_module_real_visit_break_statement (ValaCCodeModule* base, ValaBreakStatement* stmt);
 static void vala_ccode_control_flow_module_real_visit_continue_statement (ValaCCodeModule* base, ValaContinueStatement* stmt);
 static gpointer vala_ccode_control_flow_module_parent_class = NULL;
@@ -363,7 +679,7 @@ static void vala_ccode_control_flow_module_visit_string_switch_statement (ValaCC
 							GeeIterator* _cstmt_it;
 							_tmp40 = NULL;
 							_tmp41 = NULL;
-							_cstmt_it = (_tmp41 = gee_iterable_iterator ((GeeIterable*) (_tmp40 = vala_ccode_fragment_get_children ((VALA_CCODE_FRAGMENT (vala_code_node_get_ccodenode (body_stmt)))))), (_tmp40 == NULL) ? NULL : (_tmp40 = (gee_collection_object_unref (_tmp40), NULL)), _tmp41);
+							_cstmt_it = (_tmp41 = gee_iterable_iterator ((GeeIterable*) (_tmp40 = vala_ccode_fragment_get_children (VALA_CCODE_FRAGMENT (vala_code_node_get_ccodenode (body_stmt))))), (_tmp40 == NULL) ? NULL : (_tmp40 = (gee_collection_object_unref (_tmp40), NULL)), _tmp41);
 							while (gee_iterator_next (_cstmt_it)) {
 								ValaCCodeNode* cstmt;
 								cstmt = (ValaCCodeNode*) gee_iterator_get (_cstmt_it);
@@ -791,7 +1107,7 @@ static void vala_ccode_control_flow_module_real_visit_foreach_statement (ValaCCo
 			vala_ccode_block_add_statement (cblock, (ValaCCodeNode*) clendecl);
 			(clendecl == NULL) ? NULL : (clendecl = (vala_ccode_node_unref (clendecl), NULL));
 		}
-		it_name = (g_strconcat (vala_foreach_statement_get_variable_name (stmt), "_it", NULL));
+		it_name = g_strconcat (vala_foreach_statement_get_variable_name (stmt), "_it", NULL);
 		_tmp19 = FALSE;
 		if (((ValaCCodeBaseModule*) self)->current_method != NULL) {
 			_tmp19 = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
