@@ -20,33 +20,105 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gobject/valaclassregisterfunction.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <vala/valadatatype.h>
-#include <vala/valainterface.h>
-#include <ccode/valaccodedeclaration.h>
-#include <ccode/valaccodemodifiers.h>
-#include <ccode/valaccodevariabledeclarator.h>
-#include <ccode/valaccodeconstant.h>
-#include <ccode/valaccodeexpression.h>
-#include <ccode/valaccodedeclarator.h>
-#include <ccode/valaccodenode.h>
-#include <ccode/valaccodefunctioncall.h>
-#include <ccode/valaccodeidentifier.h>
-#include <ccode/valaccodeexpressionstatement.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <vala.h>
+#include <stdlib.h>
+#include <string.h>
+#include <valaccode.h>
+#include <gee.h>
 
 
+#define VALA_TYPE_TYPEREGISTER_FUNCTION (vala_typeregister_function_get_type ())
+#define VALA_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunction))
+#define VALA_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
+#define VALA_IS_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION))
+#define VALA_IS_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION))
+#define VALA_TYPEREGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
 
+typedef struct _ValaTypeRegisterFunction ValaTypeRegisterFunction;
+typedef struct _ValaTypeRegisterFunctionClass ValaTypeRegisterFunctionClass;
+typedef struct _ValaTypeRegisterFunctionPrivate ValaTypeRegisterFunctionPrivate;
+
+#define VALA_TYPE_CLASS_REGISTER_FUNCTION (vala_class_register_function_get_type ())
+#define VALA_CLASS_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CLASS_REGISTER_FUNCTION, ValaClassRegisterFunction))
+#define VALA_CLASS_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CLASS_REGISTER_FUNCTION, ValaClassRegisterFunctionClass))
+#define VALA_IS_CLASS_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CLASS_REGISTER_FUNCTION))
+#define VALA_IS_CLASS_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CLASS_REGISTER_FUNCTION))
+#define VALA_CLASS_REGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CLASS_REGISTER_FUNCTION, ValaClassRegisterFunctionClass))
+
+typedef struct _ValaClassRegisterFunction ValaClassRegisterFunction;
+typedef struct _ValaClassRegisterFunctionClass ValaClassRegisterFunctionClass;
+typedef struct _ValaClassRegisterFunctionPrivate ValaClassRegisterFunctionPrivate;
+
+/**
+ * C function to register a type at runtime.
+ */
+struct _ValaTypeRegisterFunction {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaTypeRegisterFunctionPrivate * priv;
+};
+
+struct _ValaTypeRegisterFunctionClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaTypeRegisterFunction *self);
+	ValaTypeSymbol* (*get_type_declaration) (ValaTypeRegisterFunction* self);
+	char* (*get_type_struct_name) (ValaTypeRegisterFunction* self);
+	char* (*get_base_init_func_name) (ValaTypeRegisterFunction* self);
+	char* (*get_class_finalize_func_name) (ValaTypeRegisterFunction* self);
+	char* (*get_base_finalize_func_name) (ValaTypeRegisterFunction* self);
+	char* (*get_class_init_func_name) (ValaTypeRegisterFunction* self);
+	char* (*get_instance_struct_size) (ValaTypeRegisterFunction* self);
+	char* (*get_instance_init_func_name) (ValaTypeRegisterFunction* self);
+	char* (*get_parent_type_name) (ValaTypeRegisterFunction* self);
+	char* (*get_gtype_value_table_init_function_name) (ValaTypeRegisterFunction* self);
+	char* (*get_gtype_value_table_peek_pointer_function_name) (ValaTypeRegisterFunction* self);
+	char* (*get_gtype_value_table_free_function_name) (ValaTypeRegisterFunction* self);
+	char* (*get_gtype_value_table_copy_function_name) (ValaTypeRegisterFunction* self);
+	char* (*get_gtype_value_table_lcopy_value_function_name) (ValaTypeRegisterFunction* self);
+	char* (*get_gtype_value_table_collect_value_function_name) (ValaTypeRegisterFunction* self);
+	char* (*get_type_flags) (ValaTypeRegisterFunction* self);
+	ValaCCodeFragment* (*get_type_interface_init_declaration) (ValaTypeRegisterFunction* self);
+	ValaCCodeFragment* (*get_type_interface_init_statements) (ValaTypeRegisterFunction* self);
+	ValaSymbolAccessibility (*get_accessibility) (ValaTypeRegisterFunction* self);
+};
+
+/**
+ * C function to register a class at runtime.
+ */
+struct _ValaClassRegisterFunction {
+	ValaTypeRegisterFunction parent_instance;
+	ValaClassRegisterFunctionPrivate * priv;
+};
+
+struct _ValaClassRegisterFunctionClass {
+	ValaTypeRegisterFunctionClass parent_class;
+};
 
 struct _ValaClassRegisterFunctionPrivate {
 	ValaClass* _class_reference;
 };
 
+
+
+gpointer vala_typeregister_function_ref (gpointer instance);
+void vala_typeregister_function_unref (gpointer instance);
+GParamSpec* vala_param_spec_typeregister_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_typeregister_function (GValue* value, gpointer v_object);
+gpointer vala_value_get_typeregister_function (const GValue* value);
+GType vala_typeregister_function_get_type (void);
+GType vala_class_register_function_get_type (void);
 #define VALA_CLASS_REGISTER_FUNCTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CLASS_REGISTER_FUNCTION, ValaClassRegisterFunctionPrivate))
 enum  {
 	VALA_CLASS_REGISTER_FUNCTION_DUMMY_PROPERTY
 };
+void vala_class_register_function_set_class_reference (ValaClassRegisterFunction* self, ValaClass* value);
+void vala_typeregister_function_set_context (ValaTypeRegisterFunction* self, ValaCodeContext* value);
+ValaClassRegisterFunction* vala_class_register_function_new (ValaClass* cl, ValaCodeContext* context);
+ValaClassRegisterFunction* vala_class_register_function_construct (GType object_type, ValaClass* cl, ValaCodeContext* context);
+ValaClassRegisterFunction* vala_class_register_function_new (ValaClass* cl, ValaCodeContext* context);
+ValaClass* vala_class_register_function_get_class_reference (ValaClassRegisterFunction* self);
 static ValaTypeSymbol* vala_class_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base);
 static char* vala_class_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base);
 static char* vala_class_register_function_real_get_base_init_func_name (ValaTypeRegisterFunction* base);
@@ -394,7 +466,7 @@ static ValaCCodeFragment* vala_class_register_function_real_get_type_interface_i
 			char* _tmp6;
 			char* _tmp5;
 			base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
-			if (!(VALA_IS_INTERFACE (vala_data_type_get_data_type (base_type)))) {
+			if (!VALA_IS_INTERFACE (vala_data_type_get_data_type (base_type))) {
 				(base_type == NULL) ? NULL : (base_type = (vala_code_node_unref (base_type), NULL));
 				continue;
 			}
@@ -459,7 +531,7 @@ static ValaCCodeFragment* vala_class_register_function_real_get_type_interface_i
 			char* _tmp12;
 			ValaCCodeExpressionStatement* _tmp14;
 			base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
-			if (!(VALA_IS_INTERFACE (vala_data_type_get_data_type (base_type)))) {
+			if (!VALA_IS_INTERFACE (vala_data_type_get_data_type (base_type))) {
 				(base_type == NULL) ? NULL : (base_type = (vala_code_node_unref (base_type), NULL));
 				continue;
 			}
