@@ -20,26 +20,37 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gobject/valagirwriter.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <vala.h>
 #include <stdio.h>
-#include <vala/valatypesymbol.h>
-#include <vala/valasymbol.h>
-#include <vala/valascope.h>
-#include <vala/valacodenode.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <vala/valadatatype.h>
-#include <vala/valaobjecttype.h>
-#include <gee/collection.h>
-#include <vala/valaarraytype.h>
-#include <vala/valavoidtype.h>
-#include <gee/list.h>
-#include <vala/valaformalparameter.h>
-#include <vala/valamember.h>
-#include <gobject/valaccodebasemodule.h>
+#include <stdlib.h>
+#include <string.h>
+#include <gee.h>
 
 
+#define VALA_TYPE_GIR_WRITER (vala_gir_writer_get_type ())
+#define VALA_GIR_WRITER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GIR_WRITER, ValaGIRWriter))
+#define VALA_GIR_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GIR_WRITER, ValaGIRWriterClass))
+#define VALA_IS_GIR_WRITER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GIR_WRITER))
+#define VALA_IS_GIR_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GIR_WRITER))
+#define VALA_GIR_WRITER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GIR_WRITER, ValaGIRWriterClass))
 
+typedef struct _ValaGIRWriter ValaGIRWriter;
+typedef struct _ValaGIRWriterClass ValaGIRWriterClass;
+typedef struct _ValaGIRWriterPrivate ValaGIRWriterPrivate;
+
+/**
+ * Code visitor generating .gir file for the public interface.
+ */
+struct _ValaGIRWriter {
+	ValaCodeVisitor parent_instance;
+	ValaGIRWriterPrivate * priv;
+};
+
+struct _ValaGIRWriterClass {
+	ValaCodeVisitorClass parent_class;
+};
 
 struct _ValaGIRWriterPrivate {
 	ValaCodeContext* context;
@@ -48,11 +59,19 @@ struct _ValaGIRWriterPrivate {
 	ValaTypeSymbol* gobject_type;
 };
 
+
+
+GType vala_gir_writer_get_type (void);
 #define VALA_GIR_WRITER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_GIR_WRITER, ValaGIRWriterPrivate))
 enum  {
 	VALA_GIR_WRITER_DUMMY_PROPERTY
 };
+void vala_gir_writer_write_file (ValaGIRWriter* self, ValaCodeContext* context, const char* filename);
+static void vala_gir_writer_write_c_include (ValaGIRWriter* self, const char* name);
+static void vala_gir_writer_write_c_includes (ValaGIRWriter* self, ValaNamespace* ns);
+static void vala_gir_writer_write_indent (ValaGIRWriter* self);
 static void vala_gir_writer_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
+static gboolean vala_gir_writer_check_accessibility (ValaGIRWriter* self, ValaSymbol* sym);
 static void vala_gir_writer_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
 static void vala_gir_writer_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st);
 static void vala_gir_writer_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface);
@@ -61,17 +80,19 @@ static void vala_gir_writer_real_visit_enum_value (ValaCodeVisitor* base, ValaEn
 static void vala_gir_writer_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* edomain);
 static void vala_gir_writer_real_visit_error_code (ValaCodeVisitor* base, ValaErrorCode* ecode);
 static void vala_gir_writer_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
+static void vala_gir_writer_write_type (ValaGIRWriter* self, ValaDataType* type);
 static void vala_gir_writer_real_visit_field (ValaCodeVisitor* base, ValaField* f);
 static void vala_gir_writer_write_params (ValaGIRWriter* self, GeeList* params, ValaDataType* instance_type);
+static void vala_gir_writer_write_return_type (ValaGIRWriter* self, ValaDataType* type);
 static void vala_gir_writer_real_visit_delegate (ValaCodeVisitor* base, ValaDelegate* cb);
+ValaDataType* vala_ccode_base_module_get_data_type_for_symbol (ValaTypeSymbol* sym);
 static void vala_gir_writer_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
 static void vala_gir_writer_real_visit_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
 static void vala_gir_writer_real_visit_property (ValaCodeVisitor* base, ValaProperty* prop);
 static void vala_gir_writer_real_visit_signal (ValaCodeVisitor* base, ValaSignal* sig);
-static void vala_gir_writer_write_indent (ValaGIRWriter* self);
-static void vala_gir_writer_write_return_type (ValaGIRWriter* self, ValaDataType* type);
-static void vala_gir_writer_write_type (ValaGIRWriter* self, ValaDataType* type);
-static gboolean vala_gir_writer_check_accessibility (ValaGIRWriter* self, ValaSymbol* sym);
+ValaGIRWriter* vala_gir_writer_new (void);
+ValaGIRWriter* vala_gir_writer_construct (GType object_type);
+ValaGIRWriter* vala_gir_writer_new (void);
 static gpointer vala_gir_writer_parent_class = NULL;
 static void vala_gir_writer_finalize (ValaCodeVisitor* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
@@ -109,8 +130,14 @@ void vala_gir_writer_write_file (ValaGIRWriter* self, ValaCodeContext* context,
 	_tmp4 = NULL;
 	self->priv->stream = (_tmp4 = fopen (filename, "w"), (self->priv->stream == NULL) ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL)), _tmp4);
 	fprintf (self->priv->stream, "<?xml version=\"1.0\"?>\n");
-	fprintf (self->priv->stream, "<repository version=\"1.0\">\n");
+	fprintf (self->priv->stream, "<repository version=\"1.0\"");
+	fprintf (self->priv->stream, " xmlns=\"http://www.gtk.org/introspection/core/1.0\"");
+	fprintf (self->priv->stream, " xmlns:c=\"http://www.gtk.org/introspection/c/1.0\"");
+	fprintf (self->priv->stream, " xmlns:glib=\"http://www.gtk.org/introspection/glib/1.0\"");
+	fprintf (self->priv->stream, ">\n");
+	self->priv->indent++;
 	vala_code_context_accept (context, (ValaCodeVisitor*) self);
+	self->priv->indent--;
 	fprintf (self->priv->stream, "</repository>\n");
 	_tmp5 = NULL;
 	self->priv->stream = (_tmp5 = NULL, (self->priv->stream == NULL) ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL)), _tmp5);
@@ -119,6 +146,85 @@ void vala_gir_writer_write_file (ValaGIRWriter* self, ValaCodeContext* context,
 }
 
 
+static void vala_gir_writer_write_c_includes (ValaGIRWriter* self, ValaNamespace* ns) {
+	GeeSet* header_filenames;
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (ns != NULL);
+	/* Collect C header filenames*/
+	header_filenames = (GeeSet*) gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal);
+	{
+		GeeList* _tmp0;
+		GeeIterator* _tmp1;
+		GeeIterator* _c_header_filename_it;
+		_tmp0 = NULL;
+		_tmp1 = NULL;
+		_c_header_filename_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_symbol_get_cheader_filenames ((ValaSymbol*) ns))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
+		while (gee_iterator_next (_c_header_filename_it)) {
+			char* c_header_filename;
+			c_header_filename = (char*) gee_iterator_get (_c_header_filename_it);
+			gee_collection_add ((GeeCollection*) header_filenames, c_header_filename);
+			c_header_filename = (g_free (c_header_filename), NULL);
+		}
+		(_c_header_filename_it == NULL) ? NULL : (_c_header_filename_it = (gee_collection_object_unref (_c_header_filename_it), NULL));
+	}
+	{
+		GeeCollection* _tmp3;
+		GeeMap* _tmp2;
+		GeeIterator* _tmp4;
+		GeeIterator* _symbol_it;
+		_tmp3 = NULL;
+		_tmp2 = NULL;
+		_tmp4 = NULL;
+		_symbol_it = (_tmp4 = gee_iterable_iterator ((GeeIterable*) (_tmp3 = gee_map_get_values (_tmp2 = vala_scope_get_symbol_table (vala_symbol_get_scope ((ValaSymbol*) ns))))), (_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL)), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp4);
+		while (gee_iterator_next (_symbol_it)) {
+			ValaSymbol* symbol;
+			symbol = (ValaSymbol*) gee_iterator_get (_symbol_it);
+			{
+				GeeList* _tmp5;
+				GeeIterator* _tmp6;
+				GeeIterator* _c_header_filename_it;
+				_tmp5 = NULL;
+				_tmp6 = NULL;
+				_c_header_filename_it = (_tmp6 = gee_iterable_iterator ((GeeIterable*) (_tmp5 = vala_symbol_get_cheader_filenames (symbol))), (_tmp5 == NULL) ? NULL : (_tmp5 = (gee_collection_object_unref (_tmp5), NULL)), _tmp6);
+				while (gee_iterator_next (_c_header_filename_it)) {
+					char* c_header_filename;
+					c_header_filename = (char*) gee_iterator_get (_c_header_filename_it);
+					gee_collection_add ((GeeCollection*) header_filenames, c_header_filename);
+					c_header_filename = (g_free (c_header_filename), NULL);
+				}
+				(_c_header_filename_it == NULL) ? NULL : (_c_header_filename_it = (gee_collection_object_unref (_c_header_filename_it), NULL));
+			}
+			(symbol == NULL) ? NULL : (symbol = (vala_code_node_unref (symbol), NULL));
+		}
+		(_symbol_it == NULL) ? NULL : (_symbol_it = (gee_collection_object_unref (_symbol_it), NULL));
+	}
+	/* Generate c:include tags*/
+	{
+		GeeIterator* _c_header_filename_it;
+		/* Generate c:include tags*/
+		_c_header_filename_it = gee_iterable_iterator ((GeeIterable*) header_filenames);
+		/* Generate c:include tags*/
+		while (gee_iterator_next (_c_header_filename_it)) {
+			char* c_header_filename;
+			/* Generate c:include tags*/
+			c_header_filename = (char*) gee_iterator_get (_c_header_filename_it);
+			vala_gir_writer_write_c_include (self, c_header_filename);
+			c_header_filename = (g_free (c_header_filename), NULL);
+		}
+		(_c_header_filename_it == NULL) ? NULL : (_c_header_filename_it = (gee_collection_object_unref (_c_header_filename_it), NULL));
+	}
+	(header_filenames == NULL) ? NULL : (header_filenames = (gee_collection_object_unref (header_filenames), NULL));
+}
+
+
+static void vala_gir_writer_write_c_include (ValaGIRWriter* self, const char* name) {
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (name != NULL);
+	vala_gir_writer_write_indent (self);
+	fprintf (self->priv->stream, "<c:include name=\"%s\"/>\n", name);
+}
+
+
 static void vala_gir_writer_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns) {
 	ValaGIRWriter * self;
 	self = (ValaGIRWriter*) base;
@@ -136,6 +242,7 @@ static void vala_gir_writer_real_visit_namespace (ValaCodeVisitor* base, ValaNam
 		 not supported in GIR at the moment*/
 		return;
 	}
+	vala_gir_writer_write_c_includes (self, ns);
 	vala_gir_writer_write_indent (self);
 	fprintf (self->priv->stream, "<namespace name=\"%s\" version=\"1.0\">\n", vala_symbol_get_name ((ValaSymbol*) ns));
 	self->priv->indent++;
