@@ -20,15 +20,58 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vala/valabasicblock.h>
-#include <gee/arraylist.h>
-#include <gee/hashset.h>
-#include <gee/collection.h>
-#include <gee/readonlylist.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <gee.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_BASIC_BLOCK (vala_basic_block_get_type ())
+#define VALA_BASIC_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BASIC_BLOCK, ValaBasicBlock))
+#define VALA_BASIC_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockClass))
+#define VALA_IS_BASIC_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BASIC_BLOCK))
+#define VALA_IS_BASIC_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BASIC_BLOCK))
+#define VALA_BASIC_BLOCK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockClass))
 
+typedef struct _ValaBasicBlock ValaBasicBlock;
+typedef struct _ValaBasicBlockClass ValaBasicBlockClass;
+typedef struct _ValaBasicBlockPrivate ValaBasicBlockPrivate;
+
+#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
+#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
+#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
+#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
+#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+
+typedef struct _ValaCodeNode ValaCodeNode;
+typedef struct _ValaCodeNodeClass ValaCodeNodeClass;
+
+#define VALA_TYPE_PHI_FUNCTION (vala_phi_function_get_type ())
+#define VALA_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunction))
+#define VALA_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))
+#define VALA_IS_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PHI_FUNCTION))
+#define VALA_IS_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PHI_FUNCTION))
+#define VALA_PHI_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))
+
+typedef struct _ValaPhiFunction ValaPhiFunction;
+typedef struct _ValaPhiFunctionClass ValaPhiFunctionClass;
+typedef struct _ValaParamSpecBasicBlock ValaParamSpecBasicBlock;
+
+/**
+ * Represents a basic block, i.e. a straight-line piece of code without any
+ * jumps or jump targets.
+ */
+struct _ValaBasicBlock {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaBasicBlockPrivate * priv;
+};
+
+struct _ValaBasicBlockClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaBasicBlock *self);
+};
 
 struct _ValaBasicBlockPrivate {
 	GeeList* nodes;
@@ -40,11 +83,56 @@ struct _ValaBasicBlockPrivate {
 	GeeSet* phi_functions;
 };
 
+struct _ValaParamSpecBasicBlock {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_basic_block_ref (gpointer instance);
+void vala_basic_block_unref (gpointer instance);
+GParamSpec* vala_param_spec_basic_block (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_basic_block (GValue* value, gpointer v_object);
+gpointer vala_value_get_basic_block (const GValue* value);
+GType vala_basic_block_get_type (void);
+gpointer vala_code_node_ref (gpointer instance);
+void vala_code_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_node (const GValue* value);
+GType vala_code_node_get_type (void);
+gpointer vala_phi_function_ref (gpointer instance);
+void vala_phi_function_unref (gpointer instance);
+GParamSpec* vala_param_spec_phi_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_phi_function (GValue* value, gpointer v_object);
+gpointer vala_value_get_phi_function (const GValue* value);
+GType vala_phi_function_get_type (void);
 #define VALA_BASIC_BLOCK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockPrivate))
 enum  {
 	VALA_BASIC_BLOCK_DUMMY_PROPERTY
 };
+ValaBasicBlock* vala_basic_block_new (void);
+ValaBasicBlock* vala_basic_block_construct (GType object_type);
+ValaBasicBlock* vala_basic_block_new (void);
+ValaBasicBlock* vala_basic_block_new_entry (void);
+ValaBasicBlock* vala_basic_block_construct_entry (GType object_type);
+ValaBasicBlock* vala_basic_block_new_entry (void);
+ValaBasicBlock* vala_basic_block_new_exit (void);
+ValaBasicBlock* vala_basic_block_construct_exit (GType object_type);
+ValaBasicBlock* vala_basic_block_new_exit (void);
+void vala_basic_block_add_node (ValaBasicBlock* self, ValaCodeNode* node);
+GeeList* vala_basic_block_get_nodes (ValaBasicBlock* self);
+void vala_basic_block_connect (ValaBasicBlock* self, ValaBasicBlock* target);
+GeeList* vala_basic_block_get_predecessors (ValaBasicBlock* self);
+GeeList* vala_basic_block_get_successors (ValaBasicBlock* self);
 static void vala_basic_block_set_parent (ValaBasicBlock* self, ValaBasicBlock* value);
+void vala_basic_block_add_child (ValaBasicBlock* self, ValaBasicBlock* block);
+GeeList* vala_basic_block_get_children (ValaBasicBlock* self);
+void vala_basic_block_add_dominator_frontier (ValaBasicBlock* self, ValaBasicBlock* block);
+GeeSet* vala_basic_block_get_dominator_frontier (ValaBasicBlock* self);
+void vala_basic_block_add_phi_function (ValaBasicBlock* self, ValaPhiFunction* phi);
+GeeSet* vala_basic_block_get_phi_functions (ValaBasicBlock* self);
+ValaBasicBlock* vala_basic_block_get_parent (ValaBasicBlock* self);
 static gpointer vala_basic_block_parent_class = NULL;
 static void vala_basic_block_finalize (ValaBasicBlock* obj);
 
