@@ -20,26 +20,141 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vala/valacodecontext.h>
-#include <gee/collection.h>
-#include <gee/arraylist.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <gee/readonlylist.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
+#include <gee.h>
 #include <glib/gstdio.h>
 #include <config.h>
-#include <vala/valareport.h>
-#include <vala/valanamespace.h>
-#include <vala/valasourcereference.h>
-#include <vala/valamethod.h>
-#include <vala/valacodevisitor.h>
-#include <vala/valasourcefile.h>
-#include <vala/valasourcefilecycle.h>
-#include <vala/valacodegenerator.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_CODE_CONTEXT (vala_code_context_get_type ())
+#define VALA_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContext))
+#define VALA_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))
+#define VALA_IS_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_CONTEXT))
+#define VALA_IS_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_CONTEXT))
+#define VALA_CODE_CONTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))
 
+typedef struct _ValaCodeContext ValaCodeContext;
+typedef struct _ValaCodeContextClass ValaCodeContextClass;
+typedef struct _ValaCodeContextPrivate ValaCodeContextPrivate;
+
+#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
+#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
+#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
+#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
+#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+
+typedef struct _ValaCodeNode ValaCodeNode;
+typedef struct _ValaCodeNodeClass ValaCodeNodeClass;
+
+#define VALA_TYPE_SYMBOL (vala_symbol_get_type ())
+#define VALA_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SYMBOL, ValaSymbol))
+#define VALA_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SYMBOL, ValaSymbolClass))
+#define VALA_IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SYMBOL))
+#define VALA_IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SYMBOL))
+#define VALA_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SYMBOL, ValaSymbolClass))
+
+typedef struct _ValaSymbol ValaSymbol;
+typedef struct _ValaSymbolClass ValaSymbolClass;
+
+#define VALA_TYPE_MEMBER (vala_member_get_type ())
+#define VALA_MEMBER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_MEMBER, ValaMember))
+#define VALA_MEMBER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_MEMBER, ValaMemberClass))
+#define VALA_IS_MEMBER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_MEMBER))
+#define VALA_IS_MEMBER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_MEMBER))
+#define VALA_MEMBER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_MEMBER, ValaMemberClass))
+
+typedef struct _ValaMember ValaMember;
+typedef struct _ValaMemberClass ValaMemberClass;
+
+#define VALA_TYPE_METHOD (vala_method_get_type ())
+#define VALA_METHOD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_METHOD, ValaMethod))
+#define VALA_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_METHOD, ValaMethodClass))
+#define VALA_IS_METHOD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_METHOD))
+#define VALA_IS_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_METHOD))
+#define VALA_METHOD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_METHOD, ValaMethodClass))
+
+typedef struct _ValaMethod ValaMethod;
+typedef struct _ValaMethodClass ValaMethodClass;
+
+#define VALA_TYPE_REPORT (vala_report_get_type ())
+#define VALA_REPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_REPORT, ValaReport))
+#define VALA_REPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_REPORT, ValaReportClass))
+#define VALA_IS_REPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_REPORT))
+#define VALA_IS_REPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_REPORT))
+#define VALA_REPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_REPORT, ValaReportClass))
+
+typedef struct _ValaReport ValaReport;
+typedef struct _ValaReportClass ValaReportClass;
+
+#define VALA_TYPE_SOURCE_FILE (vala_source_file_get_type ())
+#define VALA_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFile))
+#define VALA_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
+#define VALA_IS_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_FILE))
+#define VALA_IS_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_FILE))
+#define VALA_SOURCE_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
+
+typedef struct _ValaSourceFile ValaSourceFile;
+typedef struct _ValaSourceFileClass ValaSourceFileClass;
+
+#define VALA_TYPE_NAMESPACE (vala_namespace_get_type ())
+#define VALA_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_NAMESPACE, ValaNamespace))
+#define VALA_NAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_NAMESPACE, ValaNamespaceClass))
+#define VALA_IS_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_NAMESPACE))
+#define VALA_IS_NAMESPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_NAMESPACE))
+#define VALA_NAMESPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_NAMESPACE, ValaNamespaceClass))
+
+typedef struct _ValaNamespace ValaNamespace;
+typedef struct _ValaNamespaceClass ValaNamespaceClass;
+
+#define VALA_TYPE_CODE_VISITOR (vala_code_visitor_get_type ())
+#define VALA_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitor))
+#define VALA_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))
+#define VALA_IS_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_VISITOR))
+#define VALA_IS_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_VISITOR))
+#define VALA_CODE_VISITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))
+
+typedef struct _ValaCodeVisitor ValaCodeVisitor;
+typedef struct _ValaCodeVisitorClass ValaCodeVisitorClass;
+
+#define VALA_TYPE_CODE_GENERATOR (vala_code_generator_get_type ())
+#define VALA_CODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_GENERATOR, ValaCodeGenerator))
+#define VALA_CODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_GENERATOR, ValaCodeGeneratorClass))
+#define VALA_IS_CODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_GENERATOR))
+#define VALA_IS_CODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_GENERATOR))
+#define VALA_CODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_GENERATOR, ValaCodeGeneratorClass))
+
+typedef struct _ValaCodeGenerator ValaCodeGenerator;
+typedef struct _ValaCodeGeneratorClass ValaCodeGeneratorClass;
+
+#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
+#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
+#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+
+typedef struct _ValaSourceReference ValaSourceReference;
+typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
+typedef struct _ValaParamSpecCodeContext ValaParamSpecCodeContext;
+
+/**
+ * The root of the code tree.
+ */
+struct _ValaCodeContext {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaCodeContextPrivate * priv;
+};
+
+struct _ValaCodeContextClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaCodeContext *self);
+};
 
 struct _ValaCodeContextPrivate {
 	char* _library;
@@ -52,6 +167,7 @@ struct _ValaCodeContextPrivate {
 	gboolean _dbus_transformation;
 	gboolean _ccode_only;
 	char* _header_filename;
+	char* _internal_header_filename;
 	gboolean _compile_only;
 	char* _output;
 	char* _basedir;
@@ -67,18 +183,136 @@ struct _ValaCodeContextPrivate {
 	GeeList* source_files;
 	GeeList* c_source_files;
 	ValaNamespace* _root;
-	GeeList* cycles;
 	GeeList* packages;
+	GeeSet* defines;
 	ValaCodeGenerator* _codegen;
 };
 
+struct _ValaParamSpecCodeContext {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_code_context_ref (gpointer instance);
+void vala_code_context_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_context (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_context (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_context (const GValue* value);
+GType vala_code_context_get_type (void);
+gpointer vala_code_node_ref (gpointer instance);
+void vala_code_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_node (const GValue* value);
+GType vala_code_node_get_type (void);
+GType vala_symbol_get_type (void);
+GType vala_member_get_type (void);
+GType vala_method_get_type (void);
+GType vala_report_get_type (void);
+gpointer vala_source_file_ref (gpointer instance);
+void vala_source_file_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_file (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_file (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_file (const GValue* value);
+GType vala_source_file_get_type (void);
+GType vala_namespace_get_type (void);
+gpointer vala_code_visitor_ref (gpointer instance);
+void vala_code_visitor_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_visitor (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_visitor (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_visitor (const GValue* value);
+GType vala_code_visitor_get_type (void);
+GType vala_code_generator_get_type (void);
 #define VALA_CODE_CONTEXT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CODE_CONTEXT, ValaCodeContextPrivate))
 enum  {
 	VALA_CODE_CONTEXT_DUMMY_PROPERTY
 };
+ValaReport* vala_report_new (void);
+ValaReport* vala_report_construct (GType object_type);
+gpointer vala_source_reference_ref (gpointer instance);
+void vala_source_reference_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_reference (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_reference (const GValue* value);
+GType vala_source_reference_get_type (void);
+ValaNamespace* vala_namespace_new (const char* name, ValaSourceReference* source_reference);
+ValaNamespace* vala_namespace_construct (GType object_type, const char* name, ValaSourceReference* source_reference);
+static GStaticPrivate vala_code_context_context_stack_key;
 static GStaticPrivate vala_code_context_context_stack_key = {0};
-static ValaSourceFile* vala_code_context_find_cycle_head (ValaCodeContext* self, ValaSourceFile* file);
-static void vala_code_context_visit (ValaCodeContext* self, ValaSourceFile* file, GeeList* chain);
+ValaCodeGenerator* vala_code_generator_new (void);
+ValaCodeGenerator* vala_code_generator_construct (GType object_type);
+gint vala_code_context_get_target_glib_major (ValaCodeContext* self);
+gint vala_code_context_get_target_glib_minor (ValaCodeContext* self);
+gboolean vala_code_context_require_glib_version (ValaCodeContext* self, gint major, gint minor);
+ValaCodeContext* vala_code_context_new (void);
+ValaCodeContext* vala_code_context_construct (GType object_type);
+ValaCodeContext* vala_code_context_new (void);
+ValaCodeContext* vala_code_context_get (void);
+void vala_code_context_push (ValaCodeContext* context);
+void vala_code_context_pop (void);
+GeeList* vala_code_context_get_source_files (ValaCodeContext* self);
+GeeList* vala_code_context_get_c_source_files (ValaCodeContext* self);
+void vala_code_context_add_source_file (ValaCodeContext* self, ValaSourceFile* file);
+void vala_code_context_add_c_source_file (ValaCodeContext* self, const char* file);
+GeeList* vala_code_context_get_packages (ValaCodeContext* self);
+gboolean vala_code_context_has_package (ValaCodeContext* self, const char* pkg);
+void vala_code_context_add_package (ValaCodeContext* self, const char* pkg);
+ValaNamespace* vala_code_context_get_root (ValaCodeContext* self);
+void vala_code_node_accept (ValaCodeNode* self, ValaCodeVisitor* visitor);
+void vala_source_file_accept (ValaSourceFile* self, ValaCodeVisitor* visitor);
+void vala_code_context_accept (ValaCodeContext* self, ValaCodeVisitor* visitor);
+void vala_code_context_add_define (ValaCodeContext* self, const char* define);
+gboolean vala_code_context_is_defined (ValaCodeContext* self, const char* define);
+char* vala_code_context_get_package_path (ValaCodeContext* self, const char* pkg, char** vapi_directories, int vapi_directories_length1);
+const char* vala_code_context_get_library (ValaCodeContext* self);
+void vala_code_context_set_library (ValaCodeContext* self, const char* value);
+gboolean vala_code_context_get_memory_management (ValaCodeContext* self);
+void vala_code_context_set_memory_management (ValaCodeContext* self, gboolean value);
+gboolean vala_code_context_get_assert (ValaCodeContext* self);
+void vala_code_context_set_assert (ValaCodeContext* self, gboolean value);
+gboolean vala_code_context_get_checking (ValaCodeContext* self);
+void vala_code_context_set_checking (ValaCodeContext* self, gboolean value);
+gboolean vala_code_context_get_deprecated (ValaCodeContext* self);
+void vala_code_context_set_deprecated (ValaCodeContext* self, gboolean value);
+gboolean vala_code_context_get_experimental (ValaCodeContext* self);
+void vala_code_context_set_experimental (ValaCodeContext* self, gboolean value);
+gboolean vala_code_context_get_non_null_experimental (ValaCodeContext* self);
+void vala_code_context_set_non_null_experimental (ValaCodeContext* self, gboolean value);
+gboolean vala_code_context_get_dbus_transformation (ValaCodeContext* self);
+void vala_code_context_set_dbus_transformation (ValaCodeContext* self, gboolean value);
+gboolean vala_code_context_get_ccode_only (ValaCodeContext* self);
+void vala_code_context_set_ccode_only (ValaCodeContext* self, gboolean value);
+const char* vala_code_context_get_header_filename (ValaCodeContext* self);
+void vala_code_context_set_header_filename (ValaCodeContext* self, const char* value);
+const char* vala_code_context_get_internal_header_filename (ValaCodeContext* self);
+void vala_code_context_set_internal_header_filename (ValaCodeContext* self, const char* value);
+gboolean vala_code_context_get_compile_only (ValaCodeContext* self);
+void vala_code_context_set_compile_only (ValaCodeContext* self, gboolean value);
+const char* vala_code_context_get_output (ValaCodeContext* self);
+void vala_code_context_set_output (ValaCodeContext* self, const char* value);
+const char* vala_code_context_get_basedir (ValaCodeContext* self);
+void vala_code_context_set_basedir (ValaCodeContext* self, const char* value);
+const char* vala_code_context_get_directory (ValaCodeContext* self);
+void vala_code_context_set_directory (ValaCodeContext* self, const char* value);
+gboolean vala_code_context_get_debug (ValaCodeContext* self);
+void vala_code_context_set_debug (ValaCodeContext* self, gboolean value);
+gint vala_code_context_get_optlevel (ValaCodeContext* self);
+void vala_code_context_set_optlevel (ValaCodeContext* self, gint value);
+gboolean vala_code_context_get_thread (ValaCodeContext* self);
+void vala_code_context_set_thread (ValaCodeContext* self, gboolean value);
+ValaMethod* vala_code_context_get_module_init_method (ValaCodeContext* self);
+void vala_code_context_set_module_init_method (ValaCodeContext* self, ValaMethod* value);
+gboolean vala_code_context_get_save_temps (ValaCodeContext* self);
+void vala_code_context_set_save_temps (ValaCodeContext* self, gboolean value);
+void vala_code_context_set_target_glib_major (ValaCodeContext* self, gint value);
+void vala_code_context_set_target_glib_minor (ValaCodeContext* self, gint value);
+gboolean vala_code_context_get_save_csources (ValaCodeContext* self);
+gboolean vala_code_context_get_save_cheaders (ValaCodeContext* self);
+ValaReport* vala_code_context_get_report (ValaCodeContext* self);
+void vala_code_context_set_report (ValaCodeContext* self, ValaReport* value);
+ValaCodeGenerator* vala_code_context_get_codegen (ValaCodeContext* self);
+void vala_code_context_set_codegen (ValaCodeContext* self, ValaCodeGenerator* value);
 static gpointer vala_code_context_parent_class = NULL;
 static void vala_code_context_finalize (ValaCodeContext* obj);
 static gint _vala_array_length (gpointer array);
@@ -261,290 +495,17 @@ void vala_code_context_accept (ValaCodeContext* self, ValaCodeVisitor* visitor)
 }
 
 
-/**
- * Find and resolve cycles in source file dependencies.
- */
-void vala_code_context_find_header_cycles (ValaCodeContext* self) {
+void vala_code_context_add_define (ValaCodeContext* self, const char* define) {
 	g_return_if_fail (self != NULL);
-	/* find cycles in dependencies between source files */
-	{
-		GeeIterator* _file_it;
-		/* find cycles in dependencies between source files */
-		_file_it = gee_iterable_iterator ((GeeIterable*) self->priv->source_files);
-		/* find cycles in dependencies between source files */
-		while (gee_iterator_next (_file_it)) {
-			ValaSourceFile* file;
-			/* find cycles in dependencies between source files */
-			file = (ValaSourceFile*) gee_iterator_get (_file_it);
-			/* we're only interested in internal source files */
-			if (vala_source_file_get_external_package (file)) {
-				(file == NULL) ? NULL : (file = (vala_source_file_unref (file), NULL));
-				continue;
-			}
-			if (vala_source_file_get_mark (file) == 0) {
-				GeeArrayList* _tmp0;
-				_tmp0 = NULL;
-				vala_code_context_visit (self, file, (GeeList*) (_tmp0 = gee_array_list_new (VALA_TYPE_SOURCE_FILE, (GBoxedCopyFunc) vala_source_file_ref, vala_source_file_unref, g_direct_equal)));
-				(_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL));
-			}
-			(file == NULL) ? NULL : (file = (vala_source_file_unref (file), NULL));
-		}
-		(_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL));
-	}
-	/* find one head for each cycle, it must not have any
-	 * hard dependencies on other files in the cycle
-	 */
-	{
-		GeeIterator* _cycle_it;
-		/* find one head for each cycle, it must not have any
-		 * hard dependencies on other files in the cycle
-		 */
-		_cycle_it = gee_iterable_iterator ((GeeIterable*) self->priv->cycles);
-		/* find one head for each cycle, it must not have any
-		 * hard dependencies on other files in the cycle
-		 */
-		while (gee_iterator_next (_cycle_it)) {
-			ValaSourceFileCycle* cycle;
-			/* find one head for each cycle, it must not have any
-			 * hard dependencies on other files in the cycle
-			 */
-			cycle = (ValaSourceFileCycle*) gee_iterator_get (_cycle_it);
-			cycle->head = vala_code_context_find_cycle_head (self, (ValaSourceFile*) gee_list_get (cycle->files, 0));
-			vala_source_file_set_is_cycle_head (cycle->head, TRUE);
-			(cycle == NULL) ? NULL : (cycle = (vala_source_file_cycle_unref (cycle), NULL));
-		}
-		(_cycle_it == NULL) ? NULL : (_cycle_it = (gee_collection_object_unref (_cycle_it), NULL));
-	}
-	/* connect the source files in a non-cyclic way
-	 * cycle members connect to the head of their cycle
-	 */
-	{
-		GeeIterator* _file2_it;
-		/* connect the source files in a non-cyclic way
-		 * cycle members connect to the head of their cycle
-		 */
-		_file2_it = gee_iterable_iterator ((GeeIterable*) self->priv->source_files);
-		/* connect the source files in a non-cyclic way
-		 * cycle members connect to the head of their cycle
-		 */
-		while (gee_iterator_next (_file2_it)) {
-			ValaSourceFile* file2;
-			/* connect the source files in a non-cyclic way
-			 * cycle members connect to the head of their cycle
-			 */
-			file2 = (ValaSourceFile*) gee_iterator_get (_file2_it);
-			/* we're only interested in internal source files */
-			if (vala_source_file_get_external_package (file2)) {
-				(file2 == NULL) ? NULL : (file2 = (vala_source_file_unref (file2), NULL));
-				continue;
-			}
-			{
-				GeeList* _tmp1;
-				GeeIterator* _tmp2;
-				GeeIterator* _dep_it;
-				_tmp1 = NULL;
-				_tmp2 = NULL;
-				_dep_it = (_tmp2 = gee_iterable_iterator ((GeeIterable*) (_tmp1 = vala_source_file_get_header_internal_dependencies (file2))), (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2);
-				while (gee_iterator_next (_dep_it)) {
-					ValaSourceFile* _tmp3;
-					ValaSourceFile* dep;
-					gboolean _tmp4;
-					_tmp3 = NULL;
-					dep = (_tmp3 = (ValaSourceFile*) gee_iterator_get (_dep_it), (_tmp3 == NULL) ? NULL : vala_source_file_ref (_tmp3));
-					_tmp4 = FALSE;
-					if (vala_source_file_get_cycle (file2) != NULL) {
-						_tmp4 = vala_source_file_get_cycle (dep) == vala_source_file_get_cycle (file2);
-					} else {
-						_tmp4 = FALSE;
-					}
-					if (_tmp4) {
-						/* in the same cycle */
-						if (!vala_source_file_get_is_cycle_head (file2)) {
-							char* _tmp5;
-							/* include header of cycle head */
-							_tmp5 = NULL;
-							vala_source_file_add_header_internal_include (file2, _tmp5 = vala_source_file_get_cinclude_filename (vala_source_file_get_cycle (file2)->head));
-							_tmp5 = (g_free (_tmp5), NULL);
-						}
-					} else {
-						char* _tmp6;
-						/* we can just include the headers if they are not in a cycle or not in the same cycle as the current file */
-						_tmp6 = NULL;
-						vala_source_file_add_header_internal_include (file2, _tmp6 = vala_source_file_get_cinclude_filename (dep));
-						_tmp6 = (g_free (_tmp6), NULL);
-					}
-					(dep == NULL) ? NULL : (dep = (vala_source_file_unref (dep), NULL));
-				}
-				(_dep_it == NULL) ? NULL : (_dep_it = (gee_collection_object_unref (_dep_it), NULL));
-			}
-			(file2 == NULL) ? NULL : (file2 = (vala_source_file_unref (file2), NULL));
-		}
-		(_file2_it == NULL) ? NULL : (_file2_it = (gee_collection_object_unref (_file2_it), NULL));
-	}
+	g_return_if_fail (define != NULL);
+	gee_collection_add ((GeeCollection*) self->priv->defines, define);
 }
 
 
-static ValaSourceFile* vala_code_context_find_cycle_head (ValaCodeContext* self, ValaSourceFile* file) {
-	g_return_val_if_fail (self != NULL, NULL);
-	g_return_val_if_fail (file != NULL, NULL);
-	{
-		GeeList* _tmp0;
-		GeeIterator* _tmp1;
-		GeeIterator* _dep_it;
-		_tmp0 = NULL;
-		_tmp1 = NULL;
-		_dep_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_source_file_get_header_internal_full_dependencies (file))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
-		while (gee_iterator_next (_dep_it)) {
-			ValaSourceFile* _tmp2;
-			ValaSourceFile* dep;
-			_tmp2 = NULL;
-			dep = (_tmp2 = (ValaSourceFile*) gee_iterator_get (_dep_it), (_tmp2 == NULL) ? NULL : vala_source_file_ref (_tmp2));
-			if (dep == file) {
-				/* ignore file-internal dependencies */
-				(dep == NULL) ? NULL : (dep = (vala_source_file_unref (dep), NULL));
-				continue;
-			}
-			{
-				GeeIterator* _cycle_file_it;
-				_cycle_file_it = gee_iterable_iterator ((GeeIterable*) vala_source_file_get_cycle (file)->files);
-				while (gee_iterator_next (_cycle_file_it)) {
-					ValaSourceFile* _tmp3;
-					ValaSourceFile* cycle_file;
-					_tmp3 = NULL;
-					cycle_file = (_tmp3 = (ValaSourceFile*) gee_iterator_get (_cycle_file_it), (_tmp3 == NULL) ? NULL : vala_source_file_ref (_tmp3));
-					if (dep == cycle_file) {
-						ValaSourceFile* _tmp4;
-						_tmp4 = NULL;
-						return (_tmp4 = vala_code_context_find_cycle_head (self, dep), (cycle_file == NULL) ? NULL : (cycle_file = (vala_source_file_unref (cycle_file), NULL)), (_cycle_file_it == NULL) ? NULL : (_cycle_file_it = (gee_collection_object_unref (_cycle_file_it), NULL)), (dep == NULL) ? NULL : (dep = (vala_source_file_unref (dep), NULL)), (_dep_it == NULL) ? NULL : (_dep_it = (gee_collection_object_unref (_dep_it), NULL)), _tmp4);
-					}
-					(cycle_file == NULL) ? NULL : (cycle_file = (vala_source_file_unref (cycle_file), NULL));
-				}
-				(_cycle_file_it == NULL) ? NULL : (_cycle_file_it = (gee_collection_object_unref (_cycle_file_it), NULL));
-			}
-			(dep == NULL) ? NULL : (dep = (vala_source_file_unref (dep), NULL));
-		}
-		(_dep_it == NULL) ? NULL : (_dep_it = (gee_collection_object_unref (_dep_it), NULL));
-	}
-	/* no hard dependencies on members of the same cycle found
-	 * source file suitable as cycle head
-	 */
-	return file;
-}
-
-
-static void vala_code_context_visit (ValaCodeContext* self, ValaSourceFile* file, GeeList* chain) {
-	GeeList* l;
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (file != NULL);
-	g_return_if_fail (chain != NULL);
-	l = (GeeList*) gee_array_list_new (VALA_TYPE_SOURCE_FILE, (GBoxedCopyFunc) vala_source_file_ref, vala_source_file_unref, g_direct_equal);
-	{
-		GeeIterator* _chain_file_it;
-		_chain_file_it = gee_iterable_iterator ((GeeIterable*) chain);
-		while (gee_iterator_next (_chain_file_it)) {
-			ValaSourceFile* chain_file;
-			chain_file = (ValaSourceFile*) gee_iterator_get (_chain_file_it);
-			gee_collection_add ((GeeCollection*) l, chain_file);
-			(chain_file == NULL) ? NULL : (chain_file = (vala_source_file_unref (chain_file), NULL));
-		}
-		(_chain_file_it == NULL) ? NULL : (_chain_file_it = (gee_collection_object_unref (_chain_file_it), NULL));
-	}
-	gee_collection_add ((GeeCollection*) l, file);
-	/* mark file as currently being visited */
-	vala_source_file_set_mark (file, 1);
-	{
-		GeeList* _tmp0;
-		GeeIterator* _tmp1;
-		GeeIterator* _dep_it;
-		_tmp0 = NULL;
-		_tmp1 = NULL;
-		_dep_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_source_file_get_header_internal_dependencies (file))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
-		while (gee_iterator_next (_dep_it)) {
-			ValaSourceFile* _tmp2;
-			ValaSourceFile* dep;
-			_tmp2 = NULL;
-			dep = (_tmp2 = (ValaSourceFile*) gee_iterator_get (_dep_it), (_tmp2 == NULL) ? NULL : vala_source_file_ref (_tmp2));
-			if (file == dep) {
-				(dep == NULL) ? NULL : (dep = (vala_source_file_unref (dep), NULL));
-				continue;
-			}
-			if (vala_source_file_get_mark (dep) == 1) {
-				ValaSourceFileCycle* cycle;
-				gboolean cycle_start_found;
-				/* found cycle */
-				cycle = vala_source_file_cycle_new ();
-				gee_collection_add ((GeeCollection*) self->priv->cycles, cycle);
-				cycle_start_found = FALSE;
-				{
-					GeeIterator* _cycle_file_it;
-					_cycle_file_it = gee_iterable_iterator ((GeeIterable*) l);
-					while (gee_iterator_next (_cycle_file_it)) {
-						ValaSourceFile* cycle_file;
-						ValaSourceFileCycle* _tmp3;
-						ValaSourceFileCycle* ref_cycle_file_cycle;
-						cycle_file = (ValaSourceFile*) gee_iterator_get (_cycle_file_it);
-						_tmp3 = NULL;
-						ref_cycle_file_cycle = (_tmp3 = vala_source_file_get_cycle (cycle_file), (_tmp3 == NULL) ? NULL : vala_source_file_cycle_ref (_tmp3));
-						if (!cycle_start_found) {
-							if (cycle_file == dep) {
-								cycle_start_found = TRUE;
-							}
-						}
-						if (!cycle_start_found) {
-							(cycle_file == NULL) ? NULL : (cycle_file = (vala_source_file_unref (cycle_file), NULL));
-							(ref_cycle_file_cycle == NULL) ? NULL : (ref_cycle_file_cycle = (vala_source_file_cycle_unref (ref_cycle_file_cycle), NULL));
-							continue;
-						}
-						if (ref_cycle_file_cycle != NULL) {
-							/* file already in a cycle */
-							if (vala_source_file_get_cycle (cycle_file) == cycle) {
-								/* file is in the same cycle, nothing to do */
-								(cycle_file == NULL) ? NULL : (cycle_file = (vala_source_file_unref (cycle_file), NULL));
-								(ref_cycle_file_cycle == NULL) ? NULL : (ref_cycle_file_cycle = (vala_source_file_cycle_unref (ref_cycle_file_cycle), NULL));
-								continue;
-							}
-							/* file is in an other cycle, merge the two cycles */
-							gee_collection_remove ((GeeCollection*) self->priv->cycles, vala_source_file_get_cycle (cycle_file));
-							{
-								GeeIterator* _inner_cycle_file_it;
-								_inner_cycle_file_it = gee_iterable_iterator ((GeeIterable*) vala_source_file_get_cycle (cycle_file)->files);
-								while (gee_iterator_next (_inner_cycle_file_it)) {
-									ValaSourceFile* _tmp4;
-									ValaSourceFile* inner_cycle_file;
-									_tmp4 = NULL;
-									inner_cycle_file = (_tmp4 = (ValaSourceFile*) gee_iterator_get (_inner_cycle_file_it), (_tmp4 == NULL) ? NULL : vala_source_file_ref (_tmp4));
-									if (vala_source_file_get_cycle (inner_cycle_file) != cycle) {
-										/* file in inner cycle not yet added to outer cycle */
-										gee_collection_add ((GeeCollection*) cycle->files, inner_cycle_file);
-										vala_source_file_set_cycle (inner_cycle_file, cycle);
-									}
-									(inner_cycle_file == NULL) ? NULL : (inner_cycle_file = (vala_source_file_unref (inner_cycle_file), NULL));
-								}
-								(_inner_cycle_file_it == NULL) ? NULL : (_inner_cycle_file_it = (gee_collection_object_unref (_inner_cycle_file_it), NULL));
-							}
-						} else {
-							gee_collection_add ((GeeCollection*) cycle->files, cycle_file);
-							vala_source_file_set_cycle (cycle_file, cycle);
-						}
-						(cycle_file == NULL) ? NULL : (cycle_file = (vala_source_file_unref (cycle_file), NULL));
-						(ref_cycle_file_cycle == NULL) ? NULL : (ref_cycle_file_cycle = (vala_source_file_cycle_unref (ref_cycle_file_cycle), NULL));
-					}
-					(_cycle_file_it == NULL) ? NULL : (_cycle_file_it = (gee_collection_object_unref (_cycle_file_it), NULL));
-				}
-				(cycle == NULL) ? NULL : (cycle = (vala_source_file_cycle_unref (cycle), NULL));
-			} else {
-				if (vala_source_file_get_mark (dep) == 0) {
-					/* found not yet visited file */
-					vala_code_context_visit (self, dep, l);
-				}
-			}
-			(dep == NULL) ? NULL : (dep = (vala_source_file_unref (dep), NULL));
-		}
-		(_dep_it == NULL) ? NULL : (_dep_it = (gee_collection_object_unref (_dep_it), NULL));
-	}
-	/* mark file as successfully visited */
-	vala_source_file_set_mark (file, 2);
-	(l == NULL) ? NULL : (l = (gee_collection_object_unref (l), NULL));
+gboolean vala_code_context_is_defined (ValaCodeContext* self, const char* define) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	g_return_val_if_fail (define != NULL, FALSE);
+	return gee_collection_contains ((GeeCollection*) self->priv->defines, define);
 }
 
 
@@ -749,6 +710,22 @@ void vala_code_context_set_header_filename (ValaCodeContext* self, const char* v
 }
 
 
+const char* vala_code_context_get_internal_header_filename (ValaCodeContext* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return self->priv->_internal_header_filename;
+}
+
+
+void vala_code_context_set_internal_header_filename (ValaCodeContext* self, const char* value) {
+	char* _tmp2;
+	const char* _tmp1;
+	g_return_if_fail (self != NULL);
+	_tmp2 = NULL;
+	_tmp1 = NULL;
+	self->priv->_internal_header_filename = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_internal_header_filename = (g_free (self->priv->_internal_header_filename), NULL), _tmp2);
+}
+
+
 gboolean vala_code_context_get_compile_only (ValaCodeContext* self) {
 	g_return_val_if_fail (self != NULL, FALSE);
 	return self->priv->_compile_only;
@@ -809,12 +786,6 @@ void vala_code_context_set_directory (ValaCodeContext* self, const char* value)
 }
 
 
-gboolean vala_code_context_get_legacy_headers (ValaCodeContext* self) {
-	g_return_val_if_fail (self != NULL, FALSE);
-	return self->priv->_header_filename == NULL;
-}
-
-
 gboolean vala_code_context_get_debug (ValaCodeContext* self) {
 	g_return_val_if_fail (self != NULL, FALSE);
 	return self->priv->_debug;
@@ -1068,8 +1039,8 @@ static void vala_code_context_instance_init (ValaCodeContext * self) {
 	self->priv->source_files = (GeeList*) gee_array_list_new (VALA_TYPE_SOURCE_FILE, (GBoxedCopyFunc) vala_source_file_ref, vala_source_file_unref, g_direct_equal);
 	self->priv->c_source_files = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
 	self->priv->_root = vala_namespace_new (NULL, NULL);
-	self->priv->cycles = (GeeList*) gee_array_list_new (VALA_TYPE_SOURCE_FILE_CYCLE, (GBoxedCopyFunc) vala_source_file_cycle_ref, vala_source_file_cycle_unref, g_direct_equal);
 	self->priv->packages = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_equal);
+	self->priv->defines = (GeeSet*) gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal);
 	self->priv->_codegen = vala_code_generator_new ();
 	self->ref_count = 1;
 }
@@ -1080,6 +1051,7 @@ static void vala_code_context_finalize (ValaCodeContext* obj) {
 	self = VALA_CODE_CONTEXT (obj);
 	self->priv->_library = (g_free (self->priv->_library), NULL);
 	self->priv->_header_filename = (g_free (self->priv->_header_filename), NULL);
+	self->priv->_internal_header_filename = (g_free (self->priv->_internal_header_filename), NULL);
 	self->priv->_output = (g_free (self->priv->_output), NULL);
 	self->priv->_basedir = (g_free (self->priv->_basedir), NULL);
 	self->priv->_directory = (g_free (self->priv->_directory), NULL);
@@ -1088,8 +1060,8 @@ static void vala_code_context_finalize (ValaCodeContext* obj) {
 	(self->priv->source_files == NULL) ? NULL : (self->priv->source_files = (gee_collection_object_unref (self->priv->source_files), NULL));
 	(self->priv->c_source_files == NULL) ? NULL : (self->priv->c_source_files = (gee_collection_object_unref (self->priv->c_source_files), NULL));
 	(self->priv->_root == NULL) ? NULL : (self->priv->_root = (vala_code_node_unref (self->priv->_root), NULL));
-	(self->priv->cycles == NULL) ? NULL : (self->priv->cycles = (gee_collection_object_unref (self->priv->cycles), NULL));
 	(self->priv->packages == NULL) ? NULL : (self->priv->packages = (gee_collection_object_unref (self->priv->packages), NULL));
+	(self->priv->defines == NULL) ? NULL : (self->priv->defines = (gee_collection_object_unref (self->priv->defines), NULL));
 	(self->priv->_codegen == NULL) ? NULL : (self->priv->_codegen = (vala_code_visitor_unref (self->priv->_codegen), NULL));
 }
 
