@@ -21,29 +21,282 @@
  *	Raffaele Sandrini <raffaele@sandrini.ch>
  */
 
-#include <vala/valadatatype.h>
-#include <gee/arraylist.h>
-#include <gee/collection.h>
-#include <gee/readonlylist.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <vala/valatypesymbol.h>
-#include <vala/valatypeparameter.h>
-#include <vala/valacodevisitor.h>
-#include <vala/valareport.h>
-#include <vala/valasourcereference.h>
-#include <vala/valascope.h>
-#include <vala/valasymbol.h>
-#include <vala/valagenerictype.h>
-#include <vala/valasemanticanalyzer.h>
-#include <vala/valastruct.h>
-#include <vala/valadelegatetype.h>
-#include <vala/valapointertype.h>
-#include <vala/valaarraytype.h>
-#include <vala/valaenum.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
+#include <gee.h>
 
 
+#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
+#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
+#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
+#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
+#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
 
+typedef struct _ValaCodeNode ValaCodeNode;
+typedef struct _ValaCodeNodeClass ValaCodeNodeClass;
+typedef struct _ValaCodeNodePrivate ValaCodeNodePrivate;
+
+#define VALA_TYPE_CODE_VISITOR (vala_code_visitor_get_type ())
+#define VALA_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitor))
+#define VALA_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))
+#define VALA_IS_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_VISITOR))
+#define VALA_IS_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_VISITOR))
+#define VALA_CODE_VISITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))
+
+typedef struct _ValaCodeVisitor ValaCodeVisitor;
+typedef struct _ValaCodeVisitorClass ValaCodeVisitorClass;
+
+#define VALA_TYPE_SEMANTIC_ANALYZER (vala_semantic_analyzer_get_type ())
+#define VALA_SEMANTIC_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzer))
+#define VALA_SEMANTIC_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerClass))
+#define VALA_IS_SEMANTIC_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SEMANTIC_ANALYZER))
+#define VALA_IS_SEMANTIC_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SEMANTIC_ANALYZER))
+#define VALA_SEMANTIC_ANALYZER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerClass))
+
+typedef struct _ValaSemanticAnalyzer ValaSemanticAnalyzer;
+typedef struct _ValaSemanticAnalyzerClass ValaSemanticAnalyzerClass;
+
+#define VALA_TYPE_DATA_TYPE (vala_data_type_get_type ())
+#define VALA_DATA_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DATA_TYPE, ValaDataType))
+#define VALA_DATA_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DATA_TYPE, ValaDataTypeClass))
+#define VALA_IS_DATA_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DATA_TYPE))
+#define VALA_IS_DATA_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DATA_TYPE))
+#define VALA_DATA_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DATA_TYPE, ValaDataTypeClass))
+
+typedef struct _ValaDataType ValaDataType;
+typedef struct _ValaDataTypeClass ValaDataTypeClass;
+
+#define VALA_TYPE_EXPRESSION (vala_expression_get_type ())
+#define VALA_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_EXPRESSION, ValaExpression))
+#define VALA_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_EXPRESSION, ValaExpressionClass))
+#define VALA_IS_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_EXPRESSION))
+#define VALA_IS_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_EXPRESSION))
+#define VALA_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_EXPRESSION, ValaExpressionClass))
+
+typedef struct _ValaExpression ValaExpression;
+typedef struct _ValaExpressionClass ValaExpressionClass;
+
+#define VALA_TYPE_SYMBOL (vala_symbol_get_type ())
+#define VALA_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SYMBOL, ValaSymbol))
+#define VALA_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SYMBOL, ValaSymbolClass))
+#define VALA_IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SYMBOL))
+#define VALA_IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SYMBOL))
+#define VALA_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SYMBOL, ValaSymbolClass))
+
+typedef struct _ValaSymbol ValaSymbol;
+typedef struct _ValaSymbolClass ValaSymbolClass;
+
+#define VALA_TYPE_LOCAL_VARIABLE (vala_local_variable_get_type ())
+#define VALA_LOCAL_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariable))
+#define VALA_LOCAL_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariableClass))
+#define VALA_IS_LOCAL_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_LOCAL_VARIABLE))
+#define VALA_IS_LOCAL_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_LOCAL_VARIABLE))
+#define VALA_LOCAL_VARIABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariableClass))
+
+typedef struct _ValaLocalVariable ValaLocalVariable;
+typedef struct _ValaLocalVariableClass ValaLocalVariableClass;
+
+#define VALA_TYPE_ATTRIBUTE (vala_attribute_get_type ())
+#define VALA_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ATTRIBUTE, ValaAttribute))
+#define VALA_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ATTRIBUTE, ValaAttributeClass))
+#define VALA_IS_ATTRIBUTE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ATTRIBUTE))
+#define VALA_IS_ATTRIBUTE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ATTRIBUTE))
+#define VALA_ATTRIBUTE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ATTRIBUTE, ValaAttributeClass))
+
+typedef struct _ValaAttribute ValaAttribute;
+typedef struct _ValaAttributeClass ValaAttributeClass;
+typedef struct _ValaDataTypePrivate ValaDataTypePrivate;
+
+#define VALA_TYPE_SCOPE (vala_scope_get_type ())
+#define VALA_SCOPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SCOPE, ValaScope))
+#define VALA_SCOPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SCOPE, ValaScopeClass))
+#define VALA_IS_SCOPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SCOPE))
+#define VALA_IS_SCOPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SCOPE))
+#define VALA_SCOPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SCOPE, ValaScopeClass))
+
+typedef struct _ValaScope ValaScope;
+typedef struct _ValaScopeClass ValaScopeClass;
+
+#define VALA_TYPE_FORMAL_PARAMETER (vala_formal_parameter_get_type ())
+#define VALA_FORMAL_PARAMETER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_FORMAL_PARAMETER, ValaFormalParameter))
+#define VALA_FORMAL_PARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_FORMAL_PARAMETER, ValaFormalParameterClass))
+#define VALA_IS_FORMAL_PARAMETER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_FORMAL_PARAMETER))
+#define VALA_IS_FORMAL_PARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_FORMAL_PARAMETER))
+#define VALA_FORMAL_PARAMETER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_FORMAL_PARAMETER, ValaFormalParameterClass))
+
+typedef struct _ValaFormalParameter ValaFormalParameter;
+typedef struct _ValaFormalParameterClass ValaFormalParameterClass;
+
+#define VALA_TYPE_TYPESYMBOL (vala_typesymbol_get_type ())
+#define VALA_TYPESYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPESYMBOL, ValaTypeSymbol))
+#define VALA_TYPESYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPESYMBOL, ValaTypeSymbolClass))
+#define VALA_IS_TYPESYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPESYMBOL))
+#define VALA_IS_TYPESYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPESYMBOL))
+#define VALA_TYPESYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPESYMBOL, ValaTypeSymbolClass))
+
+typedef struct _ValaTypeSymbol ValaTypeSymbol;
+typedef struct _ValaTypeSymbolClass ValaTypeSymbolClass;
+
+#define VALA_TYPE_TYPEPARAMETER (vala_typeparameter_get_type ())
+#define VALA_TYPEPARAMETER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEPARAMETER, ValaTypeParameter))
+#define VALA_TYPEPARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEPARAMETER, ValaTypeParameterClass))
+#define VALA_IS_TYPEPARAMETER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEPARAMETER))
+#define VALA_IS_TYPEPARAMETER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEPARAMETER))
+#define VALA_TYPEPARAMETER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEPARAMETER, ValaTypeParameterClass))
+
+typedef struct _ValaTypeParameter ValaTypeParameter;
+typedef struct _ValaTypeParameterClass ValaTypeParameterClass;
+
+#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
+#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
+#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+
+typedef struct _ValaSourceReference ValaSourceReference;
+typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
+
+#define VALA_TYPE_REFERENCE_TYPE (vala_reference_type_get_type ())
+#define VALA_REFERENCE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_REFERENCE_TYPE, ValaReferenceType))
+#define VALA_REFERENCE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_REFERENCE_TYPE, ValaReferenceTypeClass))
+#define VALA_IS_REFERENCE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_REFERENCE_TYPE))
+#define VALA_IS_REFERENCE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_REFERENCE_TYPE))
+#define VALA_REFERENCE_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_REFERENCE_TYPE, ValaReferenceTypeClass))
+
+typedef struct _ValaReferenceType ValaReferenceType;
+typedef struct _ValaReferenceTypeClass ValaReferenceTypeClass;
+
+#define VALA_TYPE_ARRAY_TYPE (vala_array_type_get_type ())
+#define VALA_ARRAY_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ARRAY_TYPE, ValaArrayType))
+#define VALA_ARRAY_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ARRAY_TYPE, ValaArrayTypeClass))
+#define VALA_IS_ARRAY_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ARRAY_TYPE))
+#define VALA_IS_ARRAY_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ARRAY_TYPE))
+#define VALA_ARRAY_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ARRAY_TYPE, ValaArrayTypeClass))
+
+typedef struct _ValaArrayType ValaArrayType;
+typedef struct _ValaArrayTypeClass ValaArrayTypeClass;
+
+#define VALA_TYPE_DELEGATE_TYPE (vala_delegate_type_get_type ())
+#define VALA_DELEGATE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DELEGATE_TYPE, ValaDelegateType))
+#define VALA_DELEGATE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DELEGATE_TYPE, ValaDelegateTypeClass))
+#define VALA_IS_DELEGATE_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DELEGATE_TYPE))
+#define VALA_IS_DELEGATE_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DELEGATE_TYPE))
+#define VALA_DELEGATE_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DELEGATE_TYPE, ValaDelegateTypeClass))
+
+typedef struct _ValaDelegateType ValaDelegateType;
+typedef struct _ValaDelegateTypeClass ValaDelegateTypeClass;
+
+#define VALA_TYPE_DELEGATE (vala_delegate_get_type ())
+#define VALA_DELEGATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_DELEGATE, ValaDelegate))
+#define VALA_DELEGATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_DELEGATE, ValaDelegateClass))
+#define VALA_IS_DELEGATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_DELEGATE))
+#define VALA_IS_DELEGATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_DELEGATE))
+#define VALA_DELEGATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_DELEGATE, ValaDelegateClass))
+
+typedef struct _ValaDelegate ValaDelegate;
+typedef struct _ValaDelegateClass ValaDelegateClass;
+
+#define VALA_TYPE_POINTER_TYPE (vala_pointer_type_get_type ())
+#define VALA_POINTER_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_POINTER_TYPE, ValaPointerType))
+#define VALA_POINTER_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_POINTER_TYPE, ValaPointerTypeClass))
+#define VALA_IS_POINTER_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_POINTER_TYPE))
+#define VALA_IS_POINTER_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_POINTER_TYPE))
+#define VALA_POINTER_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_POINTER_TYPE, ValaPointerTypeClass))
+
+typedef struct _ValaPointerType ValaPointerType;
+typedef struct _ValaPointerTypeClass ValaPointerTypeClass;
+
+#define VALA_TYPE_ENUM (vala_enum_get_type ())
+#define VALA_ENUM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_ENUM, ValaEnum))
+#define VALA_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_ENUM, ValaEnumClass))
+#define VALA_IS_ENUM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_ENUM))
+#define VALA_IS_ENUM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_ENUM))
+#define VALA_ENUM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_ENUM, ValaEnumClass))
+
+typedef struct _ValaEnum ValaEnum;
+typedef struct _ValaEnumClass ValaEnumClass;
+
+#define VALA_TYPE_STRUCT (vala_struct_get_type ())
+#define VALA_STRUCT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_STRUCT, ValaStruct))
+#define VALA_STRUCT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_STRUCT, ValaStructClass))
+#define VALA_IS_STRUCT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_STRUCT))
+#define VALA_IS_STRUCT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_STRUCT))
+#define VALA_STRUCT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_STRUCT, ValaStructClass))
+
+typedef struct _ValaStruct ValaStruct;
+typedef struct _ValaStructClass ValaStructClass;
+
+#define VALA_TYPE_GENERIC_TYPE (vala_generic_type_get_type ())
+#define VALA_GENERIC_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GENERIC_TYPE, ValaGenericType))
+#define VALA_GENERIC_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GENERIC_TYPE, ValaGenericTypeClass))
+#define VALA_IS_GENERIC_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GENERIC_TYPE))
+#define VALA_IS_GENERIC_TYPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GENERIC_TYPE))
+#define VALA_GENERIC_TYPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GENERIC_TYPE, ValaGenericTypeClass))
+
+typedef struct _ValaGenericType ValaGenericType;
+typedef struct _ValaGenericTypeClass ValaGenericTypeClass;
+
+/**
+ * Represents a part of the parsed source code.
+ *
+ * Code nodes get created by the parser and are used throughout the whole
+ * compilation process.
+ */
+struct _ValaCodeNode {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaCodeNodePrivate * priv;
+	GList* attributes;
+};
+
+struct _ValaCodeNodeClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaCodeNode *self);
+	void (*accept) (ValaCodeNode* self, ValaCodeVisitor* visitor);
+	void (*accept_children) (ValaCodeNode* self, ValaCodeVisitor* visitor);
+	gboolean (*check) (ValaCodeNode* self, ValaSemanticAnalyzer* analyzer);
+	void (*replace_type) (ValaCodeNode* self, ValaDataType* old_type, ValaDataType* new_type);
+	void (*replace_expression) (ValaCodeNode* self, ValaExpression* old_node, ValaExpression* new_node);
+	char* (*to_string) (ValaCodeNode* self);
+	void (*get_defined_variables) (ValaCodeNode* self, GeeCollection* collection);
+	void (*get_used_variables) (ValaCodeNode* self, GeeCollection* collection);
+};
+
+/**
+ * A reference to a data type. This is used to specify static types of
+ * expressions.
+ */
+struct _ValaDataType {
+	ValaCodeNode parent_instance;
+	ValaDataTypePrivate * priv;
+};
+
+struct _ValaDataTypeClass {
+	ValaCodeNodeClass parent_class;
+	char* (*get_cname) (ValaDataType* self);
+	char* (*get_lower_case_cname) (ValaDataType* self, const char* infix);
+	char* (*to_qualified_string) (ValaDataType* self, ValaScope* scope);
+	ValaDataType* (*copy) (ValaDataType* self);
+	gboolean (*equals) (ValaDataType* self, ValaDataType* type2);
+	gboolean (*stricter) (ValaDataType* self, ValaDataType* type2);
+	gboolean (*compatible) (ValaDataType* self, ValaDataType* target_type);
+	gboolean (*is_invokable) (ValaDataType* self);
+	ValaDataType* (*get_return_type) (ValaDataType* self);
+	GeeList* (*get_parameters) (ValaDataType* self);
+	gboolean (*is_reference_type_or_type_parameter) (ValaDataType* self);
+	gboolean (*is_array) (ValaDataType* self);
+	GeeList* (*get_symbols) (ValaDataType* self);
+	ValaSymbol* (*get_member) (ValaDataType* self, const char* member_name);
+	ValaSymbol* (*get_pointer_member) (ValaDataType* self, const char* member_name);
+	gboolean (*is_real_struct_type) (ValaDataType* self);
+	char* (*get_type_id) (ValaDataType* self);
+	char* (*get_type_signature) (ValaDataType* self);
+	gboolean (*is_disposable) (ValaDataType* self);
+};
 
 struct _ValaDataTypePrivate {
 	gboolean _value_owned;
@@ -55,33 +308,138 @@ struct _ValaDataTypePrivate {
 	GeeList* type_argument_list;
 };
 
+
+
+gpointer vala_code_node_ref (gpointer instance);
+void vala_code_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_node (const GValue* value);
+GType vala_code_node_get_type (void);
+gpointer vala_code_visitor_ref (gpointer instance);
+void vala_code_visitor_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_visitor (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_visitor (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_visitor (const GValue* value);
+GType vala_code_visitor_get_type (void);
+GType vala_semantic_analyzer_get_type (void);
+GType vala_data_type_get_type (void);
+GType vala_expression_get_type (void);
+GType vala_symbol_get_type (void);
+GType vala_local_variable_get_type (void);
+GType vala_attribute_get_type (void);
+gpointer vala_scope_ref (gpointer instance);
+void vala_scope_unref (gpointer instance);
+GParamSpec* vala_param_spec_scope (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_scope (GValue* value, gpointer v_object);
+gpointer vala_value_get_scope (const GValue* value);
+GType vala_scope_get_type (void);
+GType vala_formal_parameter_get_type (void);
+GType vala_typesymbol_get_type (void);
+GType vala_typeparameter_get_type (void);
 #define VALA_DATA_TYPE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_DATA_TYPE, ValaDataTypePrivate))
 enum  {
 	VALA_DATA_TYPE_DUMMY_PROPERTY
 };
+static GeeList* vala_data_type__empty_type_list;
 static GeeList* vala_data_type__empty_type_list = NULL;
+void vala_code_node_set_parent_node (ValaCodeNode* self, ValaCodeNode* value);
+void vala_data_type_add_type_argument (ValaDataType* self, ValaDataType* arg);
+GeeList* vala_data_type_get_type_arguments (ValaDataType* self);
+void vala_data_type_remove_all_type_arguments (ValaDataType* self);
+void vala_code_node_accept (ValaCodeNode* self, ValaCodeVisitor* visitor);
+void vala_code_visitor_visit_data_type (ValaCodeVisitor* self, ValaDataType* type);
 static void vala_data_type_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
+gpointer vala_source_reference_ref (gpointer instance);
+void vala_source_reference_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_reference (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_reference (const GValue* value);
+GType vala_source_reference_get_type (void);
+void vala_report_error (ValaSourceReference* source, const char* message);
+ValaSourceReference* vala_code_node_get_source_reference (ValaCodeNode* self);
+char* vala_data_type_get_cname (ValaDataType* self);
 static char* vala_data_type_real_get_cname (ValaDataType* self);
+GType vala_reference_type_get_type (void);
+GType vala_array_type_get_type (void);
+ValaDataType* vala_array_type_get_element_type (ValaArrayType* self);
+ValaTypeSymbol* vala_data_type_get_data_type (ValaDataType* self);
+gboolean vala_typesymbol_is_reference_type (ValaTypeSymbol* self);
+char* vala_typesymbol_get_cname (ValaTypeSymbol* self, gboolean const_type);
+char* vala_data_type_get_const_cname (ValaDataType* self);
+char* vala_symbol_get_lower_case_cname (ValaSymbol* self, const char* infix);
+char* vala_data_type_get_lower_case_cname (ValaDataType* self, const char* infix);
 static char* vala_data_type_real_get_lower_case_cname (ValaDataType* self, const char* infix);
+char* vala_data_type_to_qualified_string (ValaDataType* self, ValaScope* scope);
 static char* vala_data_type_real_to_string (ValaCodeNode* base);
+ValaSymbol* vala_symbol_get_parent_symbol (ValaSymbol* self);
+const char* vala_symbol_get_name (ValaSymbol* self);
+ValaSymbol* vala_scope_lookup (ValaScope* self, const char* name);
+ValaScope* vala_scope_get_parent_scope (ValaScope* self);
+char* vala_symbol_get_full_name (ValaSymbol* self);
+gboolean vala_data_type_get_value_owned (ValaDataType* self);
+gboolean vala_data_type_get_nullable (ValaDataType* self);
 static char* vala_data_type_real_to_qualified_string (ValaDataType* self, ValaScope* scope);
+ValaDataType* vala_data_type_copy (ValaDataType* self);
 static ValaDataType* vala_data_type_real_copy (ValaDataType* self);
+ValaTypeParameter* vala_data_type_get_type_parameter (ValaDataType* self);
+gboolean vala_typeparameter_equals (ValaTypeParameter* self, ValaTypeParameter* param2);
+gboolean vala_data_type_get_floating_reference (ValaDataType* self);
+gboolean vala_data_type_equals (ValaDataType* self, ValaDataType* type2);
 static gboolean vala_data_type_real_equals (ValaDataType* self, ValaDataType* type2);
+gboolean vala_data_type_stricter (ValaDataType* self, ValaDataType* type2);
 static gboolean vala_data_type_real_stricter (ValaDataType* self, ValaDataType* type2);
 static void vala_data_type_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
+GType vala_delegate_type_get_type (void);
+GType vala_delegate_get_type (void);
+ValaDelegate* vala_delegate_type_get_delegate_symbol (ValaDelegateType* self);
+GType vala_pointer_type_get_type (void);
+GType vala_enum_get_type (void);
+GType vala_struct_get_type (void);
+gboolean vala_struct_is_integer_type (ValaStruct* self);
+gboolean vala_struct_is_floating_type (ValaStruct* self);
+gint vala_struct_get_rank (ValaStruct* self);
+gboolean vala_typesymbol_is_subtype_of (ValaTypeSymbol* self, ValaTypeSymbol* t);
+gboolean vala_data_type_compatible (ValaDataType* self, ValaDataType* target_type);
 static gboolean vala_data_type_real_compatible (ValaDataType* self, ValaDataType* target_type);
+gboolean vala_data_type_is_invokable (ValaDataType* self);
 static gboolean vala_data_type_real_is_invokable (ValaDataType* self);
+ValaDataType* vala_data_type_get_return_type (ValaDataType* self);
 static ValaDataType* vala_data_type_real_get_return_type (ValaDataType* self);
+GeeList* vala_data_type_get_parameters (ValaDataType* self);
 static GeeList* vala_data_type_real_get_parameters (ValaDataType* self);
+gboolean vala_data_type_is_reference_type_or_type_parameter (ValaDataType* self);
 static gboolean vala_data_type_real_is_reference_type_or_type_parameter (ValaDataType* self);
+gboolean vala_data_type_is_array (ValaDataType* self);
 static gboolean vala_data_type_real_is_array (ValaDataType* self);
+GeeList* vala_data_type_get_symbols (ValaDataType* self);
 static GeeList* vala_data_type_real_get_symbols (ValaDataType* self);
+ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSymbol* sym, const char* name);
+ValaSymbol* vala_data_type_get_member (ValaDataType* self, const char* member_name);
 static ValaSymbol* vala_data_type_real_get_member (ValaDataType* self, const char* member_name);
+ValaSymbol* vala_data_type_get_pointer_member (ValaDataType* self, const char* member_name);
 static ValaSymbol* vala_data_type_real_get_pointer_member (ValaDataType* self, const char* member_name);
+gboolean vala_struct_is_simple_type (ValaStruct* self);
+gboolean vala_data_type_is_real_struct_type (ValaDataType* self);
 static gboolean vala_data_type_real_is_real_struct_type (ValaDataType* self);
+char* vala_typesymbol_get_type_id (ValaTypeSymbol* self);
+char* vala_data_type_get_type_id (ValaDataType* self);
 static char* vala_data_type_real_get_type_id (ValaDataType* self);
+char* vala_typesymbol_get_type_signature (ValaTypeSymbol* self);
+char* vala_data_type_get_type_signature (ValaDataType* self);
 static char* vala_data_type_real_get_type_signature (ValaDataType* self);
+gboolean vala_data_type_is_disposable (ValaDataType* self);
 static gboolean vala_data_type_real_is_disposable (ValaDataType* self);
+GType vala_generic_type_get_type (void);
+ValaDataType* vala_semantic_analyzer_get_actual_type (ValaDataType* derived_instance_type, ValaGenericType* generic_type, ValaCodeNode* node_reference);
+ValaDataType* vala_data_type_get_actual_type (ValaDataType* self, ValaDataType* derived_instance_type, ValaCodeNode* node_reference);
+void vala_data_type_set_value_owned (ValaDataType* self, gboolean value);
+void vala_data_type_set_nullable (ValaDataType* self, gboolean value);
+void vala_data_type_set_data_type (ValaDataType* self, ValaTypeSymbol* value);
+void vala_data_type_set_type_parameter (ValaDataType* self, ValaTypeParameter* value);
+void vala_data_type_set_floating_reference (ValaDataType* self, gboolean value);
+gboolean vala_data_type_get_is_dynamic (ValaDataType* self);
+void vala_data_type_set_is_dynamic (ValaDataType* self, gboolean value);
 static gpointer vala_data_type_parent_class = NULL;
 static void vala_data_type_finalize (ValaCodeNode* obj);
 
