@@ -20,25 +20,101 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vala/valaphifunction.h>
-#include <gee/arraylist.h>
-#include <gee/collection.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <gee.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_PHI_FUNCTION (vala_phi_function_get_type ())
+#define VALA_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunction))
+#define VALA_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))
+#define VALA_IS_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PHI_FUNCTION))
+#define VALA_IS_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PHI_FUNCTION))
+#define VALA_PHI_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))
 
+typedef struct _ValaPhiFunction ValaPhiFunction;
+typedef struct _ValaPhiFunctionClass ValaPhiFunctionClass;
+typedef struct _ValaPhiFunctionPrivate ValaPhiFunctionPrivate;
+
+#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
+#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
+#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
+#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
+#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+
+typedef struct _ValaCodeNode ValaCodeNode;
+typedef struct _ValaCodeNodeClass ValaCodeNodeClass;
+
+#define VALA_TYPE_SYMBOL (vala_symbol_get_type ())
+#define VALA_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SYMBOL, ValaSymbol))
+#define VALA_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SYMBOL, ValaSymbolClass))
+#define VALA_IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SYMBOL))
+#define VALA_IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SYMBOL))
+#define VALA_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SYMBOL, ValaSymbolClass))
+
+typedef struct _ValaSymbol ValaSymbol;
+typedef struct _ValaSymbolClass ValaSymbolClass;
+
+#define VALA_TYPE_LOCAL_VARIABLE (vala_local_variable_get_type ())
+#define VALA_LOCAL_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariable))
+#define VALA_LOCAL_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariableClass))
+#define VALA_IS_LOCAL_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_LOCAL_VARIABLE))
+#define VALA_IS_LOCAL_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_LOCAL_VARIABLE))
+#define VALA_LOCAL_VARIABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_LOCAL_VARIABLE, ValaLocalVariableClass))
+
+typedef struct _ValaLocalVariable ValaLocalVariable;
+typedef struct _ValaLocalVariableClass ValaLocalVariableClass;
+typedef struct _ValaParamSpecPhiFunction ValaParamSpecPhiFunction;
+
+struct _ValaPhiFunction {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaPhiFunctionPrivate * priv;
+};
+
+struct _ValaPhiFunctionClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaPhiFunction *self);
+};
 
 struct _ValaPhiFunctionPrivate {
 	ValaLocalVariable* _original_variable;
 	GeeList* _operands;
 };
 
+struct _ValaParamSpecPhiFunction {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_phi_function_ref (gpointer instance);
+void vala_phi_function_unref (gpointer instance);
+GParamSpec* vala_param_spec_phi_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_phi_function (GValue* value, gpointer v_object);
+gpointer vala_value_get_phi_function (const GValue* value);
+GType vala_phi_function_get_type (void);
+gpointer vala_code_node_ref (gpointer instance);
+void vala_code_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_node (const GValue* value);
+GType vala_code_node_get_type (void);
+GType vala_symbol_get_type (void);
+GType vala_local_variable_get_type (void);
 #define VALA_PHI_FUNCTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionPrivate))
 enum  {
 	VALA_PHI_FUNCTION_DUMMY_PROPERTY
 };
 static void vala_phi_function_set_original_variable (ValaPhiFunction* self, ValaLocalVariable* value);
 static void vala_phi_function_set_operands (ValaPhiFunction* self, GeeList* value);
+GeeList* vala_phi_function_get_operands (ValaPhiFunction* self);
+ValaPhiFunction* vala_phi_function_new (ValaLocalVariable* variable, gint num_of_ops);
+ValaPhiFunction* vala_phi_function_construct (GType object_type, ValaLocalVariable* variable, gint num_of_ops);
+ValaPhiFunction* vala_phi_function_new (ValaLocalVariable* variable, gint num_of_ops);
+ValaLocalVariable* vala_phi_function_get_original_variable (ValaPhiFunction* self);
 static gpointer vala_phi_function_parent_class = NULL;
 static void vala_phi_function_finalize (ValaPhiFunction* obj);
 
