@@ -20,13 +20,80 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vala/valascanner.h>
-#include <vala/valareport.h>
-#include <vala/valasourcereference.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_SCANNER (vala_scanner_get_type ())
+#define VALA_SCANNER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SCANNER, ValaScanner))
+#define VALA_SCANNER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SCANNER, ValaScannerClass))
+#define VALA_IS_SCANNER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SCANNER))
+#define VALA_IS_SCANNER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SCANNER))
+#define VALA_SCANNER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SCANNER, ValaScannerClass))
 
+typedef struct _ValaScanner ValaScanner;
+typedef struct _ValaScannerClass ValaScannerClass;
+typedef struct _ValaScannerPrivate ValaScannerPrivate;
+
+#define VALA_TYPE_SOURCE_FILE (vala_source_file_get_type ())
+#define VALA_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFile))
+#define VALA_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
+#define VALA_IS_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_FILE))
+#define VALA_IS_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_FILE))
+#define VALA_SOURCE_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
+
+typedef struct _ValaSourceFile ValaSourceFile;
+typedef struct _ValaSourceFileClass ValaSourceFileClass;
+typedef struct _ValaScannerConditional ValaScannerConditional;
+
+#define VALA_TYPE_TOKEN_TYPE (vala_token_type_get_type ())
+
+#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
+#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
+#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+
+typedef struct _ValaSourceReference ValaSourceReference;
+typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
+typedef struct _ValaSourceLocation ValaSourceLocation;
+
+#define VALA_TYPE_CODE_CONTEXT (vala_code_context_get_type ())
+#define VALA_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContext))
+#define VALA_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))
+#define VALA_IS_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_CONTEXT))
+#define VALA_IS_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_CONTEXT))
+#define VALA_CODE_CONTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))
+
+typedef struct _ValaCodeContext ValaCodeContext;
+typedef struct _ValaCodeContextClass ValaCodeContextClass;
+
+#define VALA_SCANNER_TYPE_CONDITIONAL (vala_scanner_conditional_get_type ())
+typedef struct _ValaParamSpecScanner ValaParamSpecScanner;
+
+/**
+ * Lexical scanner for Vala source files.
+ */
+struct _ValaScanner {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaScannerPrivate * priv;
+};
+
+struct _ValaScannerClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaScanner *self);
+};
+
+struct _ValaScannerConditional {
+	gboolean matched;
+	gboolean else_found;
+	gboolean skip_section;
+};
 
 struct _ValaScannerPrivate {
 	ValaSourceFile* _source_file;
@@ -35,23 +102,224 @@ struct _ValaScannerPrivate {
 	gint line;
 	gint column;
 	char* _comment;
+	ValaScannerConditional* conditional_stack;
+	gint conditional_stack_length1;
+	gint conditional_stack_size;
 };
 
+typedef enum  {
+	VALA_TOKEN_TYPE_NONE,
+	VALA_TOKEN_TYPE_ABSTRACT,
+	VALA_TOKEN_TYPE_AS,
+	VALA_TOKEN_TYPE_ASSIGN,
+	VALA_TOKEN_TYPE_ASSIGN_ADD,
+	VALA_TOKEN_TYPE_ASSIGN_BITWISE_AND,
+	VALA_TOKEN_TYPE_ASSIGN_BITWISE_OR,
+	VALA_TOKEN_TYPE_ASSIGN_BITWISE_XOR,
+	VALA_TOKEN_TYPE_ASSIGN_DIV,
+	VALA_TOKEN_TYPE_ASSIGN_MUL,
+	VALA_TOKEN_TYPE_ASSIGN_PERCENT,
+	VALA_TOKEN_TYPE_ASSIGN_SHIFT_LEFT,
+	VALA_TOKEN_TYPE_ASSIGN_SUB,
+	VALA_TOKEN_TYPE_BASE,
+	VALA_TOKEN_TYPE_BITWISE_AND,
+	VALA_TOKEN_TYPE_BITWISE_OR,
+	VALA_TOKEN_TYPE_BREAK,
+	VALA_TOKEN_TYPE_CARRET,
+	VALA_TOKEN_TYPE_CASE,
+	VALA_TOKEN_TYPE_CATCH,
+	VALA_TOKEN_TYPE_CHARACTER_LITERAL,
+	VALA_TOKEN_TYPE_CLASS,
+	VALA_TOKEN_TYPE_CLOSE_BRACE,
+	VALA_TOKEN_TYPE_CLOSE_BRACKET,
+	VALA_TOKEN_TYPE_CLOSE_PARENS,
+	VALA_TOKEN_TYPE_COLON,
+	VALA_TOKEN_TYPE_COMMA,
+	VALA_TOKEN_TYPE_CONST,
+	VALA_TOKEN_TYPE_CONSTRUCT,
+	VALA_TOKEN_TYPE_CONTINUE,
+	VALA_TOKEN_TYPE_DEFAULT,
+	VALA_TOKEN_TYPE_DELEGATE,
+	VALA_TOKEN_TYPE_DELETE,
+	VALA_TOKEN_TYPE_DIV,
+	VALA_TOKEN_TYPE_DO,
+	VALA_TOKEN_TYPE_DOUBLE_COLON,
+	VALA_TOKEN_TYPE_DOT,
+	VALA_TOKEN_TYPE_DYNAMIC,
+	VALA_TOKEN_TYPE_ELLIPSIS,
+	VALA_TOKEN_TYPE_ELSE,
+	VALA_TOKEN_TYPE_ENUM,
+	VALA_TOKEN_TYPE_ENSURES,
+	VALA_TOKEN_TYPE_ERRORDOMAIN,
+	VALA_TOKEN_TYPE_EOF,
+	VALA_TOKEN_TYPE_EXTERN,
+	VALA_TOKEN_TYPE_FALSE,
+	VALA_TOKEN_TYPE_FINALLY,
+	VALA_TOKEN_TYPE_FOR,
+	VALA_TOKEN_TYPE_FOREACH,
+	VALA_TOKEN_TYPE_GET,
+	VALA_TOKEN_TYPE_HASH,
+	VALA_TOKEN_TYPE_IDENTIFIER,
+	VALA_TOKEN_TYPE_IF,
+	VALA_TOKEN_TYPE_IN,
+	VALA_TOKEN_TYPE_INLINE,
+	VALA_TOKEN_TYPE_INTEGER_LITERAL,
+	VALA_TOKEN_TYPE_INTERFACE,
+	VALA_TOKEN_TYPE_INTERNAL,
+	VALA_TOKEN_TYPE_INTERR,
+	VALA_TOKEN_TYPE_IS,
+	VALA_TOKEN_TYPE_LAMBDA,
+	VALA_TOKEN_TYPE_LOCK,
+	VALA_TOKEN_TYPE_MINUS,
+	VALA_TOKEN_TYPE_NAMESPACE,
+	VALA_TOKEN_TYPE_NEW,
+	VALA_TOKEN_TYPE_NULL,
+	VALA_TOKEN_TYPE_OUT,
+	VALA_TOKEN_TYPE_OP_AND,
+	VALA_TOKEN_TYPE_OP_DEC,
+	VALA_TOKEN_TYPE_OP_EQ,
+	VALA_TOKEN_TYPE_OP_GE,
+	VALA_TOKEN_TYPE_OP_GT,
+	VALA_TOKEN_TYPE_OP_INC,
+	VALA_TOKEN_TYPE_OP_LE,
+	VALA_TOKEN_TYPE_OP_LT,
+	VALA_TOKEN_TYPE_OP_NE,
+	VALA_TOKEN_TYPE_OP_NEG,
+	VALA_TOKEN_TYPE_OP_OR,
+	VALA_TOKEN_TYPE_OP_PTR,
+	VALA_TOKEN_TYPE_OP_SHIFT_LEFT,
+	VALA_TOKEN_TYPE_OPEN_BRACE,
+	VALA_TOKEN_TYPE_OPEN_BRACKET,
+	VALA_TOKEN_TYPE_OPEN_PARENS,
+	VALA_TOKEN_TYPE_OVERRIDE,
+	VALA_TOKEN_TYPE_OWNED,
+	VALA_TOKEN_TYPE_PARAMS,
+	VALA_TOKEN_TYPE_PERCENT,
+	VALA_TOKEN_TYPE_PLUS,
+	VALA_TOKEN_TYPE_PRIVATE,
+	VALA_TOKEN_TYPE_PROTECTED,
+	VALA_TOKEN_TYPE_PUBLIC,
+	VALA_TOKEN_TYPE_REAL_LITERAL,
+	VALA_TOKEN_TYPE_REF,
+	VALA_TOKEN_TYPE_REQUIRES,
+	VALA_TOKEN_TYPE_RETURN,
+	VALA_TOKEN_TYPE_SEMICOLON,
+	VALA_TOKEN_TYPE_SET,
+	VALA_TOKEN_TYPE_SIGNAL,
+	VALA_TOKEN_TYPE_SIZEOF,
+	VALA_TOKEN_TYPE_STAR,
+	VALA_TOKEN_TYPE_STATIC,
+	VALA_TOKEN_TYPE_STRING_LITERAL,
+	VALA_TOKEN_TYPE_STRUCT,
+	VALA_TOKEN_TYPE_SWITCH,
+	VALA_TOKEN_TYPE_THIS,
+	VALA_TOKEN_TYPE_THROW,
+	VALA_TOKEN_TYPE_THROWS,
+	VALA_TOKEN_TYPE_TILDE,
+	VALA_TOKEN_TYPE_TRUE,
+	VALA_TOKEN_TYPE_TRY,
+	VALA_TOKEN_TYPE_TYPEOF,
+	VALA_TOKEN_TYPE_UNOWNED,
+	VALA_TOKEN_TYPE_USING,
+	VALA_TOKEN_TYPE_VAR,
+	VALA_TOKEN_TYPE_VERBATIM_STRING_LITERAL,
+	VALA_TOKEN_TYPE_VIRTUAL,
+	VALA_TOKEN_TYPE_VOID,
+	VALA_TOKEN_TYPE_VOLATILE,
+	VALA_TOKEN_TYPE_WEAK,
+	VALA_TOKEN_TYPE_WHILE,
+	VALA_TOKEN_TYPE_YIELD,
+	VALA_TOKEN_TYPE_YIELDS
+} ValaTokenType;
+
+/**
+ * Represents a position in a source file.
+ */
+struct _ValaSourceLocation {
+	gchar* pos;
+	gint line;
+	gint column;
+};
+
+struct _ValaParamSpecScanner {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_scanner_ref (gpointer instance);
+void vala_scanner_unref (gpointer instance);
+GParamSpec* vala_param_spec_scanner (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_scanner (GValue* value, gpointer v_object);
+gpointer vala_value_get_scanner (const GValue* value);
+GType vala_scanner_get_type (void);
+gpointer vala_source_file_ref (gpointer instance);
+void vala_source_file_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_file (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_file (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_file (const GValue* value);
+GType vala_source_file_get_type (void);
 #define VALA_SCANNER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SCANNER, ValaScannerPrivate))
 enum  {
 	VALA_SCANNER_DUMMY_PROPERTY
 };
+static void vala_scanner_set_source_file (ValaScanner* self, ValaSourceFile* value);
+gchar* vala_source_file_get_mapped_contents (ValaSourceFile* self);
+gsize vala_source_file_get_mapped_length (ValaSourceFile* self);
+ValaScanner* vala_scanner_new (ValaSourceFile* source_file);
+ValaScanner* vala_scanner_construct (GType object_type, ValaSourceFile* source_file);
+ValaScanner* vala_scanner_new (ValaSourceFile* source_file);
 static gboolean vala_scanner_is_ident_char (ValaScanner* self, gchar c);
+static gboolean vala_scanner_matches (ValaScanner* self, gchar* begin, const char* keyword);
+GType vala_token_type_get_type (void);
 static ValaTokenType vala_scanner_get_identifier_or_keyword (ValaScanner* self, gchar* begin, gint len);
 static ValaTokenType vala_scanner_read_number (ValaScanner* self);
-static gboolean vala_scanner_matches (ValaScanner* self, gchar* begin, const char* keyword);
-static gboolean vala_scanner_whitespace (ValaScanner* self);
-static gboolean vala_scanner_comment (ValaScanner* self);
 static void vala_scanner_space (ValaScanner* self);
+gpointer vala_source_reference_ref (gpointer instance);
+void vala_source_reference_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_reference (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_reference (const GValue* value);
+GType vala_source_reference_get_type (void);
+void vala_report_error (ValaSourceReference* source, const char* message);
+ValaSourceFile* vala_scanner_get_source_file (ValaScanner* self);
+ValaSourceReference* vala_source_reference_new (ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column);
+ValaSourceReference* vala_source_reference_construct (GType object_type, ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column);
+ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* token_begin, ValaSourceLocation* token_end);
+static gboolean vala_scanner_pp_whitespace (ValaScanner* self);
+static void vala_scanner_parse_pp_if (ValaScanner* self);
+static void vala_scanner_parse_pp_elif (ValaScanner* self);
+static void vala_scanner_parse_pp_else (ValaScanner* self);
+static void vala_scanner_parse_pp_endif (ValaScanner* self);
+static void vala_scanner_pp_directive (ValaScanner* self);
+static void vala_scanner_pp_eol (ValaScanner* self);
+static gboolean vala_scanner_parse_pp_expression (ValaScanner* self);
+static void _vala_array_add1 (ValaScannerConditional** array, int* length, int* size, const ValaScannerConditional* value);
+gpointer vala_code_context_ref (gpointer instance);
+void vala_code_context_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_context (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_context (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_context (const GValue* value);
+GType vala_code_context_get_type (void);
+ValaCodeContext* vala_source_file_get_context (ValaSourceFile* self);
+gboolean vala_code_context_is_defined (ValaCodeContext* self, const char* define);
+static gboolean vala_scanner_parse_pp_symbol (ValaScanner* self);
+static gboolean vala_scanner_parse_pp_primary_expression (ValaScanner* self);
+static gboolean vala_scanner_parse_pp_unary_expression (ValaScanner* self);
+static gboolean vala_scanner_parse_pp_equality_expression (ValaScanner* self);
+static gboolean vala_scanner_parse_pp_and_expression (ValaScanner* self);
+static gboolean vala_scanner_parse_pp_or_expression (ValaScanner* self);
+static gboolean vala_scanner_whitespace (ValaScanner* self);
 static void vala_scanner_push_comment (ValaScanner* self, const char* comment_item, gboolean file_comment);
-static void vala_scanner_set_source_file (ValaScanner* self, ValaSourceFile* value);
+static gboolean vala_scanner_comment (ValaScanner* self);
+void vala_source_file_set_comment (ValaSourceFile* self, const char* value);
+char* vala_scanner_pop_comment (ValaScanner* self);
+static ValaScannerConditional* vala_scanner_conditional_dup (const ValaScannerConditional* self);
+static void vala_scanner_conditional_free (ValaScannerConditional* self);
+static GType vala_scanner_conditional_get_type (void);
 static gpointer vala_scanner_parent_class = NULL;
 static void vala_scanner_finalize (ValaScanner* obj);
+static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
 
@@ -84,7 +352,7 @@ static gboolean vala_scanner_is_ident_char (ValaScanner* self, gchar c) {
 	} else {
 		_tmp0 = c == '_';
 	}
-	return (_tmp0);
+	return _tmp0;
 }
 
 
@@ -701,7 +969,7 @@ static ValaTokenType vala_scanner_read_number (ValaScanner* self) {
 	/* integer part*/
 	if (_tmp0) {
 		/* hexadecimal integer literal*/
-		self->priv->current = self->priv->current + (2);
+		self->priv->current = self->priv->current + 2;
 		while (TRUE) {
 			gboolean _tmp3;
 			_tmp3 = FALSE;
@@ -780,7 +1048,7 @@ static ValaTokenType vala_scanner_read_number (ValaScanner* self) {
 			} else {
 				_tmp10 = self->priv->current[0] == '-';
 			}
-			_tmp9 = (_tmp10);
+			_tmp9 = _tmp10;
 		} else {
 			_tmp9 = FALSE;
 		}
@@ -805,7 +1073,7 @@ static ValaTokenType vala_scanner_read_number (ValaScanner* self) {
 	if (self->priv->current < self->priv->end) {
 		gboolean real_literal;
 		gboolean _tmp15;
-		real_literal = (type == VALA_TOKEN_TYPE_REAL_LITERAL);
+		real_literal = type == VALA_TOKEN_TYPE_REAL_LITERAL;
 		switch (self->priv->current[0]) {
 			case 'l':
 			case 'L':
@@ -1007,7 +1275,7 @@ ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* to
 								}
 								if (_tmp3) {
 									type = VALA_TOKEN_TYPE_ELLIPSIS;
-									self->priv->current = self->priv->current + (2);
+									self->priv->current = self->priv->current + 2;
 								}
 							}
 							break;
@@ -1336,7 +1604,7 @@ ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* to
 									gboolean _tmp18;
 									type = VALA_TOKEN_TYPE_VERBATIM_STRING_LITERAL;
 									token_length_in_chars = 6;
-									self->priv->current = self->priv->current + (3);
+									self->priv->current = self->priv->current + 3;
 									while (self->priv->current < (self->priv->end - 4)) {
 										gboolean _tmp14;
 										gboolean _tmp15;
@@ -1362,9 +1630,9 @@ ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* to
 												token_length_in_chars = 3;
 											} else {
 												gunichar u;
-												u = g_utf8_get_char_validated (((const char*) self->priv->current), (glong) (self->priv->end - self->priv->current));
+												u = g_utf8_get_char_validated ((const char*) self->priv->current, (glong) (self->priv->end - self->priv->current));
 												if (u != ((gunichar) (-1))) {
-													self->priv->current = self->priv->current + (g_unichar_to_utf8 (u, NULL));
+													self->priv->current = self->priv->current + g_unichar_to_utf8 (u, NULL);
 													token_length_in_chars++;
 												} else {
 													ValaSourceReference* _tmp16;
@@ -1388,7 +1656,7 @@ ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* to
 										_tmp17 = FALSE;
 									}
 									if (_tmp17) {
-										self->priv->current = self->priv->current + (3);
+										self->priv->current = self->priv->current + 3;
 									} else {
 										ValaSourceReference* _tmp19;
 										_tmp19 = NULL;
@@ -1450,9 +1718,9 @@ ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* to
 										break;
 									} else {
 										gunichar u;
-										u = g_utf8_get_char_validated (((const char*) self->priv->current), (glong) (self->priv->end - self->priv->current));
+										u = g_utf8_get_char_validated ((const char*) self->priv->current, (glong) (self->priv->end - self->priv->current));
 										if (u != ((gunichar) (-1))) {
-											self->priv->current = self->priv->current + (g_unichar_to_utf8 (u, NULL));
+											self->priv->current = self->priv->current + g_unichar_to_utf8 (u, NULL);
 											token_length_in_chars++;
 										} else {
 											ValaSourceReference* _tmp23;
@@ -1485,10 +1753,10 @@ ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* to
 						default:
 						{
 							gunichar u;
-							u = g_utf8_get_char_validated (((const char*) self->priv->current), (glong) (self->priv->end - self->priv->current));
+							u = g_utf8_get_char_validated ((const char*) self->priv->current, (glong) (self->priv->end - self->priv->current));
 							if (u != ((gunichar) (-1))) {
 								ValaSourceReference* _tmp27;
-								self->priv->current = self->priv->current + (g_unichar_to_utf8 (u, NULL));
+								self->priv->current = self->priv->current + g_unichar_to_utf8 (u, NULL);
 								_tmp27 = NULL;
 								vala_report_error (_tmp27 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, unexpected character");
 								(_tmp27 == NULL) ? NULL : (_tmp27 = (vala_source_reference_unref (_tmp27), NULL));
@@ -1510,7 +1778,7 @@ ValaTokenType vala_scanner_read_token (ValaScanner* self, ValaSourceLocation* to
 	if (token_length_in_chars < 0) {
 		self->priv->column = self->priv->column + ((gint) (self->priv->current - begin));
 	} else {
-		self->priv->column = self->priv->column + (token_length_in_chars);
+		self->priv->column = self->priv->column + token_length_in_chars;
 	}
 	(*token_end).pos = self->priv->current;
 	(*token_end).line = self->priv->line;
@@ -1539,10 +1807,582 @@ static gboolean vala_scanner_matches (ValaScanner* self, gchar* begin, const cha
 }
 
 
-static gboolean vala_scanner_whitespace (ValaScanner* self) {
+static gboolean vala_scanner_pp_whitespace (ValaScanner* self) {
 	gboolean found;
 	g_return_val_if_fail (self != NULL, FALSE);
 	found = FALSE;
+	while (TRUE) {
+		gboolean _tmp0;
+		gboolean _tmp1;
+		_tmp0 = FALSE;
+		_tmp1 = FALSE;
+		if (self->priv->current < self->priv->end) {
+			_tmp1 = g_ascii_isspace (self->priv->current[0]);
+		} else {
+			_tmp1 = FALSE;
+		}
+		if (_tmp1) {
+			_tmp0 = self->priv->current[0] != '\n';
+		} else {
+			_tmp0 = FALSE;
+		}
+		if (!_tmp0) {
+			break;
+		}
+		found = TRUE;
+		self->priv->current++;
+		self->priv->column++;
+	}
+	return found;
+}
+
+
+static void vala_scanner_pp_directive (ValaScanner* self) {
+	gchar* begin;
+	gint len;
+	gboolean _tmp1;
+	gboolean _tmp6;
+	g_return_if_fail (self != NULL);
+	/* hash sign*/
+	self->priv->current++;
+	self->priv->column++;
+	vala_scanner_pp_whitespace (self);
+	begin = self->priv->current;
+	len = 0;
+	while (TRUE) {
+		gboolean _tmp0;
+		_tmp0 = FALSE;
+		if (self->priv->current < self->priv->end) {
+			_tmp0 = g_ascii_isalnum (self->priv->current[0]);
+		} else {
+			_tmp0 = FALSE;
+		}
+		if (!_tmp0) {
+			break;
+		}
+		self->priv->current++;
+		self->priv->column++;
+		len++;
+	}
+	_tmp1 = FALSE;
+	if (len == 2) {
+		_tmp1 = vala_scanner_matches (self, begin, "if");
+	} else {
+		_tmp1 = FALSE;
+	}
+	if (_tmp1) {
+		vala_scanner_parse_pp_if (self);
+	} else {
+		gboolean _tmp2;
+		_tmp2 = FALSE;
+		if (len == 4) {
+			_tmp2 = vala_scanner_matches (self, begin, "elif");
+		} else {
+			_tmp2 = FALSE;
+		}
+		if (_tmp2) {
+			vala_scanner_parse_pp_elif (self);
+		} else {
+			gboolean _tmp3;
+			_tmp3 = FALSE;
+			if (len == 4) {
+				_tmp3 = vala_scanner_matches (self, begin, "else");
+			} else {
+				_tmp3 = FALSE;
+			}
+			if (_tmp3) {
+				vala_scanner_parse_pp_else (self);
+			} else {
+				gboolean _tmp4;
+				_tmp4 = FALSE;
+				if (len == 5) {
+					_tmp4 = vala_scanner_matches (self, begin, "endif");
+				} else {
+					_tmp4 = FALSE;
+				}
+				if (_tmp4) {
+					vala_scanner_parse_pp_endif (self);
+				} else {
+					ValaSourceReference* _tmp5;
+					_tmp5 = NULL;
+					vala_report_error (_tmp5 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column - len, self->priv->line, self->priv->column), "syntax error, invalid preprocessing directive");
+					(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_source_reference_unref (_tmp5), NULL));
+				}
+			}
+		}
+	}
+	_tmp6 = FALSE;
+	if (self->priv->conditional_stack_length1 > 0) {
+		_tmp6 = self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].skip_section;
+	} else {
+		_tmp6 = FALSE;
+	}
+	if (_tmp6) {
+		gboolean bol;
+		/* skip lines until next preprocessing directive*/
+		bol = FALSE;
+		while (self->priv->current < self->priv->end) {
+			gboolean _tmp7;
+			_tmp7 = FALSE;
+			if (bol) {
+				_tmp7 = self->priv->current[0] == '#';
+			} else {
+				_tmp7 = FALSE;
+			}
+			if (_tmp7) {
+				/* go back to begin of line*/
+				self->priv->current = self->priv->current - self->priv->column;
+				self->priv->column = 0;
+				return;
+			}
+			if (self->priv->current[0] == '\n') {
+				self->priv->line++;
+				self->priv->column = 0;
+				bol = TRUE;
+			} else {
+				if (!g_ascii_isspace (self->priv->current[0])) {
+					bol = FALSE;
+				}
+			}
+			self->priv->current++;
+			self->priv->column++;
+		}
+	}
+}
+
+
+static void vala_scanner_pp_eol (ValaScanner* self) {
+	gboolean _tmp0;
+	g_return_if_fail (self != NULL);
+	vala_scanner_pp_whitespace (self);
+	_tmp0 = FALSE;
+	if (self->priv->current >= self->priv->end) {
+		_tmp0 = TRUE;
+	} else {
+		_tmp0 = self->priv->current[0] != '\n';
+	}
+	if (_tmp0) {
+		ValaSourceReference* _tmp1;
+		_tmp1 = NULL;
+		vala_report_error (_tmp1 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, expected newline");
+		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_source_reference_unref (_tmp1), NULL));
+	}
+}
+
+
+static void _vala_array_add1 (ValaScannerConditional** array, int* length, int* size, const ValaScannerConditional* value) {
+	if ((*length) == (*size)) {
+		*size = (*size) ? (2 * (*size)) : 4;
+		*array = g_renew (ValaScannerConditional, *array, *size);
+	}
+	(*array)[(*length)++] = *value;
+}
+
+
+static void vala_scanner_parse_pp_if (ValaScanner* self) {
+	gboolean condition;
+	ValaScannerConditional _tmp1 = {0};
+	ValaScannerConditional _tmp0 = {0};
+	gboolean _tmp2;
+	g_return_if_fail (self != NULL);
+	vala_scanner_pp_whitespace (self);
+	condition = vala_scanner_parse_pp_expression (self);
+	vala_scanner_pp_eol (self);
+	_vala_array_add1 (&self->priv->conditional_stack, &self->priv->conditional_stack_length1, &self->priv->conditional_stack_size, (_tmp1 = (memset (&_tmp0, 0, sizeof (ValaScannerConditional)), _tmp0), &_tmp1));
+	_tmp2 = FALSE;
+	if (condition) {
+		gboolean _tmp3;
+		_tmp3 = FALSE;
+		if (self->priv->conditional_stack_length1 == 1) {
+			_tmp3 = TRUE;
+		} else {
+			_tmp3 = !self->priv->conditional_stack[self->priv->conditional_stack_length1 - 2].skip_section;
+		}
+		_tmp2 = _tmp3;
+	} else {
+		_tmp2 = FALSE;
+	}
+	if (_tmp2) {
+		/* condition true => process code within if*/
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].matched = TRUE;
+	} else {
+		/* skip lines until next preprocessing directive*/
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].skip_section = TRUE;
+	}
+}
+
+
+static void vala_scanner_parse_pp_elif (ValaScanner* self) {
+	gboolean condition;
+	gboolean _tmp0;
+	gboolean _tmp2;
+	gboolean _tmp3;
+	g_return_if_fail (self != NULL);
+	vala_scanner_pp_whitespace (self);
+	condition = vala_scanner_parse_pp_expression (self);
+	vala_scanner_pp_eol (self);
+	_tmp0 = FALSE;
+	if (self->priv->conditional_stack_length1 == 0) {
+		_tmp0 = TRUE;
+	} else {
+		_tmp0 = self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].else_found;
+	}
+	if (_tmp0) {
+		ValaSourceReference* _tmp1;
+		_tmp1 = NULL;
+		vala_report_error (_tmp1 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, unexpected #elif");
+		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_source_reference_unref (_tmp1), NULL));
+		return;
+	}
+	_tmp2 = FALSE;
+	_tmp3 = FALSE;
+	if (condition) {
+		_tmp3 = !self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].matched;
+	} else {
+		_tmp3 = FALSE;
+	}
+	if (_tmp3) {
+		gboolean _tmp4;
+		_tmp4 = FALSE;
+		if (self->priv->conditional_stack_length1 == 1) {
+			_tmp4 = TRUE;
+		} else {
+			_tmp4 = !self->priv->conditional_stack[self->priv->conditional_stack_length1 - 2].skip_section;
+		}
+		_tmp2 = _tmp4;
+	} else {
+		_tmp2 = FALSE;
+	}
+	if (_tmp2) {
+		/* condition true => process code within if*/
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].matched = TRUE;
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].skip_section = FALSE;
+	} else {
+		/* skip lines until next preprocessing directive*/
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].skip_section = TRUE;
+	}
+}
+
+
+static void vala_scanner_parse_pp_else (ValaScanner* self) {
+	gboolean _tmp0;
+	gboolean _tmp2;
+	g_return_if_fail (self != NULL);
+	vala_scanner_pp_eol (self);
+	_tmp0 = FALSE;
+	if (self->priv->conditional_stack_length1 == 0) {
+		_tmp0 = TRUE;
+	} else {
+		_tmp0 = self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].else_found;
+	}
+	if (_tmp0) {
+		ValaSourceReference* _tmp1;
+		_tmp1 = NULL;
+		vala_report_error (_tmp1 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, unexpected #else");
+		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_source_reference_unref (_tmp1), NULL));
+		return;
+	}
+	_tmp2 = FALSE;
+	if (!self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].matched) {
+		gboolean _tmp3;
+		_tmp3 = FALSE;
+		if (self->priv->conditional_stack_length1 == 1) {
+			_tmp3 = TRUE;
+		} else {
+			_tmp3 = !self->priv->conditional_stack[self->priv->conditional_stack_length1 - 2].skip_section;
+		}
+		_tmp2 = _tmp3;
+	} else {
+		_tmp2 = FALSE;
+	}
+	if (_tmp2) {
+		/* condition true => process code within if*/
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].matched = TRUE;
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].skip_section = FALSE;
+	} else {
+		/* skip lines until next preprocessing directive*/
+		self->priv->conditional_stack[self->priv->conditional_stack_length1 - 1].skip_section = TRUE;
+	}
+}
+
+
+static void vala_scanner_parse_pp_endif (ValaScanner* self) {
+	g_return_if_fail (self != NULL);
+	vala_scanner_pp_eol (self);
+	if (self->priv->conditional_stack_length1 == 0) {
+		ValaSourceReference* _tmp0;
+		_tmp0 = NULL;
+		vala_report_error (_tmp0 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, unexpected #endif");
+		(_tmp0 == NULL) ? NULL : (_tmp0 = (vala_source_reference_unref (_tmp0), NULL));
+		return;
+	}
+	self->priv->conditional_stack_length1--;
+}
+
+
+static gboolean vala_scanner_parse_pp_symbol (ValaScanner* self) {
+	gint len;
+	char* identifier;
+	gboolean defined;
+	gboolean _tmp3;
+	g_return_val_if_fail (self != NULL, FALSE);
+	len = 0;
+	while (TRUE) {
+		gboolean _tmp0;
+		_tmp0 = FALSE;
+		if (self->priv->current < self->priv->end) {
+			_tmp0 = vala_scanner_is_ident_char (self, self->priv->current[0]);
+		} else {
+			_tmp0 = FALSE;
+		}
+		if (!_tmp0) {
+			break;
+		}
+		self->priv->current++;
+		self->priv->column++;
+		len++;
+	}
+	if (len == 0) {
+		ValaSourceReference* _tmp1;
+		_tmp1 = NULL;
+		vala_report_error (_tmp1 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, expected identifier");
+		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_source_reference_unref (_tmp1), NULL));
+		return FALSE;
+	}
+	identifier = g_strndup ((const char*) (self->priv->current - len), (gsize) len);
+	defined = FALSE;
+	if (_vala_strcmp0 (identifier, "true") == 0) {
+		defined = TRUE;
+	} else {
+		if (_vala_strcmp0 (identifier, "false") == 0) {
+			defined = FALSE;
+		} else {
+			defined = vala_code_context_is_defined (vala_source_file_get_context (self->priv->_source_file), identifier);
+		}
+	}
+	return (_tmp3 = defined, identifier = (g_free (identifier), NULL), _tmp3);
+}
+
+
+static gboolean vala_scanner_parse_pp_primary_expression (ValaScanner* self) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	if (self->priv->current >= self->priv->end) {
+		ValaSourceReference* _tmp0;
+		_tmp0 = NULL;
+		vala_report_error (_tmp0 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, expected identifier");
+		(_tmp0 == NULL) ? NULL : (_tmp0 = (vala_source_reference_unref (_tmp0), NULL));
+	} else {
+		if (vala_scanner_is_ident_char (self, self->priv->current[0])) {
+			return vala_scanner_parse_pp_symbol (self);
+		} else {
+			if (self->priv->current[0] == '(') {
+				gboolean result;
+				gboolean _tmp2;
+				self->priv->current++;
+				self->priv->column++;
+				vala_scanner_pp_whitespace (self);
+				result = vala_scanner_parse_pp_expression (self);
+				vala_scanner_pp_whitespace (self);
+				_tmp2 = FALSE;
+				if (self->priv->current < self->priv->end) {
+					_tmp2 = self->priv->current[0] == ')';
+				} else {
+					_tmp2 = FALSE;
+				}
+				if (_tmp2) {
+					self->priv->current++;
+					self->priv->column++;
+				} else {
+					ValaSourceReference* _tmp3;
+					_tmp3 = NULL;
+					vala_report_error (_tmp3 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, expected `)'");
+					(_tmp3 == NULL) ? NULL : (_tmp3 = (vala_source_reference_unref (_tmp3), NULL));
+				}
+				return result;
+			} else {
+				ValaSourceReference* _tmp5;
+				_tmp5 = NULL;
+				vala_report_error (_tmp5 = vala_source_reference_new (self->priv->_source_file, self->priv->line, self->priv->column, self->priv->line, self->priv->column), "syntax error, expected identifier");
+				(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_source_reference_unref (_tmp5), NULL));
+			}
+		}
+	}
+	return FALSE;
+}
+
+
+static gboolean vala_scanner_parse_pp_unary_expression (ValaScanner* self) {
+	gboolean _tmp0;
+	g_return_val_if_fail (self != NULL, FALSE);
+	_tmp0 = FALSE;
+	if (self->priv->current < self->priv->end) {
+		_tmp0 = self->priv->current[0] == '!';
+	} else {
+		_tmp0 = FALSE;
+	}
+	if (_tmp0) {
+		self->priv->current++;
+		self->priv->column++;
+		vala_scanner_pp_whitespace (self);
+		return !vala_scanner_parse_pp_unary_expression (self);
+	}
+	return vala_scanner_parse_pp_primary_expression (self);
+}
+
+
+static gboolean vala_scanner_parse_pp_equality_expression (ValaScanner* self) {
+	gboolean left;
+	g_return_val_if_fail (self != NULL, FALSE);
+	left = vala_scanner_parse_pp_unary_expression (self);
+	vala_scanner_pp_whitespace (self);
+	while (TRUE) {
+		gboolean _tmp0;
+		gboolean _tmp1;
+		_tmp0 = FALSE;
+		_tmp1 = FALSE;
+		if (self->priv->current < (self->priv->end - 1)) {
+			_tmp1 = self->priv->current[0] == '=';
+		} else {
+			_tmp1 = FALSE;
+		}
+		if (_tmp1) {
+			_tmp0 = self->priv->current[1] == '=';
+		} else {
+			_tmp0 = FALSE;
+		}
+		if (_tmp0) {
+			gboolean right;
+			self->priv->current = self->priv->current + 2;
+			self->priv->column = self->priv->column + 2;
+			vala_scanner_pp_whitespace (self);
+			right = vala_scanner_parse_pp_unary_expression (self);
+			left = left == right;
+		} else {
+			gboolean _tmp2;
+			gboolean _tmp3;
+			_tmp2 = FALSE;
+			_tmp3 = FALSE;
+			if (self->priv->current < (self->priv->end - 1)) {
+				_tmp3 = self->priv->current[0] == '!';
+			} else {
+				_tmp3 = FALSE;
+			}
+			if (_tmp3) {
+				_tmp2 = self->priv->current[1] == '=';
+			} else {
+				_tmp2 = FALSE;
+			}
+			if (_tmp2) {
+				gboolean right;
+				self->priv->current = self->priv->current + 2;
+				self->priv->column = self->priv->column + 2;
+				vala_scanner_pp_whitespace (self);
+				right = vala_scanner_parse_pp_unary_expression (self);
+				left = left != right;
+			} else {
+				break;
+			}
+		}
+	}
+	return left;
+}
+
+
+static gboolean vala_scanner_parse_pp_and_expression (ValaScanner* self) {
+	gboolean left;
+	g_return_val_if_fail (self != NULL, FALSE);
+	left = vala_scanner_parse_pp_equality_expression (self);
+	vala_scanner_pp_whitespace (self);
+	while (TRUE) {
+		gboolean _tmp0;
+		gboolean _tmp1;
+		gboolean right;
+		gboolean _tmp2;
+		_tmp0 = FALSE;
+		_tmp1 = FALSE;
+		if (self->priv->current < (self->priv->end - 1)) {
+			_tmp1 = self->priv->current[0] == '&';
+		} else {
+			_tmp1 = FALSE;
+		}
+		if (_tmp1) {
+			_tmp0 = self->priv->current[1] == '&';
+		} else {
+			_tmp0 = FALSE;
+		}
+		if (!_tmp0) {
+			break;
+		}
+		self->priv->current = self->priv->current + 2;
+		self->priv->column = self->priv->column + 2;
+		vala_scanner_pp_whitespace (self);
+		right = vala_scanner_parse_pp_equality_expression (self);
+		_tmp2 = FALSE;
+		if (left) {
+			_tmp2 = right;
+		} else {
+			_tmp2 = FALSE;
+		}
+		left = _tmp2;
+	}
+	return left;
+}
+
+
+static gboolean vala_scanner_parse_pp_or_expression (ValaScanner* self) {
+	gboolean left;
+	g_return_val_if_fail (self != NULL, FALSE);
+	left = vala_scanner_parse_pp_and_expression (self);
+	vala_scanner_pp_whitespace (self);
+	while (TRUE) {
+		gboolean _tmp0;
+		gboolean _tmp1;
+		gboolean right;
+		gboolean _tmp2;
+		_tmp0 = FALSE;
+		_tmp1 = FALSE;
+		if (self->priv->current < (self->priv->end - 1)) {
+			_tmp1 = self->priv->current[0] == '|';
+		} else {
+			_tmp1 = FALSE;
+		}
+		if (_tmp1) {
+			_tmp0 = self->priv->current[1] == '|';
+		} else {
+			_tmp0 = FALSE;
+		}
+		if (!_tmp0) {
+			break;
+		}
+		self->priv->current = self->priv->current + 2;
+		self->priv->column = self->priv->column + 2;
+		vala_scanner_pp_whitespace (self);
+		right = vala_scanner_parse_pp_and_expression (self);
+		_tmp2 = FALSE;
+		if (left) {
+			_tmp2 = TRUE;
+		} else {
+			_tmp2 = right;
+		}
+		left = _tmp2;
+	}
+	return left;
+}
+
+
+static gboolean vala_scanner_parse_pp_expression (ValaScanner* self) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	return vala_scanner_parse_pp_or_expression (self);
+}
+
+
+static gboolean vala_scanner_whitespace (ValaScanner* self) {
+	gboolean found;
+	gboolean bol;
+	gboolean _tmp1;
+	g_return_val_if_fail (self != NULL, FALSE);
+	found = FALSE;
+	bol = self->priv->column == 1;
 	while (TRUE) {
 		gboolean _tmp0;
 		_tmp0 = FALSE;
@@ -1557,11 +2397,22 @@ static gboolean vala_scanner_whitespace (ValaScanner* self) {
 		if (self->priv->current[0] == '\n') {
 			self->priv->line++;
 			self->priv->column = 0;
+			bol = TRUE;
 		}
 		found = TRUE;
 		self->priv->current++;
 		self->priv->column++;
 	}
+	_tmp1 = FALSE;
+	if (bol) {
+		_tmp1 = self->priv->current[0] == '#';
+	} else {
+		_tmp1 = FALSE;
+	}
+	if (_tmp1) {
+		vala_scanner_pp_directive (self);
+		return TRUE;
+	}
 	return found;
 }
 
@@ -1587,7 +2438,7 @@ static gboolean vala_scanner_comment (ValaScanner* self) {
 		} else {
 			_tmp2 = FALSE;
 		}
-		_tmp0 = (_tmp2);
+		_tmp0 = _tmp2;
 	}
 	if (_tmp0) {
 		return FALSE;
@@ -1596,7 +2447,7 @@ static gboolean vala_scanner_comment (ValaScanner* self) {
 		gchar* begin;
 		char* _tmp5;
 		/* single-line comment*/
-		self->priv->current = self->priv->current + (2);
+		self->priv->current = self->priv->current + 2;
 		begin = self->priv->current;
 		while (TRUE) {
 			gboolean _tmp4;
@@ -1612,14 +2463,14 @@ static gboolean vala_scanner_comment (ValaScanner* self) {
 			self->priv->current++;
 		}
 		_tmp5 = NULL;
-		vala_scanner_push_comment (self, _tmp5 = g_strndup (((const char*) begin), (gsize) ((glong) (self->priv->current - begin))), self->priv->line == 1);
+		vala_scanner_push_comment (self, _tmp5 = g_strndup ((const char*) begin, (gsize) ((glong) (self->priv->current - begin))), self->priv->line == 1);
 		_tmp5 = (g_free (_tmp5), NULL);
 	} else {
 		gchar* begin;
 		gint begin_line;
 		char* _tmp10;
 		/* delimited comment*/
-		self->priv->current = self->priv->current + (2);
+		self->priv->current = self->priv->current + 2;
 		begin = self->priv->current;
 		begin_line = self->priv->line;
 		while (TRUE) {
@@ -1633,7 +2484,7 @@ static gboolean vala_scanner_comment (ValaScanner* self) {
 				} else {
 					_tmp7 = self->priv->current[1] != '/';
 				}
-				_tmp6 = (_tmp7);
+				_tmp6 = _tmp7;
 			} else {
 				_tmp6 = FALSE;
 			}
@@ -1655,10 +2506,10 @@ static gboolean vala_scanner_comment (ValaScanner* self) {
 			return TRUE;
 		}
 		_tmp10 = NULL;
-		vala_scanner_push_comment (self, _tmp10 = g_strndup (((const char*) begin), (gsize) ((glong) (self->priv->current - begin))), begin_line == 1);
+		vala_scanner_push_comment (self, _tmp10 = g_strndup ((const char*) begin, (gsize) ((glong) (self->priv->current - begin))), begin_line == 1);
 		_tmp10 = (g_free (_tmp10), NULL);
-		self->priv->current = self->priv->current + (2);
-		self->priv->column = self->priv->column + (2);
+		self->priv->current = self->priv->current + 2;
+		self->priv->column = self->priv->column + 2;
 	}
 	return TRUE;
 }
@@ -1748,6 +2599,28 @@ static void vala_scanner_set_source_file (ValaScanner* self, ValaSourceFile* val
 }
 
 
+static ValaScannerConditional* vala_scanner_conditional_dup (const ValaScannerConditional* self) {
+	ValaScannerConditional* dup;
+	dup = g_new0 (ValaScannerConditional, 1);
+	memcpy (dup, self, sizeof (ValaScannerConditional));
+	return dup;
+}
+
+
+static void vala_scanner_conditional_free (ValaScannerConditional* self) {
+	g_free (self);
+}
+
+
+static GType vala_scanner_conditional_get_type (void) {
+	static GType vala_scanner_conditional_type_id = 0;
+	if (vala_scanner_conditional_type_id == 0) {
+		vala_scanner_conditional_type_id = g_boxed_type_register_static ("ValaScannerConditional", (GBoxedCopyFunc) vala_scanner_conditional_dup, (GBoxedFreeFunc) vala_scanner_conditional_free);
+	}
+	return vala_scanner_conditional_type_id;
+}
+
+
 static void vala_value_scanner_init (GValue* value) {
 	value->data[0].v_pointer = NULL;
 }
@@ -1859,6 +2732,7 @@ static void vala_scanner_finalize (ValaScanner* obj) {
 	self = VALA_SCANNER (obj);
 	(self->priv->_source_file == NULL) ? NULL : (self->priv->_source_file = (vala_source_file_unref (self->priv->_source_file), NULL));
 	self->priv->_comment = (g_free (self->priv->_comment), NULL);
+	self->priv->conditional_stack = (g_free (self->priv->conditional_stack), NULL);
 }
 
 
@@ -1892,5 +2766,16 @@ void vala_scanner_unref (gpointer instance) {
 }
 
 
+static int _vala_strcmp0 (const char * str1, const char * str2) {
+	if (str1 == NULL) {
+		return -(str1 != str2);
+	}
+	if (str2 == NULL) {
+		return str1 != str2;
+	}
+	return strcmp (str1, str2);
+}
+
+
 
 
