@@ -20,19 +20,69 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vala/valascope.h>
-#include <gee/map.h>
-#include <gee/list.h>
-#include <gee/hashmap.h>
-#include <gee/arraylist.h>
-#include <gee/collection.h>
-#include <vala/valasymbol.h>
-#include <vala/valareport.h>
-#include <vala/valasourcereference.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <gee.h>
+#include <stdlib.h>
+#include <string.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_SCOPE (vala_scope_get_type ())
+#define VALA_SCOPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SCOPE, ValaScope))
+#define VALA_SCOPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SCOPE, ValaScopeClass))
+#define VALA_IS_SCOPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SCOPE))
+#define VALA_IS_SCOPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SCOPE))
+#define VALA_SCOPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SCOPE, ValaScopeClass))
 
+typedef struct _ValaScope ValaScope;
+typedef struct _ValaScopeClass ValaScopeClass;
+typedef struct _ValaScopePrivate ValaScopePrivate;
+
+#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
+#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
+#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
+#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
+#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+
+typedef struct _ValaCodeNode ValaCodeNode;
+typedef struct _ValaCodeNodeClass ValaCodeNodeClass;
+
+#define VALA_TYPE_SYMBOL (vala_symbol_get_type ())
+#define VALA_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SYMBOL, ValaSymbol))
+#define VALA_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SYMBOL, ValaSymbolClass))
+#define VALA_IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SYMBOL))
+#define VALA_IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SYMBOL))
+#define VALA_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SYMBOL, ValaSymbolClass))
+
+typedef struct _ValaSymbol ValaSymbol;
+typedef struct _ValaSymbolClass ValaSymbolClass;
+
+#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
+#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
+#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+
+typedef struct _ValaSourceReference ValaSourceReference;
+typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
+typedef struct _ValaParamSpecScope ValaParamSpecScope;
+
+/**
+ * Represents a part of the symbol tree.
+ */
+struct _ValaScope {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaScopePrivate * priv;
+};
+
+struct _ValaScopeClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaScope *self);
+};
 
 struct _ValaScopePrivate {
 	ValaSymbol* _owner;
@@ -41,10 +91,53 @@ struct _ValaScopePrivate {
 	GeeList* anonymous_members;
 };
 
+struct _ValaParamSpecScope {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_scope_ref (gpointer instance);
+void vala_scope_unref (gpointer instance);
+GParamSpec* vala_param_spec_scope (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_scope (GValue* value, gpointer v_object);
+gpointer vala_value_get_scope (const GValue* value);
+GType vala_scope_get_type (void);
+gpointer vala_code_node_ref (gpointer instance);
+void vala_code_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_node (const GValue* value);
+GType vala_code_node_get_type (void);
+GType vala_symbol_get_type (void);
 #define VALA_SCOPE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SCOPE, ValaScopePrivate))
 enum  {
 	VALA_SCOPE_DUMMY_PROPERTY
 };
+void vala_scope_set_owner (ValaScope* self, ValaSymbol* value);
+ValaScope* vala_scope_new (ValaSymbol* owner);
+ValaScope* vala_scope_construct (GType object_type, ValaSymbol* owner);
+ValaScope* vala_scope_new (ValaSymbol* owner);
+ValaSymbol* vala_scope_lookup (ValaScope* self, const char* name);
+void vala_code_node_set_error (ValaCodeNode* self, gboolean value);
+ValaSymbol* vala_scope_get_owner (ValaScope* self);
+gpointer vala_source_reference_ref (gpointer instance);
+void vala_source_reference_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_reference (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_reference (const GValue* value);
+GType vala_source_reference_get_type (void);
+void vala_report_error (ValaSourceReference* source, const char* message);
+ValaSourceReference* vala_code_node_get_source_reference (ValaCodeNode* self);
+char* vala_symbol_get_full_name (ValaSymbol* self);
+void vala_symbol_set_owner (ValaSymbol* self, ValaScope* value);
+void vala_scope_add (ValaScope* self, const char* name, ValaSymbol* sym);
+void vala_scope_remove (ValaScope* self, const char* name);
+gboolean vala_symbol_get_active (ValaSymbol* self);
+ValaScope* vala_scope_get_parent_scope (ValaScope* self);
+gboolean vala_scope_is_subscope_of (ValaScope* self, ValaScope* scope);
+GeeMap* vala_scope_get_symbol_table (ValaScope* self);
+void vala_scope_set_parent_scope (ValaScope* self, ValaScope* value);
 static gpointer vala_scope_parent_class = NULL;
 static void vala_scope_finalize (ValaScope* obj);
 
@@ -173,6 +266,12 @@ gboolean vala_scope_is_subscope_of (ValaScope* self, ValaScope* scope) {
 }
 
 
+GeeMap* vala_scope_get_symbol_table (ValaScope* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return (GeeMap*) gee_read_only_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, self->priv->symbol_table);
+}
+
+
 ValaSymbol* vala_scope_get_owner (ValaScope* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_owner;
