@@ -20,75 +20,225 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vala/valasourcefile.h>
-#include <gee/arraylist.h>
-#include <gee/collection.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
+#include <gee.h>
 #include <glib/gstdio.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <gee/hashset.h>
-#include <gee/readonlylist.h>
-#include <gee/readonlyset.h>
-#include <vala/valasourcefilecycle.h>
-#include <vala/valacodecontext.h>
-#include <vala/valadatatype.h>
-#include <vala/valasymbol.h>
-#include <vala/valaerrorcode.h>
-#include <vala/valaenumvalue.h>
-#include <vala/valatypesymbol.h>
-#include <vala/valamethod.h>
-#include <vala/valafield.h>
-#include <vala/valaproperty.h>
-#include <vala/valaconstant.h>
-#include <vala/valaformalparameter.h>
-#include <vala/valasourcereference.h>
-#include <vala/valausingdirective.h>
-#include <vala/valacodevisitor.h>
-#include <vala/valasemanticanalyzer.h>
-#include <vala/valaunresolvedsymbol.h>
-#include <vala/valareport.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_SOURCE_FILE (vala_source_file_get_type ())
+#define VALA_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFile))
+#define VALA_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
+#define VALA_IS_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_FILE))
+#define VALA_IS_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_FILE))
+#define VALA_SOURCE_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
 
+typedef struct _ValaSourceFile ValaSourceFile;
+typedef struct _ValaSourceFileClass ValaSourceFileClass;
+typedef struct _ValaSourceFilePrivate ValaSourceFilePrivate;
+
+#define VALA_TYPE_CODE_CONTEXT (vala_code_context_get_type ())
+#define VALA_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContext))
+#define VALA_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))
+#define VALA_IS_CODE_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_CONTEXT))
+#define VALA_IS_CODE_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_CONTEXT))
+#define VALA_CODE_CONTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_CONTEXT, ValaCodeContextClass))
+
+typedef struct _ValaCodeContext ValaCodeContext;
+typedef struct _ValaCodeContextClass ValaCodeContextClass;
+
+#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
+#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
+#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
+#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
+#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
+
+typedef struct _ValaCodeNode ValaCodeNode;
+typedef struct _ValaCodeNodeClass ValaCodeNodeClass;
+
+#define VALA_TYPE_USING_DIRECTIVE (vala_using_directive_get_type ())
+#define VALA_USING_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_USING_DIRECTIVE, ValaUsingDirective))
+#define VALA_USING_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_USING_DIRECTIVE, ValaUsingDirectiveClass))
+#define VALA_IS_USING_DIRECTIVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_USING_DIRECTIVE))
+#define VALA_IS_USING_DIRECTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_USING_DIRECTIVE))
+#define VALA_USING_DIRECTIVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_USING_DIRECTIVE, ValaUsingDirectiveClass))
+
+typedef struct _ValaUsingDirective ValaUsingDirective;
+typedef struct _ValaUsingDirectiveClass ValaUsingDirectiveClass;
+
+#define VALA_TYPE_SYMBOL (vala_symbol_get_type ())
+#define VALA_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SYMBOL, ValaSymbol))
+#define VALA_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SYMBOL, ValaSymbolClass))
+#define VALA_IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SYMBOL))
+#define VALA_IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SYMBOL))
+#define VALA_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SYMBOL, ValaSymbolClass))
+
+typedef struct _ValaSymbol ValaSymbol;
+typedef struct _ValaSymbolClass ValaSymbolClass;
+
+#define VALA_TYPE_UNRESOLVED_SYMBOL (vala_unresolved_symbol_get_type ())
+#define VALA_UNRESOLVED_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_UNRESOLVED_SYMBOL, ValaUnresolvedSymbol))
+#define VALA_UNRESOLVED_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_UNRESOLVED_SYMBOL, ValaUnresolvedSymbolClass))
+#define VALA_IS_UNRESOLVED_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_UNRESOLVED_SYMBOL))
+#define VALA_IS_UNRESOLVED_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_UNRESOLVED_SYMBOL))
+#define VALA_UNRESOLVED_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_UNRESOLVED_SYMBOL, ValaUnresolvedSymbolClass))
+
+typedef struct _ValaUnresolvedSymbol ValaUnresolvedSymbol;
+typedef struct _ValaUnresolvedSymbolClass ValaUnresolvedSymbolClass;
+
+#define VALA_TYPE_CODE_VISITOR (vala_code_visitor_get_type ())
+#define VALA_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitor))
+#define VALA_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))
+#define VALA_IS_CODE_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_VISITOR))
+#define VALA_IS_CODE_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_VISITOR))
+#define VALA_CODE_VISITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_VISITOR, ValaCodeVisitorClass))
+
+typedef struct _ValaCodeVisitor ValaCodeVisitor;
+typedef struct _ValaCodeVisitorClass ValaCodeVisitorClass;
+
+#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
+#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
+#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+
+typedef struct _ValaSourceReference ValaSourceReference;
+typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
+
+#define VALA_TYPE_SEMANTIC_ANALYZER (vala_semantic_analyzer_get_type ())
+#define VALA_SEMANTIC_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzer))
+#define VALA_SEMANTIC_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerClass))
+#define VALA_IS_SEMANTIC_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SEMANTIC_ANALYZER))
+#define VALA_IS_SEMANTIC_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SEMANTIC_ANALYZER))
+#define VALA_SEMANTIC_ANALYZER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerClass))
+
+typedef struct _ValaSemanticAnalyzer ValaSemanticAnalyzer;
+typedef struct _ValaSemanticAnalyzerClass ValaSemanticAnalyzerClass;
+typedef struct _ValaParamSpecSourceFile ValaParamSpecSourceFile;
+
+/**
+ * Represents a Vala source or VAPI package file.
+ */
+struct _ValaSourceFile {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaSourceFilePrivate * priv;
+};
+
+struct _ValaSourceFileClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaSourceFile *self);
+};
 
-static char* string_substring (const char* self, glong offset, glong len);
-static glong string_get_length (const char* self);
 struct _ValaSourceFilePrivate {
 	char* _filename;
 	char* _comment;
 	gboolean _external_package;
-	ValaSourceFileCycle* _cycle;
-	gboolean _is_cycle_head;
-	gint _mark;
 	ValaCodeContext* _context;
 	GeeList* using_directives;
 	GeeList* nodes;
 	char* cheader_filename;
 	char* csource_filename;
 	char* cinclude_filename;
-	GeeList* header_external_includes;
-	GeeList* header_internal_includes;
-	GeeList* source_external_includes;
-	GeeList* source_internal_includes;
-	GeeList* header_internal_full_dependencies;
-	GeeList* header_internal_dependencies;
-	GeeSet* source_symbol_dependencies;
 	GeeArrayList* source_array;
 	GMappedFile* mapped_file;
 	char* _content;
 };
 
+struct _ValaParamSpecSourceFile {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_source_file_ref (gpointer instance);
+void vala_source_file_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_file (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_file (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_file (const GValue* value);
+GType vala_source_file_get_type (void);
+gpointer vala_code_context_ref (gpointer instance);
+void vala_code_context_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_context (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_context (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_context (const GValue* value);
+GType vala_code_context_get_type (void);
+gpointer vala_code_node_ref (gpointer instance);
+void vala_code_node_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_node (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_node (const GValue* value);
+GType vala_code_node_get_type (void);
+GType vala_using_directive_get_type (void);
 #define VALA_SOURCE_FILE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SOURCE_FILE, ValaSourceFilePrivate))
 enum  {
 	VALA_SOURCE_FILE_DUMMY_PROPERTY
 };
+void vala_source_file_set_filename (ValaSourceFile* self, const char* value);
+void vala_source_file_set_external_package (ValaSourceFile* self, gboolean value);
+void vala_source_file_set_context (ValaSourceFile* self, ValaCodeContext* value);
+void vala_source_file_set_content (ValaSourceFile* self, const char* value);
+ValaSourceFile* vala_source_file_new (ValaCodeContext* context, const char* filename, gboolean pkg, const char* content);
+ValaSourceFile* vala_source_file_construct (GType object_type, ValaCodeContext* context, const char* filename, gboolean pkg, const char* content);
+ValaSourceFile* vala_source_file_new (ValaCodeContext* context, const char* filename, gboolean pkg, const char* content);
+GType vala_symbol_get_type (void);
 static gboolean vala_source_file_same_symbol (ValaSourceFile* self, ValaSymbol* sym1, ValaSymbol* sym2);
+ValaSymbol* vala_using_directive_get_namespace_symbol (ValaUsingDirective* self);
+void vala_source_file_add_using_directive (ValaSourceFile* self, ValaUsingDirective* ns);
+void vala_source_file_clear_using_directives (ValaSourceFile* self);
+GType vala_unresolved_symbol_get_type (void);
+ValaUnresolvedSymbol* vala_unresolved_symbol_get_inner (ValaUnresolvedSymbol* self);
+const char* vala_symbol_get_name (ValaSymbol* self);
+GeeList* vala_source_file_get_using_directives (ValaSourceFile* self);
+void vala_source_file_add_node (ValaSourceFile* self, ValaCodeNode* node);
+void vala_source_file_remove_node (ValaSourceFile* self, ValaCodeNode* node);
+GeeList* vala_source_file_get_nodes (ValaSourceFile* self);
+gpointer vala_code_visitor_ref (gpointer instance);
+void vala_code_visitor_unref (gpointer instance);
+GParamSpec* vala_param_spec_code_visitor (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_code_visitor (GValue* value, gpointer v_object);
+gpointer vala_value_get_code_visitor (const GValue* value);
+GType vala_code_visitor_get_type (void);
+void vala_code_visitor_visit_source_file (ValaCodeVisitor* self, ValaSourceFile* source_file);
+void vala_source_file_accept (ValaSourceFile* self, ValaCodeVisitor* visitor);
+void vala_code_node_accept (ValaCodeNode* self, ValaCodeVisitor* visitor);
+void vala_source_file_accept_children (ValaSourceFile* self, ValaCodeVisitor* visitor);
+ValaCodeContext* vala_source_file_get_context (ValaSourceFile* self);
+const char* vala_code_context_get_basedir (ValaCodeContext* self);
+const char* vala_source_file_get_filename (ValaSourceFile* self);
 static char* vala_source_file_get_subdir (ValaSourceFile* self);
+const char* vala_code_context_get_directory (ValaCodeContext* self);
 static char* vala_source_file_get_destination_directory (ValaSourceFile* self);
 static char* vala_source_file_get_basename (ValaSourceFile* self);
-static void vala_source_file_read_source_file (ValaSourceFile* self);
+char* vala_source_file_get_relative_filename (ValaSourceFile* self);
+char* vala_source_file_get_cheader_filename (ValaSourceFile* self);
+char* vala_source_file_get_csource_filename (ValaSourceFile* self);
+const char* vala_code_context_get_header_filename (ValaCodeContext* self);
+char* vala_source_file_get_cinclude_filename (ValaSourceFile* self);
+const char* vala_source_file_get_content (ValaSourceFile* self);
 static void vala_source_file_read_source_lines (ValaSourceFile* self, const char* cont);
+static void vala_source_file_read_source_file (ValaSourceFile* self);
+char* vala_source_file_get_source_line (ValaSourceFile* self, gint lineno);
+gpointer vala_source_reference_ref (gpointer instance);
+void vala_source_reference_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_reference (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_reference (const GValue* value);
+GType vala_source_reference_get_type (void);
+void vala_report_error (ValaSourceReference* source, const char* message);
+gchar* vala_source_file_get_mapped_contents (ValaSourceFile* self);
+gsize vala_source_file_get_mapped_length (ValaSourceFile* self);
+GType vala_semantic_analyzer_get_type (void);
+gboolean vala_code_node_check (ValaCodeNode* self, ValaSemanticAnalyzer* analyzer);
+gboolean vala_source_file_check (ValaSourceFile* self, ValaSemanticAnalyzer* analyzer);
+const char* vala_source_file_get_comment (ValaSourceFile* self);
+void vala_source_file_set_comment (ValaSourceFile* self, const char* value);
+gboolean vala_source_file_get_external_package (ValaSourceFile* self);
 static gpointer vala_source_file_parent_class = NULL;
 static void vala_source_file_finalize (ValaSourceFile* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
@@ -97,32 +247,6 @@ static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
 
-static char* string_substring (const char* self, glong offset, glong len) {
-	glong string_length;
-	const char* start;
-	g_return_val_if_fail (self != NULL, NULL);
-	string_length = g_utf8_strlen (self, -1);
-	if (offset < 0) {
-		offset = string_length + offset;
-		g_return_val_if_fail (offset >= 0, NULL);
-	} else {
-		g_return_val_if_fail (offset <= string_length, NULL);
-	}
-	if (len < 0) {
-		len = string_length - offset;
-	}
-	g_return_val_if_fail ((offset + len) <= string_length, NULL);
-	start = g_utf8_offset_to_pointer (self, offset);
-	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
-}
-
-
-static glong string_get_length (const char* self) {
-	g_return_val_if_fail (self != NULL, 0L);
-	return g_utf8_strlen (self, -1);
-}
-
-
 /**
  * Creates a new source file.
  *
@@ -293,6 +417,26 @@ void vala_source_file_accept_children (ValaSourceFile* self, ValaCodeVisitor* vi
 }
 
 
+static char* string_substring (const char* self, glong offset, glong len) {
+	glong string_length;
+	const char* start;
+	g_return_val_if_fail (self != NULL, NULL);
+	string_length = g_utf8_strlen (self, -1);
+	if (offset < 0) {
+		offset = string_length + offset;
+		g_return_val_if_fail (offset >= 0, NULL);
+	} else {
+		g_return_val_if_fail (offset <= string_length, NULL);
+	}
+	if (len < 0) {
+		len = string_length - offset;
+	}
+	g_return_val_if_fail ((offset + len) <= string_length, NULL);
+	start = g_utf8_offset_to_pointer (self, offset);
+	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
+}
+
+
 static char* vala_source_file_get_subdir (ValaSourceFile* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	if (vala_code_context_get_basedir (self->priv->_context) == NULL) {
@@ -411,322 +555,27 @@ char* vala_source_file_get_csource_filename (ValaSourceFile* self) {
  * @return C header filename to include
  */
 char* vala_source_file_get_cinclude_filename (ValaSourceFile* self) {
-	const char* _tmp3;
+	const char* _tmp4;
 	g_return_val_if_fail (self != NULL, NULL);
 	if (self->priv->cinclude_filename == NULL) {
-		char* _tmp2;
-		char* _tmp1;
-		char* _tmp0;
-		_tmp2 = NULL;
-		_tmp1 = NULL;
-		_tmp0 = NULL;
-		self->priv->cinclude_filename = (_tmp2 = g_strdup_printf ("%s%s.h", _tmp0 = vala_source_file_get_subdir (self), _tmp1 = vala_source_file_get_basename (self)), self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL), _tmp2);
-		_tmp1 = (g_free (_tmp1), NULL);
-		_tmp0 = (g_free (_tmp0), NULL);
-	}
-	_tmp3 = NULL;
-	return (_tmp3 = self->priv->cinclude_filename, (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
-}
-
-
-/**
- * Adds the specified symbol to the list of symbols code in this source
- * file depends on.
- *
- * TODO Move source and header file dependency analysis to
- * code generator.
- *
- * @param sym      a symbol
- * @param dep_type type of dependency
- */
-void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* sym, ValaSourceFileDependencyType dep_type) {
-	ValaSymbol* s;
-	gboolean _tmp0;
-	g_return_if_fail (self != NULL);
-	if (self->priv->_external_package) {
-		return;
-	}
-	s = NULL;
-	_tmp0 = FALSE;
-	if (VALA_IS_ERROR_CODE (sym)) {
-		_tmp0 = TRUE;
-	} else {
-		_tmp0 = VALA_IS_ENUM_VALUE (sym);
-	}
-	if (_tmp0) {
-		ValaSymbol* _tmp2;
-		ValaSymbol* _tmp1;
-		_tmp2 = NULL;
-		_tmp1 = NULL;
-		s = (_tmp2 = (_tmp1 = vala_symbol_get_parent_symbol (sym), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL)), _tmp2);
-	} else {
-		gboolean _tmp3;
-		gboolean _tmp4;
-		gboolean _tmp5;
-		gboolean _tmp6;
-		_tmp3 = FALSE;
-		_tmp4 = FALSE;
-		_tmp5 = FALSE;
-		_tmp6 = FALSE;
-		if (VALA_IS_TYPESYMBOL (sym)) {
-			_tmp6 = TRUE;
+		if (vala_code_context_get_header_filename (self->priv->_context) != NULL) {
+			char* _tmp0;
+			_tmp0 = NULL;
+			self->priv->cinclude_filename = (_tmp0 = g_path_get_basename (vala_code_context_get_header_filename (self->priv->_context)), self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL), _tmp0);
 		} else {
-			_tmp6 = VALA_IS_METHOD (sym);
-		}
-		if (_tmp6) {
-			_tmp5 = TRUE;
-		} else {
-			_tmp5 = VALA_IS_FIELD (sym);
-		}
-		if (_tmp5) {
-			_tmp4 = TRUE;
-		} else {
-			_tmp4 = VALA_IS_PROPERTY (sym);
-		}
-		if (_tmp4) {
-			_tmp3 = TRUE;
-		} else {
-			_tmp3 = VALA_IS_CONSTANT (sym);
-		}
-		if (_tmp3) {
-			ValaSymbol* _tmp8;
-			ValaSymbol* _tmp7;
-			_tmp8 = NULL;
-			_tmp7 = NULL;
-			s = (_tmp8 = (_tmp7 = sym, (_tmp7 == NULL) ? NULL : vala_code_node_ref (_tmp7)), (s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL)), _tmp8);
-		} else {
-			if (VALA_IS_FORMAL_PARAMETER (sym)) {
-				ValaFormalParameter* _tmp9;
-				ValaFormalParameter* fp;
-				ValaSymbol* _tmp11;
-				ValaSymbol* _tmp10;
-				_tmp9 = NULL;
-				fp = (_tmp9 = VALA_FORMAL_PARAMETER (sym), (_tmp9 == NULL) ? NULL : vala_code_node_ref (_tmp9));
-				_tmp11 = NULL;
-				_tmp10 = NULL;
-				s = (_tmp11 = (_tmp10 = (ValaSymbol*) vala_data_type_get_data_type (vala_formal_parameter_get_parameter_type (fp)), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10)), (s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL)), _tmp11);
-				if (s == NULL) {
-					/* generic type parameter */
-					(fp == NULL) ? NULL : (fp = (vala_code_node_unref (fp), NULL));
-					(s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL));
-					return;
-				}
-				(fp == NULL) ? NULL : (fp = (vala_code_node_unref (fp), NULL));
-			} else {
-				(s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL));
-				return;
-			}
+			char* _tmp3;
+			char* _tmp2;
+			char* _tmp1;
+			_tmp3 = NULL;
+			_tmp2 = NULL;
+			_tmp1 = NULL;
+			self->priv->cinclude_filename = (_tmp3 = g_strdup_printf ("%s%s.h", _tmp1 = vala_source_file_get_subdir (self), _tmp2 = vala_source_file_get_basename (self)), self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL), _tmp3);
+			_tmp2 = (g_free (_tmp2), NULL);
+			_tmp1 = (g_free (_tmp1), NULL);
 		}
 	}
-	if (dep_type == VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE) {
-		gee_collection_add ((GeeCollection*) self->priv->source_symbol_dependencies, s);
-		if (vala_symbol_get_external_package (s)) {
-			{
-				GeeList* _tmp12;
-				GeeIterator* _tmp13;
-				GeeIterator* _fn_it;
-				_tmp12 = NULL;
-				_tmp13 = NULL;
-				_fn_it = (_tmp13 = gee_iterable_iterator ((GeeIterable*) (_tmp12 = vala_symbol_get_cheader_filenames (s))), (_tmp12 == NULL) ? NULL : (_tmp12 = (gee_collection_object_unref (_tmp12), NULL)), _tmp13);
-				while (gee_iterator_next (_fn_it)) {
-					char* fn;
-					fn = (char*) gee_iterator_get (_fn_it);
-					gee_collection_add ((GeeCollection*) self->priv->source_external_includes, fn);
-					fn = (g_free (fn), NULL);
-				}
-				(_fn_it == NULL) ? NULL : (_fn_it = (gee_collection_object_unref (_fn_it), NULL));
-			}
-		} else {
-			{
-				GeeList* _tmp14;
-				GeeIterator* _tmp15;
-				GeeIterator* _fn_it;
-				_tmp14 = NULL;
-				_tmp15 = NULL;
-				_fn_it = (_tmp15 = gee_iterable_iterator ((GeeIterable*) (_tmp14 = vala_symbol_get_cheader_filenames (s))), (_tmp14 == NULL) ? NULL : (_tmp14 = (gee_collection_object_unref (_tmp14), NULL)), _tmp15);
-				while (gee_iterator_next (_fn_it)) {
-					char* fn;
-					fn = (char*) gee_iterator_get (_fn_it);
-					gee_collection_add ((GeeCollection*) self->priv->source_internal_includes, fn);
-					fn = (g_free (fn), NULL);
-				}
-				(_fn_it == NULL) ? NULL : (_fn_it = (gee_collection_object_unref (_fn_it), NULL));
-			}
-		}
-		(s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL));
-		return;
-	}
-	if (vala_symbol_get_external_package (s)) {
-		/* external package */
-		{
-			GeeList* _tmp16;
-			GeeIterator* _tmp17;
-			GeeIterator* _fn_it;
-			/* external package */
-			_tmp16 = NULL;
-			_tmp17 = NULL;
-			_fn_it = (_tmp17 = gee_iterable_iterator ((GeeIterable*) (_tmp16 = vala_symbol_get_cheader_filenames (s))), (_tmp16 == NULL) ? NULL : (_tmp16 = (gee_collection_object_unref (_tmp16), NULL)), _tmp17);
-			/* external package */
-			while (gee_iterator_next (_fn_it)) {
-				char* fn;
-				/* external package */
-				fn = (char*) gee_iterator_get (_fn_it);
-				gee_collection_add ((GeeCollection*) self->priv->header_external_includes, fn);
-				fn = (g_free (fn), NULL);
-			}
-			(_fn_it == NULL) ? NULL : (_fn_it = (gee_collection_object_unref (_fn_it), NULL));
-		}
-		(s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL));
-		return;
-	}
-	if (dep_type == VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL) {
-		{
-			GeeList* _tmp18;
-			GeeIterator* _tmp19;
-			GeeIterator* _fn_it;
-			_tmp18 = NULL;
-			_tmp19 = NULL;
-			_fn_it = (_tmp19 = gee_iterable_iterator ((GeeIterable*) (_tmp18 = vala_symbol_get_cheader_filenames (s))), (_tmp18 == NULL) ? NULL : (_tmp18 = (gee_collection_object_unref (_tmp18), NULL)), _tmp19);
-			while (gee_iterator_next (_fn_it)) {
-				char* fn;
-				fn = (char*) gee_iterator_get (_fn_it);
-				gee_collection_add ((GeeCollection*) self->priv->header_internal_includes, fn);
-				fn = (g_free (fn), NULL);
-			}
-			(_fn_it == NULL) ? NULL : (_fn_it = (gee_collection_object_unref (_fn_it), NULL));
-		}
-		gee_collection_add ((GeeCollection*) self->priv->header_internal_full_dependencies, vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) s)));
-	}
-	gee_collection_add ((GeeCollection*) self->priv->header_internal_dependencies, vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) s)));
-	(s == NULL) ? NULL : (s = (vala_code_node_unref (s), NULL));
-}
-
-
-/**
- * Adds the symbols that define the specified type to the list of
- * symbols code in this source file depends on.
- *
- * TODO Move source and header file dependency analysis to
- * code generator.
- *
- * @param type     a data type
- * @param dep_type type of dependency
- */
-void vala_source_file_add_type_dependency (ValaSourceFile* self, ValaDataType* type, ValaSourceFileDependencyType dep_type) {
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (type != NULL);
-	{
-		GeeList* _tmp0;
-		GeeIterator* _tmp1;
-		GeeIterator* _type_symbol_it;
-		_tmp0 = NULL;
-		_tmp1 = NULL;
-		_type_symbol_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_data_type_get_symbols (type))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
-		while (gee_iterator_next (_type_symbol_it)) {
-			ValaSymbol* type_symbol;
-			type_symbol = (ValaSymbol*) gee_iterator_get (_type_symbol_it);
-			vala_source_file_add_symbol_dependency (self, type_symbol, dep_type);
-			(type_symbol == NULL) ? NULL : (type_symbol = (vala_code_node_unref (type_symbol), NULL));
-		}
-		(_type_symbol_it == NULL) ? NULL : (_type_symbol_it = (gee_collection_object_unref (_type_symbol_it), NULL));
-	}
-}
-
-
-/**
- * Returns the list of external includes the generated C header file
- * requires.
- *
- * @return external include list for C header file
- */
-GeeList* vala_source_file_get_header_external_includes (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->header_external_includes);
-}
-
-
-/**
- * Adds the specified filename to the list of package-internal includes
- * the generated C header file requires.
- *
- * @param include internal include for C header file
- */
-void vala_source_file_add_header_internal_include (ValaSourceFile* self, const char* include) {
-	char* _tmp0;
-	gboolean _tmp1;
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (include != NULL);
-	/* skip includes to self */
-	_tmp0 = NULL;
-	if ((_tmp1 = _vala_strcmp0 (include, _tmp0 = vala_source_file_get_cinclude_filename (self)) != 0, _tmp0 = (g_free (_tmp0), NULL), _tmp1)) {
-		gee_collection_add ((GeeCollection*) self->priv->header_internal_includes, include);
-	}
-}
-
-
-/**
- * Returns the list of package-internal includes the generated C header
- * file requires.
- *
- * @return internal include list for C header file
- */
-GeeList* vala_source_file_get_header_internal_includes (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->header_internal_includes);
-}
-
-
-/**
- * Returns the list of external includes the generated C source file
- * requires.
- *
- * @return include list for C source file
- */
-GeeList* vala_source_file_get_source_external_includes (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->source_external_includes);
-}
-
-
-/**
- * Returns the list of package-internal includes the generated C source
- * file requires.
- *
- * @return include list for C source file
- */
-GeeList* vala_source_file_get_source_internal_includes (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->source_internal_includes);
-}
-
-
-/**
- * Returns the list of source files the generated C header file requires
- * definitely.
- *
- * @return definite source file dependencies
- */
-GeeList* vala_source_file_get_header_internal_full_dependencies (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return (GeeList*) gee_read_only_list_new (VALA_TYPE_SOURCE_FILE, NULL, NULL, self->priv->header_internal_full_dependencies);
-}
-
-
-/**
- * Returns the list of source files the generated C header file loosely
- * depends on.
- *
- * @return loose source file dependencies
- */
-GeeList* vala_source_file_get_header_internal_dependencies (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return (GeeList*) gee_read_only_list_new (VALA_TYPE_SOURCE_FILE, NULL, NULL, self->priv->header_internal_dependencies);
-}
-
-
-GeeSet* vala_source_file_get_source_symbol_dependencies (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return (GeeSet*) gee_read_only_set_new (VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, self->priv->source_symbol_dependencies);
+	_tmp4 = NULL;
+	return (_tmp4 = self->priv->cinclude_filename, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4));
 }
 
 
@@ -876,6 +725,12 @@ gchar* vala_source_file_get_mapped_contents (ValaSourceFile* self) {
 }
 
 
+static glong string_get_length (const char* self) {
+	g_return_val_if_fail (self != NULL, 0L);
+	return g_utf8_strlen (self, -1);
+}
+
+
 gsize vala_source_file_get_mapped_length (ValaSourceFile* self) {
 	g_return_val_if_fail (self != NULL, 0UL);
 	if (vala_source_file_get_content (self) != NULL) {
@@ -947,46 +802,6 @@ void vala_source_file_set_external_package (ValaSourceFile* self, gboolean value
 }
 
 
-ValaSourceFileCycle* vala_source_file_get_cycle (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, NULL);
-	return self->priv->_cycle;
-}
-
-
-void vala_source_file_set_cycle (ValaSourceFile* self, ValaSourceFileCycle* value) {
-	ValaSourceFileCycle* _tmp2;
-	ValaSourceFileCycle* _tmp1;
-	g_return_if_fail (self != NULL);
-	_tmp2 = NULL;
-	_tmp1 = NULL;
-	self->priv->_cycle = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_source_file_cycle_ref (_tmp1)), (self->priv->_cycle == NULL) ? NULL : (self->priv->_cycle = (vala_source_file_cycle_unref (self->priv->_cycle), NULL)), _tmp2);
-}
-
-
-gboolean vala_source_file_get_is_cycle_head (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, FALSE);
-	return self->priv->_is_cycle_head;
-}
-
-
-void vala_source_file_set_is_cycle_head (ValaSourceFile* self, gboolean value) {
-	g_return_if_fail (self != NULL);
-	self->priv->_is_cycle_head = value;
-}
-
-
-gint vala_source_file_get_mark (ValaSourceFile* self) {
-	g_return_val_if_fail (self != NULL, 0);
-	return self->priv->_mark;
-}
-
-
-void vala_source_file_set_mark (ValaSourceFile* self, gint value) {
-	g_return_if_fail (self != NULL);
-	self->priv->_mark = value;
-}
-
-
 ValaCodeContext* vala_source_file_get_context (ValaSourceFile* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_context;
@@ -1125,13 +940,6 @@ static void vala_source_file_instance_init (ValaSourceFile * self) {
 	self->priv->cheader_filename = NULL;
 	self->priv->csource_filename = NULL;
 	self->priv->cinclude_filename = NULL;
-	self->priv->header_external_includes = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
-	self->priv->header_internal_includes = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
-	self->priv->source_external_includes = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
-	self->priv->source_internal_includes = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
-	self->priv->header_internal_full_dependencies = (GeeList*) gee_array_list_new (VALA_TYPE_SOURCE_FILE, NULL, NULL, g_direct_equal);
-	self->priv->header_internal_dependencies = (GeeList*) gee_array_list_new (VALA_TYPE_SOURCE_FILE, NULL, NULL, g_direct_equal);
-	self->priv->source_symbol_dependencies = (GeeSet*) gee_hash_set_new (VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_hash, g_direct_equal);
 	self->priv->source_array = NULL;
 	self->priv->mapped_file = NULL;
 	self->priv->_content = NULL;
@@ -1144,19 +952,11 @@ static void vala_source_file_finalize (ValaSourceFile* obj) {
 	self = VALA_SOURCE_FILE (obj);
 	self->priv->_filename = (g_free (self->priv->_filename), NULL);
 	self->priv->_comment = (g_free (self->priv->_comment), NULL);
-	(self->priv->_cycle == NULL) ? NULL : (self->priv->_cycle = (vala_source_file_cycle_unref (self->priv->_cycle), NULL));
 	(self->priv->using_directives == NULL) ? NULL : (self->priv->using_directives = (gee_collection_object_unref (self->priv->using_directives), NULL));
 	(self->priv->nodes == NULL) ? NULL : (self->priv->nodes = (gee_collection_object_unref (self->priv->nodes), NULL));
 	self->priv->cheader_filename = (g_free (self->priv->cheader_filename), NULL);
 	self->priv->csource_filename = (g_free (self->priv->csource_filename), NULL);
 	self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL);
-	(self->priv->header_external_includes == NULL) ? NULL : (self->priv->header_external_includes = (gee_collection_object_unref (self->priv->header_external_includes), NULL));
-	(self->priv->header_internal_includes == NULL) ? NULL : (self->priv->header_internal_includes = (gee_collection_object_unref (self->priv->header_internal_includes), NULL));
-	(self->priv->source_external_includes == NULL) ? NULL : (self->priv->source_external_includes = (gee_collection_object_unref (self->priv->source_external_includes), NULL));
-	(self->priv->source_internal_includes == NULL) ? NULL : (self->priv->source_internal_includes = (gee_collection_object_unref (self->priv->source_internal_includes), NULL));
-	(self->priv->header_internal_full_dependencies == NULL) ? NULL : (self->priv->header_internal_full_dependencies = (gee_collection_object_unref (self->priv->header_internal_full_dependencies), NULL));
-	(self->priv->header_internal_dependencies == NULL) ? NULL : (self->priv->header_internal_dependencies = (gee_collection_object_unref (self->priv->header_internal_dependencies), NULL));
-	(self->priv->source_symbol_dependencies == NULL) ? NULL : (self->priv->source_symbol_dependencies = (gee_collection_object_unref (self->priv->source_symbol_dependencies), NULL));
 	(self->priv->source_array == NULL) ? NULL : (self->priv->source_array = (gee_collection_object_unref (self->priv->source_array), NULL));
 	(self->priv->mapped_file == NULL) ? NULL : (self->priv->mapped_file = (g_mapped_file_free (self->priv->mapped_file), NULL));
 	self->priv->_content = (g_free (self->priv->_content), NULL);
@@ -1193,17 +993,6 @@ void vala_source_file_unref (gpointer instance) {
 }
 
 
-
-GType vala_source_file_dependency_type_get_type (void) {
-	static GType vala_source_file_dependency_type_type_id = 0;
-	if (G_UNLIKELY (vala_source_file_dependency_type_type_id == 0)) {
-		static const GEnumValue values[] = {{VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL, "VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL", "header-full"}, {VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW, "VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW", "header-shallow"}, {VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE, "VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE", "source"}, {0, NULL, NULL}};
-		vala_source_file_dependency_type_type_id = g_enum_register_static ("ValaSourceFileDependencyType", values);
-	}
-	return vala_source_file_dependency_type_type_id;
-}
-
-
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
 	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
