@@ -20,12 +20,48 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vala/valasourcereference.h>
-#include <vala/valasourcefile.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
 #include <gobject/gvaluecollector.h>
 
 
+#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
+#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
+#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
+#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
+#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
 
+typedef struct _ValaSourceReference ValaSourceReference;
+typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
+typedef struct _ValaSourceReferencePrivate ValaSourceReferencePrivate;
+
+#define VALA_TYPE_SOURCE_FILE (vala_source_file_get_type ())
+#define VALA_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFile))
+#define VALA_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
+#define VALA_IS_SOURCE_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_FILE))
+#define VALA_IS_SOURCE_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_FILE))
+#define VALA_SOURCE_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_FILE, ValaSourceFileClass))
+
+typedef struct _ValaSourceFile ValaSourceFile;
+typedef struct _ValaSourceFileClass ValaSourceFileClass;
+typedef struct _ValaParamSpecSourceReference ValaParamSpecSourceReference;
+
+/**
+ * Represents a reference to a location in a source file.
+ */
+struct _ValaSourceReference {
+	GTypeInstance parent_instance;
+	volatile int ref_count;
+	ValaSourceReferencePrivate * priv;
+};
+
+struct _ValaSourceReferenceClass {
+	GTypeClass parent_class;
+	void (*finalize) (ValaSourceReference *self);
+};
 
 struct _ValaSourceReferencePrivate {
 	ValaSourceFile* _file;
@@ -36,10 +72,48 @@ struct _ValaSourceReferencePrivate {
 	char* _comment;
 };
 
+struct _ValaParamSpecSourceReference {
+	GParamSpec parent_instance;
+};
+
+
+
+gpointer vala_source_reference_ref (gpointer instance);
+void vala_source_reference_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_reference (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_reference (const GValue* value);
+GType vala_source_reference_get_type (void);
+gpointer vala_source_file_ref (gpointer instance);
+void vala_source_file_unref (gpointer instance);
+GParamSpec* vala_param_spec_source_file (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
+void vala_value_set_source_file (GValue* value, gpointer v_object);
+gpointer vala_value_get_source_file (const GValue* value);
+GType vala_source_file_get_type (void);
 #define VALA_SOURCE_REFERENCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferencePrivate))
 enum  {
 	VALA_SOURCE_REFERENCE_DUMMY_PROPERTY
 };
+void vala_source_reference_set_file (ValaSourceReference* self, ValaSourceFile* value);
+void vala_source_reference_set_first_line (ValaSourceReference* self, gint value);
+void vala_source_reference_set_first_column (ValaSourceReference* self, gint value);
+void vala_source_reference_set_last_line (ValaSourceReference* self, gint value);
+void vala_source_reference_set_last_column (ValaSourceReference* self, gint value);
+ValaSourceReference* vala_source_reference_new (ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column);
+ValaSourceReference* vala_source_reference_construct (GType object_type, ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column);
+ValaSourceReference* vala_source_reference_new (ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column);
+void vala_source_reference_set_comment (ValaSourceReference* self, const char* value);
+ValaSourceReference* vala_source_reference_new_with_comment (ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column, const char* _comment);
+ValaSourceReference* vala_source_reference_construct_with_comment (GType object_type, ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column, const char* _comment);
+ValaSourceReference* vala_source_reference_new_with_comment (ValaSourceFile* _file, gint _first_line, gint _first_column, gint _last_line, gint _last_column, const char* _comment);
+ValaSourceFile* vala_source_reference_get_file (ValaSourceReference* self);
+char* vala_source_file_get_relative_filename (ValaSourceFile* self);
+gint vala_source_reference_get_first_line (ValaSourceReference* self);
+gint vala_source_reference_get_first_column (ValaSourceReference* self);
+gint vala_source_reference_get_last_line (ValaSourceReference* self);
+gint vala_source_reference_get_last_column (ValaSourceReference* self);
+char* vala_source_reference_to_string (ValaSourceReference* self);
+const char* vala_source_reference_get_comment (ValaSourceReference* self);
 static gpointer vala_source_reference_parent_class = NULL;
 static void vala_source_reference_finalize (ValaSourceReference* obj);
 
