@@ -20,54 +20,58 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vapigen/valagirparser.h>
-#include <vala/valanamespace.h>
-#include <vala/valasourcelocation.h>
-#include <gee/hashmap.h>
-#include <vala/valasymbol.h>
-#include <vala/valascope.h>
-#include <vala/valareport.h>
-#include <vala/valasourcereference.h>
-#include <vala/valacodenode.h>
-#include <vala/valastruct.h>
-#include <vala/valadatatype.h>
-#include <vala/valavoidtype.h>
-#include <vala/valapointertype.h>
-#include <vala/valaarraytype.h>
-#include <vala/valaunresolvedtype.h>
-#include <vala/valaunresolvedsymbol.h>
-#include <vala/valaenum.h>
-#include <vala/valaenumvalue.h>
-#include <vala/valamethod.h>
-#include <vala/valamember.h>
-#include <gee/arraylist.h>
-#include <vala/valaformalparameter.h>
-#include <gee/collection.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <vala/valaerrortype.h>
-#include <vala/valaerrordomain.h>
-#include <vala/valaerrorcode.h>
-#include <vala/valadelegate.h>
-#include <vala/valafield.h>
-#include <vala/valaexpression.h>
-#include <vala/valacreationmethod.h>
-#include <vala/valaclass.h>
-#include <vala/valasignal.h>
-#include <vala/valaproperty.h>
-#include <vala/valapropertyaccessor.h>
-#include <vala/valablock.h>
-#include <vala/valainterface.h>
-#include <vala/valaconstant.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <vala.h>
+#include <stdlib.h>
+#include <string.h>
+#include <gee.h>
 #include <glib/gstdio.h>
-#include <gee/map.h>
-#include "vapigen/valamarkupreader.h"
 
 
+#define VALA_TYPE_GIR_PARSER (vala_gir_parser_get_type ())
+#define VALA_GIR_PARSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_GIR_PARSER, ValaGirParser))
+#define VALA_GIR_PARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_GIR_PARSER, ValaGirParserClass))
+#define VALA_IS_GIR_PARSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_GIR_PARSER))
+#define VALA_IS_GIR_PARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_GIR_PARSER))
+#define VALA_GIR_PARSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_GIR_PARSER, ValaGirParserClass))
 
+typedef struct _ValaGirParser ValaGirParser;
+typedef struct _ValaGirParserClass ValaGirParserClass;
+typedef struct _ValaGirParserPrivate ValaGirParserPrivate;
+
+#define VALA_TYPE_MARKUP_READER (vala_markup_reader_get_type ())
+#define VALA_MARKUP_READER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_MARKUP_READER, ValaMarkupReader))
+#define VALA_MARKUP_READER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_MARKUP_READER, ValaMarkupReaderClass))
+#define VALA_IS_MARKUP_READER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_MARKUP_READER))
+#define VALA_IS_MARKUP_READER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_MARKUP_READER))
+#define VALA_MARKUP_READER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_MARKUP_READER, ValaMarkupReaderClass))
+
+typedef struct _ValaMarkupReader ValaMarkupReader;
+typedef struct _ValaMarkupReaderClass ValaMarkupReaderClass;
+
+#define VALA_TYPE_MARKUP_TOKEN_TYPE (vala_markup_token_type_get_type ())
+
+/**
+ * Code visitor parsing all Vala source files.
+ */
+struct _ValaGirParser {
+	ValaCodeVisitor parent_instance;
+	ValaGirParserPrivate * priv;
+};
+
+struct _ValaGirParserClass {
+	ValaCodeVisitorClass parent_class;
+};
+
+typedef enum  {
+	VALA_MARKUP_TOKEN_TYPE_NONE,
+	VALA_MARKUP_TOKEN_TYPE_START_ELEMENT,
+	VALA_MARKUP_TOKEN_TYPE_END_ELEMENT,
+	VALA_MARKUP_TOKEN_TYPE_TEXT,
+	VALA_MARKUP_TOKEN_TYPE_EOF
+} ValaMarkupTokenType;
 
-static glong string_get_length (const char* self);
-static char* string_substring (const char* self, glong offset, glong len);
 struct _ValaGirParserPrivate {
 	char* _package_name;
 	ValaMarkupReader* reader;
@@ -83,45 +87,62 @@ struct _ValaGirParserPrivate {
 	GeeHashMap* attributes_map;
 };
 
+
+
+GType vala_gir_parser_get_type (void);
+GType vala_markup_reader_get_type (void);
+GType vala_markup_token_type_get_type (void);
 #define VALA_GIR_PARSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_GIR_PARSER, ValaGirParserPrivate))
 enum  {
 	VALA_GIR_PARSER_DUMMY_PROPERTY
 };
+void vala_gir_parser_parse (ValaGirParser* self, ValaCodeContext* context);
+void vala_gir_parser_parse_file (ValaGirParser* self, ValaSourceFile* source_file);
 static void vala_gir_parser_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
+ValaMarkupReader* vala_markup_reader_new (const char* filename);
+ValaMarkupReader* vala_markup_reader_construct (GType object_type, const char* filename);
 static void vala_gir_parser_next (ValaGirParser* self);
+static void vala_gir_parser_parse_repository (ValaGirParser* self);
+ValaMarkupTokenType vala_markup_reader_read_token (ValaMarkupReader* self, ValaSourceLocation* token_begin, ValaSourceLocation* token_end);
+const char* vala_markup_reader_get_name (ValaMarkupReader* self);
+static ValaSourceReference* vala_gir_parser_get_current_src (ValaGirParser* self);
 static void vala_gir_parser_start_element (ValaGirParser* self, const char* name);
 static void vala_gir_parser_end_element (ValaGirParser* self, const char* name);
-static ValaSourceReference* vala_gir_parser_get_current_src (ValaGirParser* self);
-static void vala_gir_parser_parse_repository (ValaGirParser* self);
+static ValaNamespace* vala_gir_parser_parse_namespace (ValaGirParser* self);
 static void vala_gir_parser_parse_include (ValaGirParser* self);
 static void vala_gir_parser_parse_package (ValaGirParser* self);
-static void _vala_array_add1 (char*** array, int* length, int* size, char* value);
 static void vala_gir_parser_parse_c_include (ValaGirParser* self);
+char* vala_markup_reader_get_attribute (ValaMarkupReader* self, const char* attr);
+static void vala_gir_parser_set_package_name (ValaGirParser* self, const char* value);
+static void _vala_array_add1 (char*** array, int* length, int* size, char* value);
+static char* vala_gir_parser_transform_namespace_name (ValaGirParser* self, const char* gir_module_name);
 static void _g_list_free_vala_code_node_unref (GList* self);
-static ValaNamespace* vala_gir_parser_parse_namespace (ValaGirParser* self);
 static ValaStruct* vala_gir_parser_parse_alias (ValaGirParser* self);
 static ValaEnum* vala_gir_parser_parse_enumeration (ValaGirParser* self);
 static ValaEnum* vala_gir_parser_parse_bitfield (ValaGirParser* self);
-static ValaEnumValue* vala_gir_parser_parse_enumeration_member (ValaGirParser* self);
-static ValaDataType* vala_gir_parser_parse_return_value (ValaGirParser* self);
-static ValaFormalParameter* vala_gir_parser_parse_parameter (ValaGirParser* self, gint* array_length_idx, gint* closure_idx, gint* destroy_idx);
-static ValaDataType* vala_gir_parser_parse_type (ValaGirParser* self, gint* array_length_index);
-static ValaDataType* vala_gir_parser_parse_type_from_name (ValaGirParser* self, const char* type_name);
-static char* vala_gir_parser_transform_namespace_name (ValaGirParser* self, const char* gir_module_name);
+static ValaMethod* vala_gir_parser_parse_method (ValaGirParser* self, const char* element_name);
+static ValaDelegate* vala_gir_parser_parse_callback (ValaGirParser* self);
 static ValaStruct* vala_gir_parser_parse_record (ValaGirParser* self);
 static ValaClass* vala_gir_parser_parse_class (ValaGirParser* self);
 static ValaInterface* vala_gir_parser_parse_interface (ValaGirParser* self);
-static ValaField* vala_gir_parser_parse_field (ValaGirParser* self);
-static ValaProperty* vala_gir_parser_parse_property (ValaGirParser* self);
-static ValaDelegate* vala_gir_parser_parse_callback (ValaGirParser* self);
-static ValaMethod* vala_gir_parser_parse_constructor (ValaGirParser* self);
-static ValaMethod* vala_gir_parser_parse_method (ValaGirParser* self, const char* element_name);
-static ValaSignal* vala_gir_parser_parse_signal (ValaGirParser* self);
 static ValaStruct* vala_gir_parser_parse_boxed (ValaGirParser* self);
 static ValaStruct* vala_gir_parser_parse_union (ValaGirParser* self);
 static ValaConstant* vala_gir_parser_parse_constant (ValaGirParser* self);
+static ValaDataType* vala_gir_parser_parse_type_from_name (ValaGirParser* self, const char* type_name);
+static ValaEnumValue* vala_gir_parser_parse_enumeration_member (ValaGirParser* self);
+static ValaDataType* vala_gir_parser_parse_type (ValaGirParser* self, gint* array_length_index);
+static ValaDataType* vala_gir_parser_parse_return_value (ValaGirParser* self);
+static ValaFormalParameter* vala_gir_parser_parse_parameter (ValaGirParser* self, gint* array_length_idx, gint* closure_idx, gint* destroy_idx);
+static ValaField* vala_gir_parser_parse_field (ValaGirParser* self);
+static ValaMethod* vala_gir_parser_parse_constructor (ValaGirParser* self);
+static ValaProperty* vala_gir_parser_parse_property (ValaGirParser* self);
+static ValaSignal* vala_gir_parser_parse_signal (ValaGirParser* self);
+void vala_gir_parser_parse_metadata (ValaGirParser* self, const char* metadata_filename);
 static char* vala_gir_parser_get_attribute (ValaGirParser* self, const char* node, const char* key);
-static void vala_gir_parser_set_package_name (ValaGirParser* self, const char* value);
+ValaGirParser* vala_gir_parser_new (void);
+ValaGirParser* vala_gir_parser_construct (GType object_type);
+ValaGirParser* vala_gir_parser_new (void);
+const char* vala_gir_parser_get_package_name (ValaGirParser* self);
 static gpointer vala_gir_parser_parent_class = NULL;
 static void vala_gir_parser_finalize (ValaCodeVisitor* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
@@ -130,32 +151,6 @@ static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
 
-static glong string_get_length (const char* self) {
-	g_return_val_if_fail (self != NULL, 0L);
-	return g_utf8_strlen (self, -1);
-}
-
-
-static char* string_substring (const char* self, glong offset, glong len) {
-	glong string_length;
-	const char* start;
-	g_return_val_if_fail (self != NULL, NULL);
-	string_length = g_utf8_strlen (self, -1);
-	if (offset < 0) {
-		offset = string_length + offset;
-		g_return_val_if_fail (offset >= 0, NULL);
-	} else {
-		g_return_val_if_fail (offset <= string_length, NULL);
-	}
-	if (len < 0) {
-		len = string_length - offset;
-	}
-	g_return_val_if_fail ((offset + len) <= string_length, NULL);
-	start = g_utf8_offset_to_pointer (self, offset);
-	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
-}
-
-
 /**
  * Parses all .gir source files in the specified code
  * context and builds a code tree.
@@ -2084,6 +2079,32 @@ static ValaConstant* vala_gir_parser_parse_constant (ValaGirParser* self) {
 }
 
 
+static char* string_substring (const char* self, glong offset, glong len) {
+	glong string_length;
+	const char* start;
+	g_return_val_if_fail (self != NULL, NULL);
+	string_length = g_utf8_strlen (self, -1);
+	if (offset < 0) {
+		offset = string_length + offset;
+		g_return_val_if_fail (offset >= 0, NULL);
+	} else {
+		g_return_val_if_fail (offset <= string_length, NULL);
+	}
+	if (len < 0) {
+		len = string_length - offset;
+	}
+	g_return_val_if_fail ((offset + len) <= string_length, NULL);
+	start = g_utf8_offset_to_pointer (self, offset);
+	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
+}
+
+
+static glong string_get_length (const char* self) {
+	g_return_val_if_fail (self != NULL, 0L);
+	return g_utf8_strlen (self, -1);
+}
+
+
 void vala_gir_parser_parse_metadata (ValaGirParser* self, const char* metadata_filename) {
 	GError * inner_error;
 	g_return_if_fail (self != NULL);
