@@ -20,14 +20,38 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <vapigen/valamarkupreader.h>
-#include <gee/hashmap.h>
-#include <gee/map.h>
-#include <vala/valareport.h>
-#include <vala/valasourcereference.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <stdlib.h>
+#include <string.h>
+#include <gee.h>
+#include <vala.h>
 
 
+#define VALA_TYPE_MARKUP_READER (vala_markup_reader_get_type ())
+#define VALA_MARKUP_READER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_MARKUP_READER, ValaMarkupReader))
+#define VALA_MARKUP_READER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_MARKUP_READER, ValaMarkupReaderClass))
+#define VALA_IS_MARKUP_READER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_MARKUP_READER))
+#define VALA_IS_MARKUP_READER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_MARKUP_READER))
+#define VALA_MARKUP_READER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_MARKUP_READER, ValaMarkupReaderClass))
 
+typedef struct _ValaMarkupReader ValaMarkupReader;
+typedef struct _ValaMarkupReaderClass ValaMarkupReaderClass;
+typedef struct _ValaMarkupReaderPrivate ValaMarkupReaderPrivate;
+
+#define VALA_TYPE_MARKUP_TOKEN_TYPE (vala_markup_token_type_get_type ())
+
+/**
+ * Simple reader for a subset of XML.
+ */
+struct _ValaMarkupReader {
+	GObject parent_instance;
+	ValaMarkupReaderPrivate * priv;
+};
+
+struct _ValaMarkupReaderClass {
+	GObjectClass parent_class;
+};
 
 struct _ValaMarkupReaderPrivate {
 	char* _filename;
@@ -42,19 +66,39 @@ struct _ValaMarkupReaderPrivate {
 	gboolean empty_element;
 };
 
+typedef enum  {
+	VALA_MARKUP_TOKEN_TYPE_NONE,
+	VALA_MARKUP_TOKEN_TYPE_START_ELEMENT,
+	VALA_MARKUP_TOKEN_TYPE_END_ELEMENT,
+	VALA_MARKUP_TOKEN_TYPE_TEXT,
+	VALA_MARKUP_TOKEN_TYPE_EOF
+} ValaMarkupTokenType;
+
+
+
+GType vala_markup_reader_get_type (void);
 #define VALA_MARKUP_READER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_MARKUP_READER, ValaMarkupReaderPrivate))
 enum  {
 	VALA_MARKUP_READER_DUMMY_PROPERTY,
 	VALA_MARKUP_READER_FILENAME,
 	VALA_MARKUP_READER_NAME
 };
-static char* vala_markup_reader_read_name (ValaMarkupReader* self);
-static void vala_markup_reader_space (ValaMarkupReader* self);
 static void vala_markup_reader_set_filename (ValaMarkupReader* self, const char* value);
+ValaMarkupReader* vala_markup_reader_new (const char* filename);
+ValaMarkupReader* vala_markup_reader_construct (GType object_type, const char* filename);
+ValaMarkupReader* vala_markup_reader_new (const char* filename);
+char* vala_markup_reader_get_attribute (ValaMarkupReader* self, const char* attr);
+static char* vala_markup_reader_read_name (ValaMarkupReader* self);
+GType vala_markup_token_type_get_type (void);
+static void vala_markup_reader_space (ValaMarkupReader* self);
+ValaMarkupTokenType vala_markup_reader_read_token (ValaMarkupReader* self, ValaSourceLocation* token_begin, ValaSourceLocation* token_end);
 static void vala_markup_reader_set_name (ValaMarkupReader* self, const char* value);
+const char* vala_markup_reader_get_filename (ValaMarkupReader* self);
+const char* vala_markup_reader_get_name (ValaMarkupReader* self);
 static GObject * vala_markup_reader_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
 static gpointer vala_markup_reader_parent_class = NULL;
 static void vala_markup_reader_finalize (GObject* obj);
+const char* vala_markup_token_type_to_string (ValaMarkupTokenType self);
 
 
 
