@@ -20,24 +20,39 @@
  * 	Mathias Hasselmann <mathias.hasselmann@gmx.de>
  */
 
-#include <vapigen/valavapicheck.h>
-#include <gee/list.h>
-#include <gee/set.h>
-#include <gee/arraylist.h>
-#include <gee/hashset.h>
+#include <glib.h>
+#include <glib-object.h>
+#include <vala.h>
+#include <gee.h>
+#include <stdlib.h>
+#include <string.h>
 #include <gidlparser.h>
 #include <gidlmodule.h>
-#include <gee/collection.h>
 #include <gidlnode.h>
 #include <stdio.h>
-#include <vala/valasourcereference.h>
-#include <vala/valareport.h>
 #include <glib/gstdio.h>
 
 
+#define VALA_TYPE_VAPI_CHECK (vala_vapi_check_get_type ())
+#define VALA_VAPI_CHECK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_VAPI_CHECK, ValaVAPICheck))
+#define VALA_VAPI_CHECK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_VAPI_CHECK, ValaVAPICheckClass))
+#define VALA_IS_VAPI_CHECK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_VAPI_CHECK))
+#define VALA_IS_VAPI_CHECK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_VAPI_CHECK))
+#define VALA_VAPI_CHECK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_VAPI_CHECK, ValaVAPICheckClass))
 
+typedef struct _ValaVAPICheck ValaVAPICheck;
+typedef struct _ValaVAPICheckClass ValaVAPICheckClass;
+typedef struct _ValaVAPICheckPrivate ValaVAPICheckPrivate;
+
+struct _ValaVAPICheck {
+	GObject parent_instance;
+	ValaVAPICheckPrivate * priv;
+};
+
+struct _ValaVAPICheckClass {
+	GObjectClass parent_class;
+};
 
-static char* string_substring (const char* self, glong offset, glong len);
 struct _ValaVAPICheckPrivate {
 	ValaCodeContext* _context;
 	ValaSourceFile* _gidl;
@@ -46,6 +61,9 @@ struct _ValaVAPICheckPrivate {
 	GeeSet* _symbols;
 };
 
+
+
+GType vala_vapi_check_get_type (void);
 #define VALA_VAPI_CHECK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_VAPI_CHECK, ValaVAPICheckPrivate))
 enum  {
 	VALA_VAPI_CHECK_DUMMY_PROPERTY,
@@ -53,18 +71,25 @@ enum  {
 	VALA_VAPI_CHECK_GIDL,
 	VALA_VAPI_CHECK_METADATA
 };
-static void _g_list_free_g_idl_module_free (GList* self);
-static void vala_vapi_check_parse_gidl (ValaVAPICheck* self);
-static void vala_vapi_check_add_symbol (ValaVAPICheck* self, const char* name, const char* separator);
-static char* vala_vapi_check_get_scope (ValaVAPICheck* self);
-static void vala_vapi_check_enter_scope (ValaVAPICheck* self, const char* name);
-static void vala_vapi_check_leave_scope (ValaVAPICheck* self);
-static void vala_vapi_check_parse_members (ValaVAPICheck* self, const char* name, GList* members);
-static gint vala_vapi_check_check_metadata (ValaVAPICheck* self);
-static gint vala_vapi_check_main (char** args, int args_length1);
-static void vala_vapi_check_set_context (ValaVAPICheck* self, ValaCodeContext* value);
 static void vala_vapi_check_set_gidl (ValaVAPICheck* self, ValaSourceFile* value);
 static void vala_vapi_check_set_metadata (ValaVAPICheck* self, ValaSourceFile* value);
+static void vala_vapi_check_set_context (ValaVAPICheck* self, ValaCodeContext* value);
+ValaVAPICheck* vala_vapi_check_new (const char* gidlname, ValaCodeContext* context);
+ValaVAPICheck* vala_vapi_check_construct (GType object_type, const char* gidlname, ValaCodeContext* context);
+ValaVAPICheck* vala_vapi_check_new (const char* gidlname, ValaCodeContext* context);
+ValaSourceFile* vala_vapi_check_get_gidl (ValaVAPICheck* self);
+static void vala_vapi_check_parse_members (ValaVAPICheck* self, const char* name, GList* members);
+static void _g_list_free_g_idl_module_free (GList* self);
+static void vala_vapi_check_parse_gidl (ValaVAPICheck* self);
+static char* vala_vapi_check_get_scope (ValaVAPICheck* self);
+static void vala_vapi_check_add_symbol (ValaVAPICheck* self, const char* name, const char* separator);
+static void vala_vapi_check_enter_scope (ValaVAPICheck* self, const char* name);
+static void vala_vapi_check_leave_scope (ValaVAPICheck* self);
+ValaSourceFile* vala_vapi_check_get_metadata (ValaVAPICheck* self);
+static gint vala_vapi_check_check_metadata (ValaVAPICheck* self);
+gint vala_vapi_check_run (ValaVAPICheck* self);
+static gint vala_vapi_check_main (char** args, int args_length1);
+ValaCodeContext* vala_vapi_check_get_context (ValaVAPICheck* self);
 static gpointer vala_vapi_check_parent_class = NULL;
 static void vala_vapi_check_finalize (GObject* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
@@ -116,17 +141,11 @@ ValaVAPICheck* vala_vapi_check_construct (GType object_type, const char* gidlnam
 	_tmp4 = NULL;
 	_tmp3 = NULL;
 	_tmp2 = NULL;
-	__params_it->name = "metadata";
-	g_value_init (&__params_it->value, VALA_TYPE_SOURCE_FILE);
-	vala_value_set_source_file (&__params_it->value, _tmp4 = vala_source_file_new (context, _tmp3 = g_strconcat (_tmp2 = string_substring (gidlname, (glong) 0, g_utf8_strlen (gidlname, -1) - 5), ".metadata", NULL), FALSE, NULL));
-	__params_it++;
+	vala_vapi_check_set_metadata (self, _tmp4 = vala_source_file_new (context, _tmp3 = g_strconcat (_tmp2 = string_substring (gidlname, (glong) 0, g_utf8_strlen (gidlname, -1) - 5), ".metadata", NULL), FALSE, NULL));
 	(_tmp4 == NULL) ? NULL : (_tmp4 = (vala_source_file_unref (_tmp4), NULL));
 	_tmp3 = (g_free (_tmp3), NULL);
 	_tmp2 = (g_free (_tmp2), NULL);
-	__params_it->name = "context";
-	g_value_init (&__params_it->value, VALA_TYPE_CODE_CONTEXT);
-	vala_value_set_code_context (&__params_it->value, context);
-	__params_it++;
+	vala_vapi_check_set_context (self, context);
 	self = g_object_newv (object_type, __params_it - __params, __params);
 	while (__params_it > __params) {
 		--__params_it;
